﻿#region "Copyright"
/******************************************************************************************
 Copyright (C) 2012 Kim O'Sullivan (kim@makina.com.au)

 Permission is hereby granted, free of charge, to any person obtaining a copy of 
 this software and associated documentation files (the "Software"), to deal in the 
 Software without restriction, including without limitation the rights to use, copy, 
 modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, 
 and to permit persons to whom the Software is furnished to do so, subject to the 
 following conditions:
 
 The above copyright notice and this permission notice shall be included in all copies 
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE 
 LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
 OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
 OTHER DEALINGS IN THE SOFTWARE.
******************************************************************************************/
#endregion

#region "Namespace definitions"
using System;
using System.Collections.Generic;
using System.Text;
using CardFramework.Helpers;
#endregion

#region "Class definitions"
namespace CardFramework.Applications.Fips201
{
  public enum FASCNPoa
  {
    /// <summary>
    /// Undefined or Erroneous value
    /// </summary>
    Undefined = 0,

    Employee = 1,
    Civil = 2,
    ExecutiveStaff = 3,
    UniformedService = 4,
    Contractor = 5,
    OrganizationalAffiliate = 6,
    OrganizationalBeneficiary = 7
  }

  public class FASCN
  {
    #region "Members - Public"

    public FASCN()
    {
      BuildDatabase();
    }

    /// <summary>
    /// Agency Code
    /// </summary>
    public string AgencyCode { get; set; }

    private Dictionary<string, string> myAgencyCodes;

    /// <summary>
    /// Agency Code String
    /// </summary>
    public string GetAgencyCodeString()
    {
      if (myAgencyCodes.ContainsKey(AgencyCode))
      {
        return myAgencyCodes[AgencyCode];
      }
      else
      {
        return "Unknown Agency";
      }
    }

    /// <summary>
    /// Credential Number
    /// </summary>
    public string CredentialNumber { get; set; }

    /// <summary>
    /// System Code
    /// </summary>
    public string SystemCode { get; set; }

    /// <summary>
    /// Credential Series
    /// </summary>
    public string CS { get; set; }
    
    /// <summary>
    /// Individual Credential Issue
    /// </summary>
    public string ICI { get; set; }
    
    /// <summary>
    /// Person Identifier
    /// </summary>
    public string PI { get; set; }
    
    /// <summary>
    /// Organizational Category
    /// </summary>
    public string OC { get; set; }
    
    /// <summary>
    /// Organizational Identifier
    /// </summary>
    public string OI { get; set; }
    
    /// <summary>
    /// Person or Organization Association
    /// </summary>
    public string POA { get; set; }

    /// <summary>
    /// FASC-N parsing according to TIG_SCEPACS_v2.2 6.1
    /// </summary>
    /// <param name="fascn"></param>
    /// <returns></returns>
    public static FASCN Parse(byte[] fascn)
    {
      FASCN result = new FASCN();

      // Test values according to the document
      //byte[] test = new byte[] { 0xD0, 0x43, 0x94, 0x58, 0x21, 0x0C, 0x2C, 0x19, 0xA0, 0x84, 0x6D, 0x83, 0x68, 0x5A, 0x10, 0x82, 0x10, 0x8C, 0xE7, 0x39, 0x84, 0x10, 0x8C, 0xA3, 0xFC };

      if (fascn.Length != 25) throw new ArgumentException("fascn");

      BinaryParser parser = new BinaryParser(fascn, ByteEndianess.BigEndian, BitEndianess.Lsb);

      // SS
      if (parser.ReadUIntN(4) != FASCN_SS) throw new ArgumentException("fascn: Invalid SS"); ;
      parser.BitPosition++;   // Skip the parity

      // Agency Code
      result.AgencyCode = "";
      result.AgencyCode += parser.ReadUIntN(4); parser.BitPosition++;
      result.AgencyCode += parser.ReadUIntN(4); parser.BitPosition++;
      result.AgencyCode += parser.ReadUIntN(4); parser.BitPosition++;
      result.AgencyCode += parser.ReadUIntN(4); parser.BitPosition++;

      // FS
      if (parser.ReadUIntN(4) != FASCN_FS) throw new ArgumentException("fascn: Invalid FS"); ;
      parser.BitPosition++;

      // System Code
      result.SystemCode = "";
      result.SystemCode += parser.ReadUIntN(4); parser.BitPosition++;
      result.SystemCode += parser.ReadUIntN(4); parser.BitPosition++;
      result.SystemCode += parser.ReadUIntN(4); parser.BitPosition++;
      result.SystemCode += parser.ReadUIntN(4); parser.BitPosition++;

      // FS
      if (parser.ReadUIntN(4) != FASCN_FS) throw new ArgumentException("fascn: Invalid FS"); ;
      parser.BitPosition++;
      
      // Credential Number
      result.CredentialNumber = "";
      result.CredentialNumber += parser.ReadUIntN(4); parser.BitPosition++;
      result.CredentialNumber += parser.ReadUIntN(4); parser.BitPosition++;
      result.CredentialNumber += parser.ReadUIntN(4); parser.BitPosition++;
      result.CredentialNumber += parser.ReadUIntN(4); parser.BitPosition++;
      result.CredentialNumber += parser.ReadUIntN(4); parser.BitPosition++;
      result.CredentialNumber += parser.ReadUIntN(4); parser.BitPosition++;

      // FS
      if (parser.ReadUIntN(4) != FASCN_FS) throw new ArgumentException("fascn: Invalid FS"); ;
      parser.BitPosition++;

      // CS (Credential Series)
      result.CS = parser.ReadUIntN(4).ToString();
      parser.BitPosition++;

      // FS
      if (parser.ReadUIntN(4) != FASCN_FS) throw new ArgumentException("fascn: Invalid FS"); ;
      parser.BitPosition++;

      // ICI (Individual Credential Issue)
      result.ICI = parser.ReadUIntN(4).ToString();
      parser.BitPosition++;

      // FS
      if (parser.ReadUIntN(4) != FASCN_FS) throw new ArgumentException("fascn: Invalid FS"); ;
      parser.BitPosition++;

      // PI (Person Identifier)
      result.PI = "";
      result.PI += parser.ReadUIntN(4); parser.BitPosition++;
      result.PI += parser.ReadUIntN(4); parser.BitPosition++;
      result.PI += parser.ReadUIntN(4); parser.BitPosition++;
      result.PI += parser.ReadUIntN(4); parser.BitPosition++;
      result.PI += parser.ReadUIntN(4); parser.BitPosition++;
      result.PI += parser.ReadUIntN(4); parser.BitPosition++;
      result.PI += parser.ReadUIntN(4); parser.BitPosition++;
      result.PI += parser.ReadUIntN(4); parser.BitPosition++;
      result.PI += parser.ReadUIntN(4); parser.BitPosition++;
      result.PI += parser.ReadUIntN(4); parser.BitPosition++;

      // OC (Organizational Category)
      result.OC = parser.ReadUIntN(4).ToString();
      parser.BitPosition++;

      // OI (Organizational Identifier)
      result.OI = "";
      result.OI += parser.ReadUIntN(4); parser.BitPosition++;
      result.OI += parser.ReadUIntN(4); parser.BitPosition++;
      result.OI += parser.ReadUIntN(4); parser.BitPosition++;
      result.OI += parser.ReadUIntN(4); parser.BitPosition++;

      // POA (Person/Organization Association)
      result.POA = parser.ReadUIntN(4).ToString();
      parser.BitPosition++;

      // ES
      if (parser.ReadUIntN(4) != FASCN_ES) throw new ArgumentException("fascn: Invalid ES"); ;
      parser.BitPosition++;

      // Ignore the LRC because it's an obsolete relic of the old mag-stripe days.
      // FIXME: Include this once BinaryParser is fixed.
      // parser.ReadUIntN(5);

      return result;
    }

    #endregion

    #region "Members - Private / Protected"

    private void BuildDatabase()
    {

      myAgencyCodes = new Dictionary<string, string>();

      myAgencyCodes.Add(@"0000", @"THE LEGISLATIVE BRANCH");
      myAgencyCodes.Add(@"0001", @"Congress, generally, no additional specification available");
      myAgencyCodes.Add(@"0010", @"Senate, The");
      myAgencyCodes.Add(@"0050", @"House of Representatives, The");
      myAgencyCodes.Add(@"0099", @"Joint House and Senate Entities");
      myAgencyCodes.Add(@"0100", @"Architect of the Capitol");
      myAgencyCodes.Add(@"0300", @"Library of Congress");
      myAgencyCodes.Add(@"0361", @"Library of Congress Trust Fund Board");
      myAgencyCodes.Add(@"0363", @"Federal Library & Information Center Committee");
      myAgencyCodes.Add(@"0400", @"Government Printing Office");
      myAgencyCodes.Add(@"0500", @"General Accounting Office");
      myAgencyCodes.Add(@"0501", @"Comptroller General of the United States");
      myAgencyCodes.Add(@"0559", @"GAO, except Comptroller General");
      myAgencyCodes.Add(@"0800", @"Congressional Budget Office");
      myAgencyCodes.Add(@"0901", @"Commission on Security and Cooperation in Europe");
      myAgencyCodes.Add(@"0902", @"Botanic Garden");
      myAgencyCodes.Add(@"0904", @"Office of Compliance");
      myAgencyCodes.Add(@"0905", @"Dwight D. Eisenhower Memorial Commission");
      myAgencyCodes.Add(@"0914", @"John C. Stennis Center for Public Service Training and Development");
      myAgencyCodes.Add(@"0915", @"Advisory Commission on Electronic Commerce");
      myAgencyCodes.Add(@"0916", @"National Commission on Terrorism");
      myAgencyCodes.Add(@"0923", @"Commission on the People's Republic of China");
      myAgencyCodes.Add(@"0929", @"U.S. Capitol Preservation Commission");
      myAgencyCodes.Add(@"0938", @"Abraham Lincoln Bicentennial Commission");
      myAgencyCodes.Add(@"0962", @"Permanent Committee for the Oliver Wendell Holmes Devise");
      myAgencyCodes.Add(@"0973", @"National Commission on Terrorist Attacks upon the United States");
      myAgencyCodes.Add(@"0975", @"Commission on the Review of the Overseas Military Facility Structure of the United");
      myAgencyCodes.Add(@"0977", @"Antitrust Modernization Commission");
      
      myAgencyCodes.Add(@"1000", @"THE JUDICIAL BRANCH");
      myAgencyCodes.Add(@"1001", @"Supreme Court of the United States");
      myAgencyCodes.Add(@"1002", @"U.S. Courts of Appeals-Judicial Circuits - except the Federal Circuit");
      myAgencyCodes.Add(@"1003", @"U.S. Court of Appeals for the Federal Circuit");
      myAgencyCodes.Add(@"1004", @"U.S. Court of International Trade");
      myAgencyCodes.Add(@"1005", @"U.S. Court of Federal Claims");
      myAgencyCodes.Add(@"1012", @"U.S. District and Territorial Courts");
      myAgencyCodes.Add(@"1018", @"U.S. Judicial Panel on Multidistrict Litigation");
      myAgencyCodes.Add(@"1021", @"Bankruptcy Courts");
      myAgencyCodes.Add(@"1023", @"Federal Public Defenders");
      myAgencyCodes.Add(@"1025", @"Court Security");
      myAgencyCodes.Add(@"1027", @"Administrative Office of the U.S. Courts");
      myAgencyCodes.Add(@"1028", @"Federal Judicial Center");
      myAgencyCodes.Add(@"1030", @"United States Sentencing Commission");
      myAgencyCodes.Add(@"1050", @"Chief Justice of the United States");
      myAgencyCodes.Add(@"1051", @"Associate Justices of the Supreme Court");
      myAgencyCodes.Add(@"1059", @"Supreme Court, except Justices");
      myAgencyCodes.Add(@"1060", @"Judicial Conference of the United States");
      
      myAgencyCodes.Add(@"1100", @"EXECUTIVE OFFICE OF THE PRESIDENT");
      myAgencyCodes.Add(@"1101", @"President of the United States");
      myAgencyCodes.Add(@"1102", @"National Security Council");
      myAgencyCodes.Add(@"1103", @"Office of Management and Budget");
      myAgencyCodes.Add(@"1105", @"Office of Administration");
      myAgencyCodes.Add(@"1109", @"Office of the United States Trade Representative");
      myAgencyCodes.Add(@"1110", @"Office of Policy Development");
      myAgencyCodes.Add(@"1112", @"Office of Science and Technology Policy");
      myAgencyCodes.Add(@"1113", @"Council of Economic Advisors");
      myAgencyCodes.Add(@"1117", @"White House Office");
      myAgencyCodes.Add(@"1118", @"Executive Residence at the White House");
      myAgencyCodes.Add(@"1119", @"Council on Environmental Quality/Office of Environmental Quality");
      myAgencyCodes.Add(@"1121", @"Director, OMB");
      myAgencyCodes.Add(@"1127", @"Office of the National Drug Control Policy");
      myAgencyCodes.Add(@"1129", @"OMB, except Director");
      myAgencyCodes.Add(@"1140", @"President's Council on Sustainable Development");
      myAgencyCodes.Add(@"1141", @"African Development Foundation");
      myAgencyCodes.Add(@"1143", @"Inter-American Foundation");
      myAgencyCodes.Add(@"1145", @"Peace Corps");
      myAgencyCodes.Add(@"1148", @"Office of Homeland Security");
      myAgencyCodes.Add(@"1153", @"Trade and Development Agency");
      myAgencyCodes.Add(@"1160", @"Office of the Vice President of the United States");
      myAgencyCodes.Add(@"1165", @"White House Commission on Presidential Scholars");
      myAgencyCodes.Add(@"1170", @"Commission on the Intelligence Capabilities of the United States Regarding Weapons of");
      myAgencyCodes.Add(@"11DA", @"International Development Association");
      myAgencyCodes.Add(@"11DB", @"African Development Fund");
      myAgencyCodes.Add(@"11DE", @"Asian Development Bank");
      myAgencyCodes.Add(@"11DF", @"International Finance Corporation");
      myAgencyCodes.Add(@"11DG", @"International Fund for Agricultural Development");
      myAgencyCodes.Add(@"11DH", @"Inter-American Development Bank");
      myAgencyCodes.Add(@"11DK", @"European Bank for Reconstruction and Development");
      myAgencyCodes.Add(@"11DM", @"International Monetary Fund");
      myAgencyCodes.Add(@"11DN", @"North American Development Bank");
      myAgencyCodes.Add(@"11DR", @"International Bank for Reconstruction and Development (World Bank)");
      myAgencyCodes.Add(@"11DT", @"Inter-American Defense Board");
      myAgencyCodes.Add(@"11DW", @"Multilateral Investment Guarantee Agency");
      myAgencyCodes.Add(@"11RF", @"Multinational Force and Observers");

      myAgencyCodes.Add(@"1200", @"AGRICULTURE, DEPARTMENT OF");
      myAgencyCodes.Add(@"1201", @"Office of the Secretary of Agriculture");
      myAgencyCodes.Add(@"1203", @"USDA, Office of the General Counsel");
      myAgencyCodes.Add(@"1204", @"Office of the Inspector General");
      myAgencyCodes.Add(@"1205", @"USDA, Office of the Chief Financial Officer");
      myAgencyCodes.Add(@"1208", @"USDA, Office of Communications");
      myAgencyCodes.Add(@"1215", @"USDA, Office of Operations");
      myAgencyCodes.Add(@"1260", @"Commodity Credit Corporation");
      myAgencyCodes.Add(@"1261", @"Rural Telephone Bank");
      myAgencyCodes.Add(@"1263", @"Federal Crop Insurance Corporation");
      myAgencyCodes.Add(@"1291", @"Land grant colleges and Tuskegee Institute");
      myAgencyCodes.Add(@"12A0", @"USDA, Office of Executive Operations");
      myAgencyCodes.Add(@"12A2", @"USDA, Office of Executive Secretariat");
      myAgencyCodes.Add(@"12A4", @"USDA, Homeland Security");
      myAgencyCodes.Add(@"12A5", @"USDA, Office of the Chief Economist");
      myAgencyCodes.Add(@"12A6", @"USDA, Office of Budget and Program Analysis");
      myAgencyCodes.Add(@"12A7", @"USDA, Office of the Chief Information Officer");
      myAgencyCodes.Add(@"12A8", @"USDA, Office of Small and Disadvantaged Business");
      myAgencyCodes.Add(@"12A9", @"USDA, National Appeals Division");
      myAgencyCodes.Add(@"12B0", @"USDA, Assistant Secretary for Administration");
      myAgencyCodes.Add(@"12B1", @"USDA, Office of Security Services");
      myAgencyCodes.Add(@"12B2", @"USDA, Civil Rights");
      myAgencyCodes.Add(@"12B3", @"Property and Procurement Management");
      myAgencyCodes.Add(@"12B5", @"USDA, Office of Human Capital Management");
      myAgencyCodes.Add(@"12B6", @"USDA, Office of the Administrative Law Judge");
      myAgencyCodes.Add(@"12B7", @"USDA, Office of the Judicial Officer");
      myAgencyCodes.Add(@"12B8", @"USDA, Board of Contract Appeals `");
      myAgencyCodes.Add(@"12BD", @"USDA, Office of Ethics");
      myAgencyCodes.Add(@"12C0", @"Under Secretary for Natural Resources and Environment");
      myAgencyCodes.Add(@"12C2", @"Forest Service");
      myAgencyCodes.Add(@"12C3", @"Natural Resources Conservation Service");
      myAgencyCodes.Add(@"12CN", @"International Institute for Cotton");
      myAgencyCodes.Add(@"12D0", @"Under Secretary for Farm and Foreign Agricultural Services");
      myAgencyCodes.Add(@"12D2", @"Farm Service Agency");
      myAgencyCodes.Add(@"12D3", @"Foreign Agricultural Service");
      myAgencyCodes.Add(@"12D4", @"Risk Management Agency");
      myAgencyCodes.Add(@"12E0", @"Under Secretary for Rural Development");
      myAgencyCodes.Add(@"12E2", @"Rural Utilities Service");
      myAgencyCodes.Add(@"12E3", @"Rural Housing Service");
      myAgencyCodes.Add(@"12E4", @"Rural Business Cooperative Service");
      myAgencyCodes.Add(@"12E6", @"National Sheep Industry Improvement Center");
      myAgencyCodes.Add(@"12F0", @"Under Secretary for Food, Nutrition, and Consumer Services");
      myAgencyCodes.Add(@"12F2", @"Food and Nutrition Service");
      myAgencyCodes.Add(@"12F3", @"Center for Nutrition Policy and Promotion");
      myAgencyCodes.Add(@"12G0", @"Under Secretary for Food Safety");
      myAgencyCodes.Add(@"12G2", @"Food Safety and Inspection Service");
      myAgencyCodes.Add(@"12H0", @"Under Secretary for Research, Education, and Economics");
      myAgencyCodes.Add(@"12H2", @"Agricultural Research Service");
      myAgencyCodes.Add(@"12H3", @"Cooperative State Research, Education, and Extension Service");
      myAgencyCodes.Add(@"12H4", @"Economic Research Service");
      myAgencyCodes.Add(@"12H5", @"National Agricultural Statistics Service");
      myAgencyCodes.Add(@"12J0", @"Assistant Secretary for Congressional Relations");
      myAgencyCodes.Add(@"12J2", @"USDA, Office of Congressional and Intergovernmental Relations");
      myAgencyCodes.Add(@"12K0", @"Under Secretary for Marketing and Regulatory Programs");
      myAgencyCodes.Add(@"12K2", @"Agricultural Marketing Service");
      myAgencyCodes.Add(@"12K3", @"Animal and Plant Health Inspection Service");
      myAgencyCodes.Add(@"12K4", @"Grain Inspection, Packers and Stockyards Administration");
      
      myAgencyCodes.Add(@"1300", @"COMMERCE, DEPARTMENT OF");
      myAgencyCodes.Add(@"1301", @"Office of the Secretary");
      myAgencyCodes.Add(@"1303", @"Office of the General Counsel");
      myAgencyCodes.Add(@"1304", @"Office of the Inspector General");
      myAgencyCodes.Add(@"1306", @"Office - Chief Financial Officer & Assist. Secretary for Admin.");
      myAgencyCodes.Add(@"1314", @"Economic and Statistics Administration/Under Secretary for Economic Affairs");
      myAgencyCodes.Add(@"1315", @"Chief Economist");
      myAgencyCodes.Add(@"1321", @"Bureau of Economic Analysis");
      myAgencyCodes.Add(@"1323", @"Bureau of the Census");
      myAgencyCodes.Add(@"1325", @"Economic Development Administration/Assistant Secretary for Economic Development");
      myAgencyCodes.Add(@"1330", @"National Oceanic and Atmospheric Administration/Under Secretary for Oceans and");
      myAgencyCodes.Add(@"1335", @"National Telecommunication and Information Administration/Assistant Secretary for");
      myAgencyCodes.Add(@"1341", @"National Institute of Standards and Technology");
      myAgencyCodes.Add(@"1342", @"National Technical Information Service");
      myAgencyCodes.Add(@"1343", @"Assistant Secretary for Technology Policy");
      myAgencyCodes.Add(@"1344", @"Patent and Trademark Office/Under Secretary for Intellectual Property");
      myAgencyCodes.Add(@"1350", @"International Trade Administration");
      myAgencyCodes.Add(@"1351", @"Under Secretary for Export Administration/Bureau of Industry and Security");
      myAgencyCodes.Add(@"1352", @"Minority Business Development Agency");
      myAgencyCodes.Add(@"1359", @"Technology Administration/Under Secretary of Technology");
      myAgencyCodes.Add(@"1363", @"Committee for the Implementation of Textile Agreements");
      myAgencyCodes.Add(@"1365", @"Export Administration Review Board");
      myAgencyCodes.Add(@"1400", @"INTERIOR, Department of the");
      myAgencyCodes.Add(@"1402", @"Office of the Deputy Secretary of the Interior");
      myAgencyCodes.Add(@"1403", @"Office of the Solicitor");
      myAgencyCodes.Add(@"1404", @"Office of the Inspector General");
      myAgencyCodes.Add(@"1406", @"Office of Policy, Management and Budget/Chief Financial Officer");
      myAgencyCodes.Add(@"1407", @"Office of Congressional and Legislative Affairs");
      myAgencyCodes.Add(@"1408", @"Office of Communications");
      myAgencyCodes.Add(@"1409", @"Office of Insular Affairs");
      myAgencyCodes.Add(@"1410", @"Office of The Chief Information Officer");
      myAgencyCodes.Add(@"140S", @"Office of the Secretary of the Interior");
      myAgencyCodes.Add(@"1411", @"Office for Equal Opportunity");
      myAgencyCodes.Add(@"1413", @"Office of Hearings and Appeals");
      myAgencyCodes.Add(@"1414", @"Executive Secretariat & Office of Regulatory Affairs");
      myAgencyCodes.Add(@"1415", @"Office of the Special Trustee for American Indians");
      myAgencyCodes.Add(@"1418", @"Office of Small and Disadvantaged Business Utilization");
      myAgencyCodes.Add(@"1422", @"Bureau of Land Management");
      myAgencyCodes.Add(@"1425", @"Bureau of Reclamation");
      myAgencyCodes.Add(@"1428", @"National Business Center");
      myAgencyCodes.Add(@"142F", @"Fish and Wildlife and Parks (Assistant Secretary)");
      myAgencyCodes.Add(@"142L", @"Land and Minerals Management (Assistant Secretary)");
      myAgencyCodes.Add(@"142W", @"Water and Science (Assistant Secretary)");
      myAgencyCodes.Add(@"1434", @"Geological Survey");
      myAgencyCodes.Add(@"1435", @"Minerals Management Service");
      myAgencyCodes.Add(@"1438", @"Office of Surface Mining, Reclamation and Enforcement");
      myAgencyCodes.Add(@"1443", @"National Park Service.");
      myAgencyCodes.Add(@"1448", @"U.S. Fish and Wildlife Service");
      myAgencyCodes.Add(@"1450", @"Indian Affairs (Assistant Secretary)");
      myAgencyCodes.Add(@"1460", @"Board on Geographic Names");
      myAgencyCodes.Add(@"1461", @"Migratory Bird Conservation Commission");
      myAgencyCodes.Add(@"1464", @"Illinois and Michigan Canal National Heritage Corridor Commission");
      myAgencyCodes.Add(@"1465", @"Metropolitan River Corridors Study Committee");
      myAgencyCodes.Add(@"1466", @"Endangered Species Committee");
      myAgencyCodes.Add(@"1467", @"Utah Reclamation Mitigation and Conservation Commission");
      myAgencyCodes.Add(@"1468", @"Indian Arts and Crafts Board");
      myAgencyCodes.Add(@"1469", @"National Indian Gaming Commission");

      myAgencyCodes.Add(@"1500", @"JUSTICE, DEPARTMENT OF");
      myAgencyCodes.Add(@"1501", @"Offices, Boards and Divisions");
      myAgencyCodes.Add(@"1504", @"Office of the Inspector General");
      myAgencyCodes.Add(@"1524", @"Drug Enforcement Administration");
      myAgencyCodes.Add(@"1526", @"Executive Office for U.S. Attorneys and the Offices of U.S. Attorneys");
      myAgencyCodes.Add(@"1528", @"Immigration and Naturalization Service");
      myAgencyCodes.Add(@"1530", @"Executive Office for Immigration Review");
      myAgencyCodes.Add(@"1535", @"Community Relations Service");
      myAgencyCodes.Add(@"1540", @"Bureau of Prisoners/Federal Prison System");
      myAgencyCodes.Add(@"1544", @"U.S. Marshals Service");
      myAgencyCodes.Add(@"1549", @"Federal Bureau of Investigation");
      myAgencyCodes.Add(@"1550", @"Office of Justice Programs");
      myAgencyCodes.Add(@"1555", @"Executive Office for U.S. Trustees");
      myAgencyCodes.Add(@"1560", @"Bureau of Alcohol, Tobacco, Firearms and Explosives");
      myAgencyCodes.Add(@"1600", @"LABOR, Department of");
      myAgencyCodes.Add(@"1601", @"Immediate Office of the Secretary of Labor");
      myAgencyCodes.Add(@"1603", @"Office of the Solicitor");
      myAgencyCodes.Add(@"1604", @"Office of Inspector General");
      myAgencyCodes.Add(@"1605", @"Office of the Assistant Secretary for Administration and Management");
      myAgencyCodes.Add(@"1606", @"Office of the Assistant Secretary for Policy");
      myAgencyCodes.Add(@"1607", @"Office of the Assistant Secretary of Labor for Congressional and Intergovernmental");
      myAgencyCodes.Add(@"1608", @"Office of Public Affairs");
      myAgencyCodes.Add(@"1609", @"Bureau of International Labor Affairs");
      myAgencyCodes.Add(@"160S", @"Office of the Secretary of Labor");
      myAgencyCodes.Add(@"160U", @"Office of the Deputy Secretary of Labor");
      myAgencyCodes.Add(@"1610", @"Office of Adjudicatory Services");
      myAgencyCodes.Add(@"1613", @"Office of Administrative Law Judges");
      myAgencyCodes.Add(@"1615", @"Benefits Review Board");
      myAgencyCodes.Add(@"1616", @"Employees Compensation Appeals Board");
      myAgencyCodes.Add(@"1617", @"Administrative Review Board");
      myAgencyCodes.Add(@"1618", @"Office of Small Business Programs");
      myAgencyCodes.Add(@"1631", @"Office of Job Corps");
      myAgencyCodes.Add(@"1619", @"Executive Secretariat");
      myAgencyCodes.Add(@"1620", @"Office of 21");
      myAgencyCodes.Add(@"1621", @"Employee Benefits Security Administration");
      myAgencyCodes.Add(@"1622", @"Office of the Chief Financial Officer");
      myAgencyCodes.Add(@"1623", @"Center for Faith-based and Community Initiatives");
      myAgencyCodes.Add(@"1625", @"Bureau of Labor Statistics");
      myAgencyCodes.Add(@"1630", @"Employment and Training Administration");
      myAgencyCodes.Add(@"1635", @"Employment Standards Administration");
      myAgencyCodes.Add(@"1645", @"Mine Safety and Health Administration");
      myAgencyCodes.Add(@"1650", @"Occupational Safety and Health Administration");
      myAgencyCodes.Add(@"1653", @"Veterans Employment and Training Services");
      myAgencyCodes.Add(@"1655", @"Women's Bureau");
      myAgencyCodes.Add(@"1665", @"Pension Benefit Guaranty Corporation");
      myAgencyCodes.Add(@"1667", @"Office of Disability Employment Policy");

      myAgencyCodes.Add(@"1700", @"NAVY, DEPARTMENT OF THE");
      myAgencyCodes.Add(@"1708", @"Immediate Office of the Secretary of the Navy");
      myAgencyCodes.Add(@"1709", @"Navy Staff Offices");
      myAgencyCodes.Add(@"1710", @"Navy Field Offices");
      myAgencyCodes.Add(@"1711", @"Immediate Office of the Chief of Naval Operations");
      myAgencyCodes.Add(@"1712", @"Navy Secretariat/Staff Offices");
      myAgencyCodes.Add(@"1714", @"Office of Naval Research");
      myAgencyCodes.Add(@"1715", @"Naval Intelligence Command");
      myAgencyCodes.Add(@"1718", @"Naval Medical Command");
      myAgencyCodes.Add(@"1719", @"Naval Air Systems Command");
      myAgencyCodes.Add(@"1722", @"Bureau of Naval Personnel");
      myAgencyCodes.Add(@"1723", @"Naval Supply Systems Command");
      myAgencyCodes.Add(@"1724", @"Naval Sea Systems Command");
      myAgencyCodes.Add(@"1725", @"Naval Facilities Engineering Command");
      myAgencyCodes.Add(@"1727", @"U.S. Marine Corps");
      myAgencyCodes.Add(@"1730", @"Special Projects Office");
      myAgencyCodes.Add(@"1733", @"Military Sealift Command");
      myAgencyCodes.Add(@"1739", @"Naval Space and Warfare Systems Command");
      myAgencyCodes.Add(@"1741", @"Naval Systems Management Activity");
      myAgencyCodes.Add(@"1752", @"Commander, Navy Installations");
      myAgencyCodes.Add(@"1760", @"U.S. Atlantic Fleet, Commander In Chief");
      myAgencyCodes.Add(@"1761", @"U.S. Naval Forces, Europe");
      myAgencyCodes.Add(@"1762", @"Chief of Naval Education and Training");
      myAgencyCodes.Add(@"1763", @"Naval Network Operations Command");
      myAgencyCodes.Add(@"1765", @"Naval Oceanography Command");
      myAgencyCodes.Add(@"1769", @"Naval Security Group Command");
      myAgencyCodes.Add(@"1770", @"U.S. Pacific Fleet, Commander in Chief");
      myAgencyCodes.Add(@"1772", @"Naval Reserve Force");
      myAgencyCodes.Add(@"1774", @"Naval Special Warfare Command");
      myAgencyCodes.Add(@"1776", @"Naval Education and Training Command");
      myAgencyCodes.Add(@"17ZS", @"U.S. Special Operations Command (Navy)");

      myAgencyCodes.Add(@"1800", @"U.S. POSTAL SERVICE");
      myAgencyCodes.Add(@"1804", @"Office -Inspector General of the U.S. Postal Service");
      myAgencyCodes.Add(@"1810", @"U.S. Postal Inspection Service");
      myAgencyCodes.Add(@"1861", @"Postal Rate Commission");
      myAgencyCodes.Add(@"1865", @"Citizens' Stamp Advisory Committee");
      myAgencyCodes.Add(@"18AP", @"Postal Union of the Americas and Spain and Portugal");

      myAgencyCodes.Add(@"1900", @"STATE, DEPARTMENT OF");
      myAgencyCodes.Add(@"1901", @"Immediate Office of the Secretary of State");
      myAgencyCodes.Add(@"1902", @"Office of the Deputy Secretary of State");
      myAgencyCodes.Add(@"1903", @"Office of the Legal Adviser");
      myAgencyCodes.Add(@"1904", @"Office of the Inspector General");
      myAgencyCodes.Add(@"1905", @"Office of the Under Secretary for Management");
      myAgencyCodes.Add(@"1906", @"Policy Planning Council");
      myAgencyCodes.Add(@"1907", @"Bureau of Legislative Affairs");
      myAgencyCodes.Add(@"190S", @"Office of the Secretary of State");
      myAgencyCodes.Add(@"1910", @"Chief of Protocol");
      myAgencyCodes.Add(@"1911", @"Office of Equal Employment Opportunity and Civil Rights");
      myAgencyCodes.Add(@"1912", @"Coordinator for Counter-Terrorism");
      myAgencyCodes.Add(@"1913", @"Office of the Under Secretary for Political Affairs");
      myAgencyCodes.Add(@"1914", @"Office of the Under Secretary for Economic and Agricultural Affairs");
      myAgencyCodes.Add(@"1915", @"Office of the Under Secretary for Arms Control & International Security Affairs");
      myAgencyCodes.Add(@"1916", @"Counselor of the Department");
      myAgencyCodes.Add(@"1917", @"Office of Executive Secretary");
      myAgencyCodes.Add(@"1918", @"Chief of Staff");
      myAgencyCodes.Add(@"1921", @"Bureau of African Affairs");
      myAgencyCodes.Add(@"1923", @"Bureau of East Asian and Pacific Affairs");
      myAgencyCodes.Add(@"1925", @"Bureau of European and Eurasian Affairs");
      myAgencyCodes.Add(@"1927", @"Bureau of Western Hemisphere Affairs");
      myAgencyCodes.Add(@"1928", @"Bureau of South Asian Affairs");
      myAgencyCodes.Add(@"1929", @"Bureau of Near Eastern Affairs");
      myAgencyCodes.Add(@"1930", @"Bureau of Consular Affairs");
      myAgencyCodes.Add(@"1931", @"Chief Financial Officer");
      myAgencyCodes.Add(@"1932", @"Foreign Service Institute");
      myAgencyCodes.Add(@"1933", @"Bureau of Economic and Business Affairs");
      myAgencyCodes.Add(@"1935", @"Director General of the Foreign Service and Director of Human Resource");
      myAgencyCodes.Add(@"1936", @"Bureau of Democracy, Human Rights and Labor");
      myAgencyCodes.Add(@"1937", @"Bureau of Administration");
      myAgencyCodes.Add(@"1938", @"Bureau of Diplomatic Security and Office of Foreign Missions");
      myAgencyCodes.Add(@"1939", @"Bureau of Intelligence and Research");
      myAgencyCodes.Add(@"1941", @"Office of the Under Secretary for Global Affairs");
      myAgencyCodes.Add(@"1942", @"Bureau of International Narcotics and Law Enforcement Affairs");
      myAgencyCodes.Add(@"1943", @"Office of Information Resources Management");
      myAgencyCodes.Add(@"1945", @"Bureau of International Organization Affairs");
      myAgencyCodes.Add(@"1946", @"Bureau of Resource Management");
      myAgencyCodes.Add(@"1948", @"Bureau of Oceans & International Environmental & Scientific Affairs");
      myAgencyCodes.Add(@"1950", @"Bureau of Political-Military Affairs");
      myAgencyCodes.Add(@"1951", @"Assistant Secretary for Arms Control");
      myAgencyCodes.Add(@"1952", @"Bureau of Public Affairs");
      myAgencyCodes.Add(@"1953", @"Bureau of Educational and Cultural Affairs");
      myAgencyCodes.Add(@"1954", @"Bureau of Population, Refugees and Migration");
      myAgencyCodes.Add(@"1955", @"Assistant Secretary for Non-Proliferation");
      myAgencyCodes.Add(@"1956", @"Assistant Secretary for Verification and Compliance");
      myAgencyCodes.Add(@"1957", @"International Public Information Core Group Secretariat");
      myAgencyCodes.Add(@"1959", @"Office of International Information Programs");
      myAgencyCodes.Add(@"195A", @"Eisenhower Exchange Fellowship Program");
      myAgencyCodes.Add(@"1960", @"Artistic Ambassador Advisory Committee");
      myAgencyCodes.Add(@"1961", @"Board of Foreign Scholarships");
      myAgencyCodes.Add(@"1962", @"Cultural Property Advisory Committee");
      myAgencyCodes.Add(@"1963", @". U.S. Advisory Commission on Public Diplomacy");
      myAgencyCodes.Add(@"1964", @"J. William Fulbright Foreign Scholarship Board");
      myAgencyCodes.Add(@"1965", @"Center for Cultural and Technical Interchange Between East and West");
      myAgencyCodes.Add(@"1966", @"Eastern Europe Student Exchange Program");
      myAgencyCodes.Add(@"1967", @"Center for Cultural and Technical Interchange Between North and South");
      myAgencyCodes.Add(@"1968", @"Russian Far East Technical Assistance Center");
      myAgencyCodes.Add(@"1969", @"National Endowment for Democracy");
      myAgencyCodes.Add(@"195B", @"Israeli Arab Scholarship Program");
      myAgencyCodes.Add(@"195C", @"Broadcasting Board of Governors");
      myAgencyCodes.Add(@"1958", @"Office of U.S. Ambassador to the United Nations");
      myAgencyCodes.Add(@"1991", @"American Institute in Taiwan");
      myAgencyCodes.Add(@"1993", @"Asia Foundation");
      myAgencyCodes.Add(@"19AA", @"Inter-American Institute for Cooperation on Agriculture");
      myAgencyCodes.Add(@"19AG", @"Pan-American Institute of Geography and History");
      myAgencyCodes.Add(@"19AH", @"Pan-American Health Organization");
      myAgencyCodes.Add(@"19AN", @"Inter-American Indian Institute");
      myAgencyCodes.Add(@"19AR", @"Pan-American Railway Congress Association");
      myAgencyCodes.Add(@"19AS", @"Organization of American States");
      myAgencyCodes.Add(@"19BC", @"U.S. and Canada International Boundary Commission");
      myAgencyCodes.Add(@"19BE", @"U.S. and Canada Border Environment Cooperation Commission");
      myAgencyCodes.Add(@"19BJ", @"U.S. and Canada International Joint Commission");
      myAgencyCodes.Add(@"19BM", @"U.S. and Mexico International Boundary and Water Commission");
      myAgencyCodes.Add(@"19CB", @"International Natural Rubber Organization");
      myAgencyCodes.Add(@"19CC", @"International Cotton Advisory Committee");
      myAgencyCodes.Add(@"19CF", @"International Coffee Organization");
      myAgencyCodes.Add(@"19CJ", @"International Jute Organization");
      myAgencyCodes.Add(@"19CL", @"International Lead and Zinc Study Group");
      myAgencyCodes.Add(@"19CR", @"International Rubber Study Group");
      myAgencyCodes.Add(@"19CS", @"International Sugar Organization");
      myAgencyCodes.Add(@"19CU", @"International Copper Study Group");
      myAgencyCodes.Add(@"19CV", @"International Office of the Vine and Wine");
      myAgencyCodes.Add(@"19CW", @"International Wheat Council");
      myAgencyCodes.Add(@"19CX", @"International Tropical Timber Organization");
      myAgencyCodes.Add(@"19DC", @"Permanent Joint Board on Defense - U.S. & Canada");
      myAgencyCodes.Add(@"19DM", @"Joint Mexican-United States Defense Commission");
      myAgencyCodes.Add(@"19EC", @"International Agency for Research on Cancer");
      myAgencyCodes.Add(@"19EM", @"International Bureau of Weights and Measures");
      myAgencyCodes.Add(@"19EP", @"Internatal Cntr -- Study of the Preserv. & Restoration of Cultural Property");
      myAgencyCodes.Add(@"19ES", @"International Seed Testing Association");
      myAgencyCodes.Add(@"19EV", @"International Union for the Protection of New Varieties of Plants");
      myAgencyCodes.Add(@"19EZ", @"International Office of Epizootics");
      myAgencyCodes.Add(@"19FA", @"North Atlantic Salmon Conservation Organization");
      myAgencyCodes.Add(@"19FD", @"Pacific Salmon Commission");
      myAgencyCodes.Add(@"19FE", @"International Council for the Exploration of the Sea");
      myAgencyCodes.Add(@"19FG", @"Great Lakes Fishery Commission");
      myAgencyCodes.Add(@"19FH", @"International Pacific Halibut Commission");
      myAgencyCodes.Add(@"19FK", @"North Pacific Anadromous Fish Commission");
      myAgencyCodes.Add(@"19FL", @"International Commission for the Conservation of Atlantic Tunas");
      myAgencyCodes.Add(@"19FN", @"Northwest Atlantic Fisheries Organization");
      myAgencyCodes.Add(@"19FR", @"Commission for the Conservation of Antarctic Marine Living Resources");
      myAgencyCodes.Add(@"19FT", @"Inter-American Tropical Tuna Commission");
      myAgencyCodes.Add(@"19FW", @"International Whaling Commission");
      myAgencyCodes.Add(@"19FY", @"North Pacific Marine Science Organization");
      myAgencyCodes.Add(@"19LA", @"International Bureau of the Permanent Court of Arbitration");
      myAgencyCodes.Add(@"19LH", @"Hague Conference on Private International Law");
      myAgencyCodes.Add(@"19LM", @"International Organization for Legal Metrology");
      myAgencyCodes.Add(@"19LU", @"International Institute for the Unification of Private Law");
      myAgencyCodes.Add(@"19MC", @"U.N. Memorial Cemetery Commission");
      myAgencyCodes.Add(@"19ME", @"International Organization for Migration");
      myAgencyCodes.Add(@"19MR", @"International Committee of the Red Cross");
      myAgencyCodes.Add(@"19NH", @"International Hydrographic Organization");
      myAgencyCodes.Add(@"19PC", @"Canada-United States Interparliamentary Group");
      myAgencyCodes.Add(@"19PM", @"Mexico-United States Interparliamentary Group");
      myAgencyCodes.Add(@"19PU", @"Interparliamentary Union");
      myAgencyCodes.Add(@"19RA", @"North Atlantic Assembly");
      myAgencyCodes.Add(@"19RC", @"Colombo Plan Council for Technical Cooperation");
      myAgencyCodes.Add(@"19RD", @"Colombo Plan for Coop. Econ. & Social Develop. in Asia and the Pacific");
      myAgencyCodes.Add(@"19RE", @"Organization for Economic Cooperation and Development");
      myAgencyCodes.Add(@"19RJ", @"Asia-Pacific Economic Cooperation");
      myAgencyCodes.Add(@"19RN", @"North Atlantic Treaty Organization");
      myAgencyCodes.Add(@"19RP", @"South Pacific Commission");
      myAgencyCodes.Add(@"19TC", @"Customs Cooperation Council");
      myAgencyCodes.Add(@"19TP", @"International Bureau for the Publication of Customs Tariffs");
      myAgencyCodes.Add(@"19TT", @"General Agreement on Tariffs and Trade");
      myAgencyCodes.Add(@"19TW", @"World Tourism Organization");
      myAgencyCodes.Add(@"19TX", @"Bureau of International Expositions");
      myAgencyCodes.Add(@"19UA", @"International Civil Aviation Organization");
      myAgencyCodes.Add(@"19UE", @"International Atomic Energy Agency");
      myAgencyCodes.Add(@"19UF", @"Food and Agriculture Organization");
      myAgencyCodes.Add(@"19UG", @"International Maritime Organization");
      myAgencyCodes.Add(@"19UH", @"World Health Organization");
      myAgencyCodes.Add(@"19UK", @"United Nations Industrial Development Organization");
      myAgencyCodes.Add(@"19UL", @"International Labor Organization");
      myAgencyCodes.Add(@"19UM", @"World Meteorological Organization");
      myAgencyCodes.Add(@"19UN", @"United Nations (and special programs)");
      myAgencyCodes.Add(@"19UP", @"Universal Postal Union");
      myAgencyCodes.Add(@"19UT", @"International Telecommunication Union");
      myAgencyCodes.Add(@"19UW", @"World Intellectual Property Organization");

      myAgencyCodes.Add(@"2000", @"TREASURY, DEPARTMENT OF THE");
      myAgencyCodes.Add(@"2001", @"Departmental Offices");
      myAgencyCodes.Add(@"2004", @"Inspector General");
      myAgencyCodes.Add(@"2014", @"Office of the Inspector General for Tax Administration");
      myAgencyCodes.Add(@"2022", @"Alcohol and Tobacco Tax and Trade Bureau");
      myAgencyCodes.Add(@"2026", @"Financial Crimes Enforcement Network");
      myAgencyCodes.Add(@"2033", @"Financial Management Service");
      myAgencyCodes.Add(@"2036", @"Bureau of the Public Debt");
      myAgencyCodes.Add(@"2041", @"Bureau of Engraving and Printing");
      myAgencyCodes.Add(@"2044", @"United States Mint");
      myAgencyCodes.Add(@"2046", @"Office of the Comptroller of the Currency");
      myAgencyCodes.Add(@"2047", @"Office of Thrift Supervision");
      myAgencyCodes.Add(@"2050", @"Internal Revenue Service");
      myAgencyCodes.Add(@"2061", @"Federal Financing Bank");
      myAgencyCodes.Add(@"2066", @"Community Development Financial Institutions");
      myAgencyCodes.Add(@"2092", @"Corporation for Public Broadcasting");
      myAgencyCodes.Add(@"2093", @"District of Columbia");
      myAgencyCodes.Add(@"2095", @"Legal Services Corporation");
      myAgencyCodes.Add(@"20LP", @"International Criminal Police Organization (INTERPOL)");

      myAgencyCodes.Add(@"2100", @"ARMY, DEPT. OF THE (except Corps of Engineers Civil program financing)");
      myAgencyCodes.Add(@"2130", @"National Guard Bureau");
      myAgencyCodes.Add(@"21AE", @"Acquisition Executive Support Command Agency");
      myAgencyCodes.Add(@"21AS", @"U.S. Army Intelligence and Security Command");
      myAgencyCodes.Add(@"21AT", @"U.S. Army Test and Evaluation Command");
      myAgencyCodes.Add(@"21AU", @"U.S. Army Audit Agency");
      myAgencyCodes.Add(@"21BA", @"U.S. Army Installation Management Agency");
      myAgencyCodes.Add(@"21CB", @"U.S. Army Criminal Investigation Command");
      myAgencyCodes.Add(@"21CE", @"U.S. Army Corps of Engineers, except civil program financing");
      myAgencyCodes.Add(@"21CS", @"Immediate Office of the Chief of Staff of the Army");
      myAgencyCodes.Add(@"21CZ", @"U.S. Army Information Systems Command");
      myAgencyCodes.Add(@"21E1", @"Immed. Office Comm.-In-Chief - U.S. Army Europe & 7th Army");
      myAgencyCodes.Add(@"21E2", @"21st Theater Army Area Command");
      myAgencyCodes.Add(@"21E3", @"U.S. Army Southern European Task Force");
      myAgencyCodes.Add(@"21E5", @"U.S. Army V Corps");
      myAgencyCodes.Add(@"21EB", @"1st Personnel Command");
      myAgencyCodes.Add(@"21ED", @"U.S. Military Community Activity, Heidelberg");
      myAgencyCodes.Add(@"21EN", @"Seventh Army Training Command");
      myAgencyCodes.Add(@"21EO", @"59th Ordnance Brigade");
      myAgencyCodes.Add(@"21FC", @"U.S. Army Forces Command");
      myAgencyCodes.Add(@"21FL", @"U.S. Army South Command");
      myAgencyCodes.Add(@"21G6", @"U.S. Army Network Enterprise Technology Command/9th Army Signal");
      myAgencyCodes.Add(@"21GB", @"Office of the Chief of the National Guard Bureau");
      myAgencyCodes.Add(@"21HR", @"U.S. Army Reserve Command");
      myAgencyCodes.Add(@"21HS", @"U.S. Army Health Services Command");
      myAgencyCodes.Add(@"21J1", @"U.S. Army Element SHAPE");
      myAgencyCodes.Add(@"21JA", @"Joint Activities");
      myAgencyCodes.Add(@"21MA", @"U.S. Military Academy");
      myAgencyCodes.Add(@"21MC", @"U.S. Army Medical Command");
      myAgencyCodes.Add(@"21MD", @"Surgeon General");
      myAgencyCodes.Add(@"21MP", @"U.S. Army Human Resources Command");
      myAgencyCodes.Add(@"21MT", @"Military Traffic Management Command");
      myAgencyCodes.Add(@"21MW", @"U.S. Army Military District of Washington");
      myAgencyCodes.Add(@"21NG", @"Army National Guard Units (Title 32)");
      myAgencyCodes.Add(@"21P1", @"U.S. Army, Pacific");
      myAgencyCodes.Add(@"21P8", @"Eighth U.S. Army");
      myAgencyCodes.Add(@"21PC", @"Military Entrance Processing Command");
      myAgencyCodes.Add(@"21RC", @"U.S. Army Recruiting Command");
      myAgencyCodes.Add(@"21SA", @"Office of the Secretary of the Army");
      myAgencyCodes.Add(@"21SB", @"Field Operating Offices of the Office of the Secretary of the Army");
      myAgencyCodes.Add(@"21SC", @"U.S. Army Space and Strategic Defense Command");
      myAgencyCodes.Add(@"21SE", @"Field Operating Agencies of the Army Staff Resourced through OA-22");
      myAgencyCodes.Add(@"21SF", @"Field Operating Agencies of the Army Staff");
      myAgencyCodes.Add(@"21SJ", @"Joint Services & Activities Supported by the Office, Secretary of the Army");
      myAgencyCodes.Add(@"21SP", @"U.S. Special Operation Command (Army)");
      myAgencyCodes.Add(@"21SS", @"Staff Support Agencies of the Chief of Staff, Army");
      myAgencyCodes.Add(@"21SU", @"U.S. Army Southern Command");
      myAgencyCodes.Add(@"21TC", @"U.S. Army Training and Doctrine Command");
      myAgencyCodes.Add(@"21X1", @"U.S. Army Materiel Command (AMC)");
      myAgencyCodes.Add(@"21X2", @"Headquarters, Army Materiel Command");
      myAgencyCodes.Add(@"21X3", @"Headquarters, Staff Support Activities, AMC");
      myAgencyCodes.Add(@"21X4", @"Training Activities, AMC");
      myAgencyCodes.Add(@"21X5", @"U.S. Army Materiel Command, all others");
      myAgencyCodes.Add(@"21X6", @"U.S. Army Missile Command");
      myAgencyCodes.Add(@"21X7", @"U.S. Army Tank-Auto. & Armament Command (TACOM)");
      myAgencyCodes.Add(@"21X8", @"U.S. Army Communications Electronics Command");
      myAgencyCodes.Add(@"21X9", @"U.S.A. Simulation, Training & Instrumentation Command");
      myAgencyCodes.Add(@"21XA", @"U.S. Army Chemical And Biological Defense Command");
      myAgencyCodes.Add(@"21XB", @"U.S. Army Chemical Materials Command");
      myAgencyCodes.Add(@"21XC", @"U.S. Army Soldiers System Command (SSC)");
      myAgencyCodes.Add(@"21XD", @"U.S. Army Research Laboratory Command");
      myAgencyCodes.Add(@"21XK", @"Materiel Acquisition Activities");
      myAgencyCodes.Add(@"21XL", @"Materiel Acquisition Project Managers");
      myAgencyCodes.Add(@"21XP", @"U.S. Army Security Assistance Command");
      myAgencyCodes.Add(@"21XQ", @"U.S. Army Operations Support Command");
      myAgencyCodes.Add(@"21XR", @"U.S. Army Research, Development and Engineering Command");
      myAgencyCodes.Add(@"21XT", @"U.S. Army Test, Measure, & Diagnostic Equip. Activity");
      myAgencyCodes.Add(@"21XX", @"Materiel Readiness Activities");
      myAgencyCodes.Add(@"2281", @"Resolution Funding Corporation (REFCORP)");
      myAgencyCodes.Add(@"2300", @"U.S. Tax Court");
      
      myAgencyCodes.Add(@"2400", @"OFFICE OF PERSONNEL MANAGEMENT");
      myAgencyCodes.Add(@"2460", @"Federal Prevailing Rate Advisory Committee");
      myAgencyCodes.Add(@"2461", @"President's Commission on White House Fellowships");
      myAgencyCodes.Add(@"24P1", @"Federal Executive Board-Albuquerque, NM");
      myAgencyCodes.Add(@"24P4", @"Federal Executive Board-Atlanta, GA");
      myAgencyCodes.Add(@"24P7", @"Federal Executive Board-Baltimore, MD");
      myAgencyCodes.Add(@"24Q1", @"Federal Executive Board-Boston, MA");
      myAgencyCodes.Add(@"24Q4", @"Federal Executive Board-Buffalo, NY");
      myAgencyCodes.Add(@"24Q7", @"Federal Executive Board-Chicago, IL");
      myAgencyCodes.Add(@"24R1", @"Federal Executive Board-Cincinnati, OH");
      myAgencyCodes.Add(@"24R4", @"Federal Executive Board-Cleveland, OH");
      myAgencyCodes.Add(@"24R7", @"Federal Executive Board-Dallas-Ft.Worth, TX");
      myAgencyCodes.Add(@"24S1", @"Federal Executive Board-Denver, CO");
      myAgencyCodes.Add(@"24S4", @"Federal Executive Board-Detroit, MI");
      myAgencyCodes.Add(@"24S7", @"Federal Executive Board-Honolulu, HI");
      myAgencyCodes.Add(@"24T1", @"Federal Executive Board-Houston, TX");
      myAgencyCodes.Add(@"24T4", @"Federal Executive Board-Kansas City, KS");
      myAgencyCodes.Add(@"24T7", @"Federal Executive Board-Los Angeles, CA");
      myAgencyCodes.Add(@"24V1", @"Federal Executive Board-Miami, FL");
      myAgencyCodes.Add(@"24V4", @"Federal Executive Board-Newark, NJ");
      myAgencyCodes.Add(@"24V7", @"Federal Executive Board-New Orleans, LA");
      myAgencyCodes.Add(@"24W1", @"Federal Executive Board-New York City, NY");
      myAgencyCodes.Add(@"24W4", @"Federal Executive Board-Oklahoma, City, OK");
      myAgencyCodes.Add(@"24W7", @"Federal Executive Board-Philadelphia, PA");
      myAgencyCodes.Add(@"24X1", @"Federal Executive Board-Pittsburgh, PA");
      myAgencyCodes.Add(@"24X4", @"Federal Executive Board-Portland, OR");
      myAgencyCodes.Add(@"24X7", @"Federal Executive Board-St. Louis, MO");
      myAgencyCodes.Add(@"24Y1", @"Federal Executive Board-San Antonio, TX");
      myAgencyCodes.Add(@"24Y4", @"Federal Executive Board-San Francisco, CA");
      myAgencyCodes.Add(@"24Y7", @"Federal Executive Board-Seattle, WA");
      myAgencyCodes.Add(@"24Z1", @"Federal Executive Board-Twin Cities, MN");
      myAgencyCodes.Add(@"2500", @"National Credit Union Administration");
      myAgencyCodes.Add(@"2600", @"Federal Retirement Thrift Investment Board");
      myAgencyCodes.Add(@"2700", @"Federal Communications Commission");
      
      myAgencyCodes.Add(@"2800", @"SOCIAL SECURITY ADMINISTRATION");
      myAgencyCodes.Add(@"2801", @"Office of the Commissioner");
      myAgencyCodes.Add(@"2803", @"Office of the General Counsel");
      myAgencyCodes.Add(@"2804", @"Office of the Inspector General");
      myAgencyCodes.Add(@"2805", @"Dep. Comm. - Fin., Assessment & Mgmt. & Senior Fin. Exec.");
      myAgencyCodes.Add(@"2806", @"Deputy Comm. - Programs, Policy, Evaluation, & Commun.");
      myAgencyCodes.Add(@"2807", @"Deputy Commissioner - Legislation and Congressional Affairs");
      myAgencyCodes.Add(@"2810", @"Deputy Commissioner - Human Resources");
      myAgencyCodes.Add(@"2811", @"Deputy Commissioner - Systems");
      myAgencyCodes.Add(@"2812", @"Deputy Commissioner - Operations");
      myAgencyCodes.Add(@"2814", @"Office of the Chief Actuary");
      myAgencyCodes.Add(@"2816", @"Deputy Commissioner - Communications");
      myAgencyCodes.Add(@"2818", @"Deputy Commissioner - Disability and Income Security Programs");
      myAgencyCodes.Add(@"2900", @"Federal Trade Commission");
      myAgencyCodes.Add(@"3100", @"Nuclear Regulatory Commission");
      myAgencyCodes.Add(@"3300", @"Smithsonian Institution");
      myAgencyCodes.Add(@"3320", @"Smithsonian Institution, except units admin under separate Bds of Trustees");
      myAgencyCodes.Add(@"3352", @"John F. Kennedy Center for the Performing Arts");
      myAgencyCodes.Add(@"3355", @"National Gallery of Art");
      myAgencyCodes.Add(@"3358", @"Woodrow Wilson International Center for Scholars");
      myAgencyCodes.Add(@"3400", @"U.S. International Trade Commission");

      myAgencyCodes.Add(@"3600", @"VETERANS AFFAIRS, Department of");
      myAgencyCodes.Add(@"3601", @"Office of the Secretary");
      myAgencyCodes.Add(@"3602", @"Office of the Deputy Secretary");
      myAgencyCodes.Add(@"3603", @"General Counsel");
      myAgencyCodes.Add(@"3604", @"Inspector General");
      myAgencyCodes.Add(@"3605", @"Assistant Secretary for Management");
      myAgencyCodes.Add(@"3606", @"Assistant Secretary for Policy and Planning");
      myAgencyCodes.Add(@"3607", @"Assistant Secretary for Public and Intergovernmental Affairs");
      myAgencyCodes.Add(@"3608", @"Assistant Secretary for Congressional and Legislative Affairs");
      myAgencyCodes.Add(@"3610", @"Board of Veterans Appeals");
      myAgencyCodes.Add(@"3611", @"Assistant Secretary for Human Resources and Administration");
      myAgencyCodes.Add(@"3612", @"Board of Contract Appeals");
      myAgencyCodes.Add(@"3613", @"Office of Small and Disadvantaged Business Utilization");
      myAgencyCodes.Add(@"3615", @"Center for Minority Veterans");
      myAgencyCodes.Add(@"3616", @"Center for Women Veterans");
      myAgencyCodes.Add(@"3617", @"Office of Employment Discrimination Complaint Adjudication");
      myAgencyCodes.Add(@"3618", @"Special Assistant for Veterans Service Organization Liaison");
      myAgencyCodes.Add(@"3619", @"Chief of Staff");
      myAgencyCodes.Add(@"3620", @"Under Secretary for Health / Veterans Health Administration");
      myAgencyCodes.Add(@"3621", @"Director, Austin Automation Center");
      myAgencyCodes.Add(@"3622", @"Deputy Assistant Secretary for Policy");
      myAgencyCodes.Add(@"3623", @"Deputy Assistant Secretary for Planning and Evaluation");
      myAgencyCodes.Add(@"3624", @"Immediate Office of Assistance Secretary for Policy and Planning");
      myAgencyCodes.Add(@"3625", @"Deputy Assistant Secretary for Budget");
      myAgencyCodes.Add(@"3627", @"Deputy Assistant Secretary for Financial Management");
      myAgencyCodes.Add(@"3628", @"Immediate Office of Assistant Secretary for Management");
      myAgencyCodes.Add(@"3630", @"Under Secretary for Memorial Affairs / National Cemetery System");
      myAgencyCodes.Add(@"3632", @"Deputy Assistant Secretary for Diversity and Equal Employment Opportunity");
      myAgencyCodes.Add(@"3633", @"Director - Office of Administration");
      myAgencyCodes.Add(@"3634", @"Deputy Assistant Secretary for Human Resources Management and Labor");
      myAgencyCodes.Add(@"3636", @"Immed. Office - Assist. Secretary for Public & Intergov. Affairs");
      myAgencyCodes.Add(@"3637", @"Deputy Assistant Secretary for Intergovernmental Affairs and International");
      myAgencyCodes.Add(@"3638", @"Deputy Assistant Secretary for Public Affairs");
      myAgencyCodes.Add(@"3640", @"Under Secretary for Benefits / Veterans Benefit Administration");
      myAgencyCodes.Add(@"3641", @"Deputy Assist. Secretary for Office of Resolution Management");
      myAgencyCodes.Add(@"3643", @"Deputy Assist. Secretary for Acquisition and Materiel Management");
      myAgencyCodes.Add(@"3646", @"Immediate Office of Assistant Secretary for Congressional and and Legislative");
      myAgencyCodes.Add(@"3644", @"Director - Asset Enterprise Management");
      myAgencyCodes.Add(@"3645", @"Associate Deputy for Labor - Management Relations");
      myAgencyCodes.Add(@"3647", @"Director, Congressional Affairs");
      myAgencyCodes.Add(@"3648", @"Deputy Assistant Secretary for Legislative Affairs");
      myAgencyCodes.Add(@"3649", @"Immed. Office - Assist. Secretary - Human Resources & Admin.");
      myAgencyCodes.Add(@"3650", @"Assistant Secretary for Information and Technology");
      myAgencyCodes.Add(@"3651", @"Immediate Office of the Assist. Sec. - Info. & Technology");
      myAgencyCodes.Add(@"3652", @"Deputy Assistant Sec. For Information & Technology");
      myAgencyCodes.Add(@"3655", @"Deputy Assistant Secretary for Security Preparedness");
      myAgencyCodes.Add(@"3656", @"Deputy Assistant Secretary for Security and Law Enforcement");
      myAgencyCodes.Add(@"4100", @"Merit Systems Protection Board");
      myAgencyCodes.Add(@"4500", @"Equal Employment Opportunity Commission");
      myAgencyCodes.Add(@"4567", @"Interagency Committee on Employment of People with Disabilities");
      myAgencyCodes.Add(@"4602", @"Appalachian Regional Commission");
      myAgencyCodes.Add(@"4607", @"Interstate Commission on the Potomac River Basin");
      myAgencyCodes.Add(@"4608", @"The Port Authority of NY and NJ");
      myAgencyCodes.Add(@"4609", @"Washington Metropolitan Area Transit Authority");

      myAgencyCodes.Add(@"4700", @"GENERAL SERVICES ADMINISTRATION");
      myAgencyCodes.Add(@"4701", @"Immediate Office of the Administrator");
      myAgencyCodes.Add(@"4703", @"Office of General Counsel");
      myAgencyCodes.Add(@"4704", @"Office of Inspector General");
      myAgencyCodes.Add(@"4705", @"Office of the Chief People Officer");
      myAgencyCodes.Add(@"4707", @"Office of Congressional and Intergovernmental Affairs");
      myAgencyCodes.Add(@"4708", @"Office of Citizen Services and Communications");
      myAgencyCodes.Add(@"4712", @"GSA Board of Contract Appeals");
      myAgencyCodes.Add(@"4717", @"Office of the Chief Financial Officer");
      myAgencyCodes.Add(@"4722", @"Offices of the Regional Administrators");
      myAgencyCodes.Add(@"4724", @"Office of Civil Rights");
      myAgencyCodes.Add(@"4728", @"Office of Small Business Utilization");
      myAgencyCodes.Add(@"4732", @"Office of the Federal Acquisition Service");
      myAgencyCodes.Add(@"4740", @"Public Buildings Service");
      myAgencyCodes.Add(@"4743", @"Office of Childcare");
      myAgencyCodes.Add(@"4745", @"Office of Governmentwide Policy");
      myAgencyCodes.Add(@"4750", @"Office of the Chief Information Officer");
      myAgencyCodes.Add(@"4760", @"Office of the Chief Acquisition Officer");
      myAgencyCodes.Add(@"4808", @"National Bipartisan Commission on the Future of Medicare");
      myAgencyCodes.Add(@"4817", @"State Justice Institute");
      myAgencyCodes.Add(@"4827", @"Commission on Ocean Policy");
      myAgencyCodes.Add(@"4835", @"Medicare Payment Advisory Commission");
      myAgencyCodes.Add(@"4836", @"Commission on Executive, Legislative and Judicial Salaries");
      myAgencyCodes.Add(@"4849", @"Office of Navajo and Hopi Indian Relocation");
      myAgencyCodes.Add(@"4856", @"Nuclear Waste Technical Review Board");
      myAgencyCodes.Add(@"4860", @"United State-China Economic and Security Review Commission");
      myAgencyCodes.Add(@"4864", @"U.S. Commission on International Religious Freedom");
      
      myAgencyCodes.Add(@"4900", @"NATIONAL SCIENCE FOUNDATION");
      myAgencyCodes.Add(@"4960", @"National Science Board");
      myAgencyCodes.Add(@"4965", @"Arctic Research Commission");
      myAgencyCodes.Add(@"49SA", @"International Astronomical Union");
      myAgencyCodes.Add(@"49SB", @"International Union of Biological Sciences");
      myAgencyCodes.Add(@"49SC", @"International Union of Pure and Applied Chemistry");
      myAgencyCodes.Add(@"49SD", @"International Union of Geodesy and Geophysics");
      myAgencyCodes.Add(@"49SE", @"International Geological Union");
      myAgencyCodes.Add(@"49SF", @"International Union of Pure and Applied Physics");
      myAgencyCodes.Add(@"49SG", @"International Geographical Union");
      myAgencyCodes.Add(@"49SK", @"International Union of Biochemistry & Molecular Biology");
      myAgencyCodes.Add(@"49SM", @"International Mathematical Union");
      myAgencyCodes.Add(@"49SN", @"International Union of Nutritional Sciences");
      myAgencyCodes.Add(@"49SO", @"International Commission on Optics");
      myAgencyCodes.Add(@"49SP", @"International Union of Physiological Sciences");
      myAgencyCodes.Add(@"49SQ", @"International Union for Quaternary Research");
      myAgencyCodes.Add(@"49SR", @"International Union of Radio Sciences");
      myAgencyCodes.Add(@"49SS", @"International Union of Psychological Science");
      myAgencyCodes.Add(@"49ST", @"International Union of Theoretical and Applied Mechanics");
      myAgencyCodes.Add(@"49SU", @"International Council of Scientific Unions");
      myAgencyCodes.Add(@"49SV", @"International Union for Pure and Applied Biophysics");
      myAgencyCodes.Add(@"49SW", @"International Union of Microbiological Societies");
      myAgencyCodes.Add(@"49SX", @"International Union of Crystallography");
      myAgencyCodes.Add(@"49SY", @"Pacific Science Association");
      myAgencyCodes.Add(@"49SZ", @"Committee on Science and Technology for Development");
      myAgencyCodes.Add(@"5000", @"Securities and Exchange Commission");
      myAgencyCodes.Add(@"5094", @"Securities Investor Protection Corporation");
      myAgencyCodes.Add(@"5100", @"Federal Deposit Insurance Corporation");
      myAgencyCodes.Add(@"5400", @"Federal Labor Relations Authority");
      myAgencyCodes.Add(@"5600", @"Central Intelligence Agency");

      myAgencyCodes.Add(@"5700", @"AIR FORCE, DEPARTMENT OF THE (Headquarters, USAF)");
      myAgencyCodes.Add(@"5701", @"Air Force Management Engineering Agency");
      myAgencyCodes.Add(@"5702", @"Air Force Inspection and Safety Center");
      myAgencyCodes.Add(@"5703", @"Air Force Operational Test and Evaluation Center");
      myAgencyCodes.Add(@"5704", @"Air Force Communications Agency");
      myAgencyCodes.Add(@"5705", @"Air Force Intelligence Service");
      myAgencyCodes.Add(@"5706", @"Air Force Audit Agency");
      myAgencyCodes.Add(@"5707", @"Air Force Office of Special Investigations");
      myAgencyCodes.Add(@"5708", @"Air Force Office of Security Police");
      myAgencyCodes.Add(@"5709", @"Air Force Personnel Center");
      myAgencyCodes.Add(@"5711", @"Air Force Manpower Agency");
      myAgencyCodes.Add(@"570B", @"U.S. Air Force Academy");
      myAgencyCodes.Add(@"570D", @"U.S. Air Forces, Europe");
      myAgencyCodes.Add(@"570I", @"Air Reserve Personnel Center");
      myAgencyCodes.Add(@"570J", @"Air Training Command");
      myAgencyCodes.Add(@"570K", @"Air University");
      myAgencyCodes.Add(@"570M", @"Headquarters, Air Force Reserve");
      myAgencyCodes.Add(@"570N", @"Immediate Office, Headquarters, USAF");
      myAgencyCodes.Add(@"570R", @"Pacific Air Forces");
      myAgencyCodes.Add(@"570U", @"Air Force Headquarters Air Intelligence Agency");
      myAgencyCodes.Add(@"570Y", @"Air Force Communications Command");
      myAgencyCodes.Add(@"571A", @"Air Force C2 & Intelligence, Surveillance, & Reconnaissance");
      myAgencyCodes.Add(@"571C", @"Air Combat Command");
      myAgencyCodes.Add(@"571G", @"Air Force Logistics Management Agency");
      myAgencyCodes.Add(@"571L", @"Air Mobility Command");
      myAgencyCodes.Add(@"571M", @"Air Force Materiel Command");
      myAgencyCodes.Add(@"571O", @"Air Force Center for Quality and Management Innovation");
      myAgencyCodes.Add(@"571P", @"Air Force Real Property Agency");
      myAgencyCodes.Add(@"571Q", @"HQ AF Flight Standards Agency");
      myAgencyCodes.Add(@"571S", @"Space Command");
      myAgencyCodes.Add(@"571W", @"Air Force Engineering and Services Center");
      myAgencyCodes.Add(@"5727", @"Air Force Agency for Modeling and Simulation");
      myAgencyCodes.Add(@"5728", @"Air Force Communication and Information Center");
      myAgencyCodes.Add(@"5729", @"Air Force National Security Emergency Preparedness");
      myAgencyCodes.Add(@"572A", @"Air Force Cost Center");
      myAgencyCodes.Add(@"572B", @"Air Force Doctrine Center");
      myAgencyCodes.Add(@"572C", @"Air Force Civilian Personnel Management Center");
      myAgencyCodes.Add(@"572D", @"Air Force Personnel Operations Agency");
      myAgencyCodes.Add(@"572E", @"Air Force Legal Services Center");
      myAgencyCodes.Add(@"572F", @"Air Force Medical Services Center");
      myAgencyCodes.Add(@"572G", @"Air Force Service Information and News Center");
      myAgencyCodes.Add(@"572H", @"Air Force Combat Operations Staff");
      myAgencyCodes.Add(@"572I", @"Air National Guard Support Center");
      myAgencyCodes.Add(@"572K", @"U.S. Air Force Historical Research Center");
      myAgencyCodes.Add(@"572L", @"Air Force Technical Applications Center");
      myAgencyCodes.Add(@"572M", @"Air Force Review Boards Office");
      myAgencyCodes.Add(@"572N", @"Air Force Center for Studies and Analyses");
      myAgencyCodes.Add(@"572P", @"Air Force Center for International Programs");
      myAgencyCodes.Add(@"572Q", @"Air Weather Service");
      myAgencyCodes.Add(@"572R", @"Air Force Program Executive Office");
      myAgencyCodes.Add(@"572S", @"HQ NORAD");
      myAgencyCodes.Add(@"572T", @"Air Force Supply Center");
      myAgencyCodes.Add(@"572U", @"Air Force Morale, Welfare and Recreation Center");
      myAgencyCodes.Add(@"572V", @"Air Force Disposal Agency");
      myAgencyCodes.Add(@"572W", @"Air Force District of Washington");
      myAgencyCodes.Add(@"572X", @"Air Force Real Estate Agency");
      myAgencyCodes.Add(@"572Y", @"Air Force Pentagon Communications. Agency");
      myAgencyCodes.Add(@"572Z", @"HQ Air Force Medical Operations Agency");
      myAgencyCodes.Add(@"5734", @"Air National Guard Units (Mobilization) (Title 5)");
      myAgencyCodes.Add(@"573C", @"Air Force Elements, U.S. Central Command");
      myAgencyCodes.Add(@"573D", @"Air Force Elements, U.S. Special Operations Command");
      myAgencyCodes.Add(@"573G", @"Air Force Elements, Europe");
      myAgencyCodes.Add(@"573I", @"Reservist, Centrally Managed");
      myAgencyCodes.Add(@"573K", @"HQ U.S. European Command");
      myAgencyCodes.Add(@"573L", @"Center for Air Force History");
      myAgencyCodes.Add(@"573M", @"Air Force Elements, U.S. Southern Command");
      myAgencyCodes.Add(@"573N", @"Air Force Elements, U.S. Atlantic Command");
      myAgencyCodes.Add(@"573O", @"Air Force Elements, U.S. Pacific Command");
      myAgencyCodes.Add(@"573Q", @"Air Force Elements, U.S. Strategic Command");
      myAgencyCodes.Add(@"573R", @"Air Force Elements, U.S. Readiness Command");
      myAgencyCodes.Add(@"573S", @"Headquarters, U.S. Space Command and NORAD");
      myAgencyCodes.Add(@"573T", @"Air Force Elements U.S. Transportation Command");
      myAgencyCodes.Add(@"573V", @"Air Force Elements, Other than Europe");
      myAgencyCodes.Add(@"573W", @"Air Force Center for Environmental Excellence");
      myAgencyCodes.Add(@"573Y", @"Air Force Frequency Management Center");
      myAgencyCodes.Add(@"573Z", @"Joint Services Survival, Evasion, Resistance and Escape Agency");
      myAgencyCodes.Add(@"574Z", @"Air National Guard");
      myAgencyCodes.Add(@"57NG", @"Air National Guard Units (Title 32)");
      myAgencyCodes.Add(@"57ZG", @"U.S. Special Operations Command (ANG Title 32)");
      myAgencyCodes.Add(@"57ZS", @"U.S. Special Operations Command (Air Force)");
      myAgencyCodes.Add(@"5900", @"NATIONAL FOUNDATION ON THE ARTS AND THE HUMANITIES");
      myAgencyCodes.Add(@"5915", @"Federal Council on the Arts and the Humanities");
      myAgencyCodes.Add(@"5920", @"National Endowment for the Arts");
      myAgencyCodes.Add(@"5940", @"National Endowment for the Humanities");
      myAgencyCodes.Add(@"5950", @"Institute of Museum Services");
      myAgencyCodes.Add(@"5963", @"National Council on the Arts");
      myAgencyCodes.Add(@"5966", @"National Council on the Humanities");
      myAgencyCodes.Add(@"6000", @"Railroad Retirement Board");
      myAgencyCodes.Add(@"6100", @"Consumer Product Safety Commission");
      myAgencyCodes.Add(@"6201", @"Office of Special Counsel");
      myAgencyCodes.Add(@"6300", @"National Labor Relations Board");
      myAgencyCodes.Add(@"6400", @"Tennessee Valley Authority");
      myAgencyCodes.Add(@"6500", @"Federal Maritime Commission");
      myAgencyCodes.Add(@"6800", @"ENVIRONMENTAL PROTECTION AGENCY");
      myAgencyCodes.Add(@"6801", @"Immediate Office of the Administrator of EPA");
      myAgencyCodes.Add(@"6802", @"Deputy Administrator");
      myAgencyCodes.Add(@"6803", @"Office of General Counsel");
      myAgencyCodes.Add(@"6804", @"Office of the Inspector General");
      myAgencyCodes.Add(@"6805", @"Assistant Administrator for Administration and Resources Management");
      myAgencyCodes.Add(@"6809", @"Assistant Administrator for International Affairs");
      myAgencyCodes.Add(@"680S", @"Office of the Administrator of EPA");
      myAgencyCodes.Add(@"6820", @"Assistant Administrator for Air and Radiation");
      myAgencyCodes.Add(@"6825", @"Assistant Administrator for Enforcement and Compliance Assurance");
      myAgencyCodes.Add(@"6830", @"Assistant Administrator for Prevention, Pesticides, and Toxic Substances");
      myAgencyCodes.Add(@"6840", @"Assistant Administrator for Research and Development");
      myAgencyCodes.Add(@"6845", @"Assistant Administrator for Water");
      myAgencyCodes.Add(@"6850", @"Assistant Administrator for Solid Waste and Emergency Response");
      myAgencyCodes.Add(@"6853", @"Office of the Chief Financial Officer");
      myAgencyCodes.Add(@"6860", @"Commission on Risk Assessment and Risk Management");
      myAgencyCodes.Add(@"68SA", @"Associate Administrator for Office of Public Affairs");
      myAgencyCodes.Add(@"68SB", @"Associate Administrator for Congressional and Intergovernmental Relations");
      myAgencyCodes.Add(@"68SC", @"Office of Homeland Security");
      myAgencyCodes.Add(@"68SD", @"Environmental Appeals Board");
      myAgencyCodes.Add(@"68SE", @"Executive Secretariat");
      myAgencyCodes.Add(@"68SF", @"Office of Executive Services");
      myAgencyCodes.Add(@"68SG", @"Office of Administrative Law Judges");
      myAgencyCodes.Add(@"68SH", @"Office of Civil Rights");
      myAgencyCodes.Add(@"68SJ", @"Office of Cooperative Environmental Management");
      myAgencyCodes.Add(@"68SK", @"Office of Small and Disadvantaged Business Utilization");
      myAgencyCodes.Add(@"68SL", @"Office of Policy, Economics and Innovation");
      myAgencyCodes.Add(@"68SM", @"Science Advisory Board");
      myAgencyCodes.Add(@"6900", @"TRANSPORTATION, Department of");
      myAgencyCodes.Add(@"6901", @"Immediate Office of the Secretary of Transportation");
      myAgencyCodes.Add(@"6902", @"Associate Deputy Secretary/ Office of Intermodalism");
      myAgencyCodes.Add(@"6903", @"Office of General Counsel");
      myAgencyCodes.Add(@"6904", @"Office of Inspector General");
      myAgencyCodes.Add(@"6905", @"Assistant Secretary for Administration");
      myAgencyCodes.Add(@"6906", @"Assist. Secretary for Budget and Programs/Chief Financial Officer");
      myAgencyCodes.Add(@"6907", @"Assistant Secretary for Governmental Affairs");
      myAgencyCodes.Add(@"6908", @"Assistant Secretary - Office of Public Affairs");
      myAgencyCodes.Add(@"6909", @"Assistant Secretary for Transportation Policy");
      myAgencyCodes.Add(@"6910", @"Office of the Chief Information Officer");
      myAgencyCodes.Add(@"690S", @"Office of the Secretary of Transportation");
      myAgencyCodes.Add(@"6911", @"Office of Civil Rights");
      myAgencyCodes.Add(@"6912", @"Board of Contract Appeals");
      myAgencyCodes.Add(@"6913", @"Office of Commercial Space Transportation");
      myAgencyCodes.Add(@"6914", @"Executive Secretariat");
      myAgencyCodes.Add(@"6915", @"Office of Intelligence, Security and Emergency Response");
      myAgencyCodes.Add(@"6917", @"Deputy Secretary of Transportation");
      myAgencyCodes.Add(@"6916", @"Office of Drug and Alcohol Policy and Compliance");
      myAgencyCodes.Add(@"6918", @"OSDBU/Minority Business Resources Center");
      myAgencyCodes.Add(@"6920", @"Federal Aviation Administration");
      myAgencyCodes.Add(@"6922", @"Assistant Secretary for Aviation and International Affairs");
      myAgencyCodes.Add(@"6925", @"Federal Highway Administration");
      myAgencyCodes.Add(@"6930", @"Federal Railroad Administration");
      myAgencyCodes.Add(@"6938", @"Maritime Administration");
      myAgencyCodes.Add(@"6940", @"National Highway Traffic Safety Administration");
      myAgencyCodes.Add(@"6943", @"Research and Innovative Technology Administration");
      myAgencyCodes.Add(@"6947", @"Saint Lawrence Seaway Development Corporation");
      myAgencyCodes.Add(@"6953", @"Federal Motor Carrier Safety Administration");
      myAgencyCodes.Add(@"6955", @"Federal Transit Administration");
      myAgencyCodes.Add(@"6957", @"Pipeline and Hazardous Materials Safety Administration");
      myAgencyCodes.Add(@"6959", @"Surface Transportation Board - formerly ICC - code 3000");
      myAgencyCodes.Add(@"6991", @"National Railroad Passenger Corporation (AMTRAK)");
      myAgencyCodes.Add(@"7000", @"HOMELAND SECURITY, Department of");
      myAgencyCodes.Add(@"7001", @"Office of the Secretary, Department of Homeland Security");
      myAgencyCodes.Add(@"7002", @"Immediate Office of the Secretary");
      myAgencyCodes.Add(@"7003", @"U.S. Citizenship and Immigration Services");
      myAgencyCodes.Add(@"7004", @"Office of the Inspector General");
      myAgencyCodes.Add(@"7008", @"U.S. Coast Guard");
      myAgencyCodes.Add(@"7009", @"U.S. Secret Service");
      myAgencyCodes.Add(@"7012", @"U.S. Immigration and Customs Enforcement");
      myAgencyCodes.Add(@"7013", @"Transportation Security Administration");
      myAgencyCodes.Add(@"7014", @"U.S. Customs and Border Protection");
      myAgencyCodes.Add(@"7015", @"Federal Law Enforcement Training Center");
      myAgencyCodes.Add(@"7022", @"Federal Emergency Management Agency");
      myAgencyCodes.Add(@"7032", @"Office for Information Analysis");
      myAgencyCodes.Add(@"7033", @"Office for Infrastructure Protection");
      myAgencyCodes.Add(@"7040", @"Under Secretary for Science and Technology");
      myAgencyCodes.Add(@"7041", @"Office of the Under Secretary for Science and Technology");
      myAgencyCodes.Add(@"7050", @"Under Secretary for Management");
      myAgencyCodes.Add(@"7051", @"Office of the Under Secretary for Management");
      myAgencyCodes.Add(@"7100", @"Overseas Private Investment Corporation");
      myAgencyCodes.Add(@"7200", @"U.S. Agency for International Development");
      myAgencyCodes.Add(@"7300", @"Small Business Administration");
      myAgencyCodes.Add(@"7360", @"National Women's Business Council");
      myAgencyCodes.Add(@"7400", @"American Battle Monuments Commission");
      myAgencyCodes.Add(@"7500", @"HEALTH AND HUMAN SERVICES, Department of");
      myAgencyCodes.Add(@"7501", @"Immediate Office of the Secretary of Health and Human Services");
      myAgencyCodes.Add(@"7502", @"Office of the Deputy Secretary of Health and Human Services");
      myAgencyCodes.Add(@"7503", @"Office of the General Counsel");
      myAgencyCodes.Add(@"7504", @"Office of the Inspector General");
      myAgencyCodes.Add(@"7505", @"Office of Assistant Secretary for Management and Budget");
      myAgencyCodes.Add(@"7506", @"Office of Assistant Secretary for Planning and Evaluation");
      myAgencyCodes.Add(@"7507", @"Office of Assistant Secretary for Legislation");
      myAgencyCodes.Add(@"7508", @"Office of Assistant Secretary for Public Affairs");
      myAgencyCodes.Add(@"7509", @"Office of Intergovernmental Affairs and Regional Directors");
      myAgencyCodes.Add(@"750S", @"Office of the Secretary of Health and Human Services");
      myAgencyCodes.Add(@"7511", @"Office for Civil Rights");
      myAgencyCodes.Add(@"7515", @"Office of Public Health and Science");
      myAgencyCodes.Add(@"7516", @"Departmental Appeals Board");
      myAgencyCodes.Add(@"7520", @"Public Health Service");
      myAgencyCodes.Add(@"7521", @"Office of the Assistant Secretary for Health & Surgeon General");
      myAgencyCodes.Add(@"7522", @"Substance Abuse and Mental Health Services Administration");
      myAgencyCodes.Add(@"7523", @"Centers for Disease Control and Prevention");
      myAgencyCodes.Add(@"7524", @"Food and Drug Administration");
      myAgencyCodes.Add(@"7525", @"Agency for Toxic Substances and Disease Registry");
      myAgencyCodes.Add(@"7526", @"Health Resources and Services Administration");
      myAgencyCodes.Add(@"7527", @"Indian Health Service");
      myAgencyCodes.Add(@"7528", @"Agency for Healthcare Research and Quality");
      myAgencyCodes.Add(@"7529", @"National Institutes of Health");
      myAgencyCodes.Add(@"7530", @"Centers for Medicare & Medicaid Services");
      myAgencyCodes.Add(@"7545", @"Administration on Aging");
      myAgencyCodes.Add(@"7555", @"Program Support Center");
      myAgencyCodes.Add(@"7590", @"Administration for Children and Families");
      myAgencyCodes.Add(@"7635", @"Christopher Columbus Fellowship Foundation");
      myAgencyCodes.Add(@"7800", @"Farm Credit Administration");
      myAgencyCodes.Add(@"7881", @"Bank for Cooperatives");
      myAgencyCodes.Add(@"7884", @"Farm Credit Banks");
      myAgencyCodes.Add(@"7886", @"Farm Credit System Financial Assistance Corporation");
      myAgencyCodes.Add(@"7888", @"Farm Credit System Insurance Corporation");
      myAgencyCodes.Add(@"7889", @"Federal Agricultural Mortgage Corporation (Farmer Mac)");
      myAgencyCodes.Add(@"8000", @"NATIONAL AERONAUTICS AND SPACE ADMINISTRATION");
      myAgencyCodes.Add(@"8001", @"Headquarters, NASA");
      myAgencyCodes.Add(@"8020", @"Ames Research Center");
      myAgencyCodes.Add(@"8022", @"Dryden Flight Research Center");
      myAgencyCodes.Add(@"8025", @"NASA Shared Services Center");
      myAgencyCodes.Add(@"8026", @"Goddard Space Flight Center");
      myAgencyCodes.Add(@"8029", @"NASA Management Office, Jet Propulsion Laboratory");
      myAgencyCodes.Add(@"8032", @"Lyndon B. Johnson Space Center");
      myAgencyCodes.Add(@"8035", @"John F. Kennedy Space Center");
      myAgencyCodes.Add(@"8038", @"Langley Research Center");
      myAgencyCodes.Add(@"8041", @"John H. Glenn Research Center at Lewis Field");
      myAgencyCodes.Add(@"8044", @"George C. Marshall Space Flight Center");
      myAgencyCodes.Add(@"8047", @"John C. Stennis Space Center");
      myAgencyCodes.Add(@"8291", @"Neighborhood Reinvestment Corporation");
      myAgencyCodes.Add(@"8300", @"Export-Import Bank of the U.S.");
      myAgencyCodes.Add(@"8400", @"United States Soldiers' and Airmen's Home");
      myAgencyCodes.Add(@"84AF", @"Armed Forces Retirement Home");
      myAgencyCodes.Add(@"8600", @"HOUSING AND URBAN DEVELOPMENT, Department of");
      myAgencyCodes.Add(@"8601", @"Office of the Secretary of Housing and Urban Development");
      myAgencyCodes.Add(@"8602", @"Deputy Secretary of Housing and Urban Development");
      myAgencyCodes.Add(@"8603", @"Office of General Counsel");
      myAgencyCodes.Add(@"8604", @"Office of Inspector General");
      myAgencyCodes.Add(@"8605", @"Assistant Secretary for Administration");
      myAgencyCodes.Add(@"8606", @"Office of Chief Financial Officer");
      myAgencyCodes.Add(@"8607", @"Assistant Secretary for Congressional and Intergovernmental Relations");
      myAgencyCodes.Add(@"8608", @"Assistant Secretary for Public Affairs");
      myAgencyCodes.Add(@"8611", @"Assistant Secretary for Fair Housing and Equal Opportunity");
      myAgencyCodes.Add(@"8613", @"HUD Board of Contract Appeals");
      myAgencyCodes.Add(@"8615", @"Office of Administrative Law Judges");
      myAgencyCodes.Add(@"8617", @"Office of Small and Disadvantaged Business Utilization");
      myAgencyCodes.Add(@"8620", @"Assistant Secretary for Community Planning and Development");
      myAgencyCodes.Add(@"8622", @"Assistant Deputy Secretary for Field Policy and Management");
      myAgencyCodes.Add(@"8625", @"Government National Mortgage Association (Ginnie Mae)");
      myAgencyCodes.Add(@"8627", @"Office of Departmental Equal Employment Opportunity");
      myAgencyCodes.Add(@"8630", @"Assistant Secretary for Housing--Federal Housing Commissioner");
      myAgencyCodes.Add(@"8635", @"Assistant Secretary for Public and Indian Housing");
      myAgencyCodes.Add(@"8645", @"Assistant Secretary for Policy Development and Research");
      myAgencyCodes.Add(@"8651", @"Office of Departmental Operations and Coordination");
      myAgencyCodes.Add(@"8652", @"Office of the Chief Information Officer");
      myAgencyCodes.Add(@"8653", @"Office Healthy Homes and Lead Hazard Control");
      myAgencyCodes.Add(@"8654", @"Office of the Chief Procurement Officer");
      myAgencyCodes.Add(@"8656", @"Office of Federal Housing Enterprise Oversight");
      myAgencyCodes.Add(@"8658", @"Center for Faith-based and Community Initiatives");
      myAgencyCodes.Add(@"8659", @"Office of Field Policy and Management");
      myAgencyCodes.Add(@"865A", @"Office of the Field Policy and Management Region I, Boston Regional Office");
      myAgencyCodes.Add(@"865B", @"Office of the Field Policy and Management Region II, New York Regional Office");
      myAgencyCodes.Add(@"865C", @"Office of the Field Policy and Management Region III, Philadelphia Regional");
      myAgencyCodes.Add(@"865D", @"Office of Field Policy and Management Region IV, Atlanta Regional Office");
      myAgencyCodes.Add(@"865E", @"Office of Field Policy and Management Region V, Chicago Regional Office");
      myAgencyCodes.Add(@"865F", @"Office of Field Policy and Management Region VI, Fort Worth Regional Office");
      myAgencyCodes.Add(@"865G", @"Office of Field Policy and Management Region VII, Kansas City Regional Office");
      myAgencyCodes.Add(@"865H", @"Office of Field Policy and Management Region VIII, Denver Regional Office");
      myAgencyCodes.Add(@"865J", @"Office of Field Policy and Management Region IX, San Francisco Regional Office");
      myAgencyCodes.Add(@"865K", @"Office of Field Policy and Management Region X, Seattle Regional Office");
      myAgencyCodes.Add(@"8681", @"Federal National Mortgage Association (Fannie Mae)");
      myAgencyCodes.Add(@"8683", @"Federal Home Loan Mortgage Corporation (Freddie Mac)");
      myAgencyCodes.Add(@"8800", @"NATIONAL ARCHIVES AND RECORDS ADMINISTRATION");
      myAgencyCodes.Add(@"8852", @"Information Security Oversight Office");
      myAgencyCodes.Add(@"8861", @"National Archives Trust Fund Board");
      myAgencyCodes.Add(@"8862", @"National Historical Publications and Records Commission");
      myAgencyCodes.Add(@"8865", @"Administrative Committee of the Federal Register");
      myAgencyCodes.Add(@"8900", @"ENERGY, Department of");
      myAgencyCodes.Add(@"8902", @"Office of the Deputy Secretary");
      myAgencyCodes.Add(@"8901", @"Office of the Secretary");
      myAgencyCodes.Add(@"8903", @"Office of General Counsel");
      myAgencyCodes.Add(@"8904", @"Office of Inspector General");
      myAgencyCodes.Add(@"8905", @"Office of Congressional and Intergovernmental Affairs");
      myAgencyCodes.Add(@"8906", @"Office of Hearings and Appeals");
      myAgencyCodes.Add(@"8911", @"Office of Civilian Radioactive Waste Management");
      myAgencyCodes.Add(@"8915", @"Office of Public Affairs");
      myAgencyCodes.Add(@"8917", @"Assistant Secretary for Environmental Management");
      myAgencyCodes.Add(@"8921", @"Assistant Secretary for Energy Efficiency and Renewable Energy");
      myAgencyCodes.Add(@"8925", @"Office of Science");
      myAgencyCodes.Add(@"8927", @"Office of Nuclear Energy, Science and Technology");
      myAgencyCodes.Add(@"8928", @"Assistant Secretary for Fossil Energy");
      myAgencyCodes.Add(@"8929", @"Office of Intelligence");
      myAgencyCodes.Add(@"8932", @"Office of Policy and International Affairs");
      myAgencyCodes.Add(@"8955", @"Consolidated Business Center");
      myAgencyCodes.Add(@"8974", @"Strategic Petroleum Reserves");
      myAgencyCodes.Add(@"8981", @"Casper Naval Pet & Oil Shale Reserves");
      myAgencyCodes.Add(@"8982", @"Elk Hills Naval Pet & Oil Shale Reserves");
      myAgencyCodes.Add(@"8990", @"Federal Energy Regulatory Commission");
      myAgencyCodes.Add(@"891C", @"Office of Economic Impact and Diversity");
      myAgencyCodes.Add(@"891G", @"Office of Security, Safety and Performance Assurance");
      myAgencyCodes.Add(@"891N", @"Office of Chief Information Officer");
      myAgencyCodes.Add(@"891S", @"Office of the Departmental Representative to the Defense Nuclear Facilities");
      myAgencyCodes.Add(@"891Y", @"Assistant Secretary for Environment, Safety and Health");
      myAgencyCodes.Add(@"892E", @"Office of the Secretary of Energy Advisory Board Support Office");
      myAgencyCodes.Add(@"892H", @"Office of Counterintelligence");
      myAgencyCodes.Add(@"892L", @"Office of Electricity Delivery and Energy Reliability");
      myAgencyCodes.Add(@"893C", @"Office of the Chief Financial Officer");
      myAgencyCodes.Add(@"893H", @"Office of Human Capital Management");
      myAgencyCodes.Add(@"893M", @"Office of Management");
      myAgencyCodes.Add(@"89AL", @"Albuquerque Operations Office (non-NNSA) (EM)");
      myAgencyCodes.Add(@"89BA", @"Chicago Office");
      myAgencyCodes.Add(@"89BB", @"Oak Ridge Office");
      myAgencyCodes.Add(@"89BC", @"Ames Site Office");
      myAgencyCodes.Add(@"89BD", @"Argonne Site Office");
      myAgencyCodes.Add(@"89BE", @"Berkeley Site Office");
      myAgencyCodes.Add(@"89BF", @"Brookhaven Site Office");
      myAgencyCodes.Add(@"89BG", @"Fermi Site Office");
      myAgencyCodes.Add(@"89BH", @"Pacific Northwest Site Office");
      myAgencyCodes.Add(@"89BI", @"Princeton Site Office");
      myAgencyCodes.Add(@"89BJ", @"Stanford Site Office");
      myAgencyCodes.Add(@"89BK", @"Thomas Jefferson Site Office");
      myAgencyCodes.Add(@"89BP", @"Bonneville Power Marketing Administration");
      myAgencyCodes.Add(@"89CB", @"Carlsbad Field Office");
      myAgencyCodes.Add(@"89GO", @"Golden Field Office");
      myAgencyCodes.Add(@"89ID", @"Idaho Operations Office");
      myAgencyCodes.Add(@"89N0", @"Deputy Under Secretary for Counterterrorism");
      myAgencyCodes.Add(@"89N1", @"Deputy Administration for Defense Programs");
      myAgencyCodes.Add(@"89N2", @"Deputy Administrator for Defense Nuclear Nonproliferation");
      myAgencyCodes.Add(@"89N3", @"Deputy Administrator for Naval Reactors");
      myAgencyCodes.Add(@"89N4", @"Office of Emergency Operations");
      myAgencyCodes.Add(@"89N5", @"Associate Administrator for Facilities and Operations");
      myAgencyCodes.Add(@"89N6", @"Associate Administrator for Management and Administration");
      myAgencyCodes.Add(@"89N7", @"Associate Administrator for Defense Nuclear Security");
      myAgencyCodes.Add(@"89NA", @"Office of Nuclear Security/National Nuclear Security Administration");
      myAgencyCodes.Add(@"89NE", @"National Energy Technology Laboratory");
      myAgencyCodes.Add(@"89NV", @"Nevada Operations Office (non-National Nuclear");
      myAgencyCodes.Add(@"89NW", @"Oakland Operations Office");
      myAgencyCodes.Add(@"89OH", @"Ohio Field Office");
      myAgencyCodes.Add(@"89PP", @"Portsmouth and Paducah Project Office");
      myAgencyCodes.Add(@"89RF", @"Rocky Flats Project Office");
      myAgencyCodes.Add(@"89RL", @"Richland Operations Office");
      myAgencyCodes.Add(@"89RP", @"Office of River Protection");
      myAgencyCodes.Add(@"89SE", @"Southeastern Power Administration");
      myAgencyCodes.Add(@"89SR", @"Savannah River Operations Office");
      myAgencyCodes.Add(@"89SV", @"Savannah River Site Office");
      myAgencyCodes.Add(@"89SW", @"Southwestern Power Administration");
      myAgencyCodes.Add(@"89WA", @"Western Area Power Marketing Administration");
      myAgencyCodes.Add(@"89X1", @"Pittsburgh Naval Reactors");
      myAgencyCodes.Add(@"89X2", @"Schenectady Naval Reactors");
      myAgencyCodes.Add(@"89XQ", @"Y-12 Site Office");
      myAgencyCodes.Add(@"89XR", @"Pantex Site Office");
      myAgencyCodes.Add(@"89XS", @"Sandia Site Office");
      myAgencyCodes.Add(@"89XT", @"Kansas City Site Office");
      myAgencyCodes.Add(@"89XU", @"Los Alamos Site Office");
      myAgencyCodes.Add(@"89XV", @"Nevada Site Office");
      myAgencyCodes.Add(@"89XW", @"Livermore Site Office");
      myAgencyCodes.Add(@"89ZA", @"National Nuclear Security Administration Service Center");
      myAgencyCodes.Add(@"9000", @"Selective Service System");
      myAgencyCodes.Add(@"9100", @"EDUCATION, Department of");
      myAgencyCodes.Add(@"9101", @"Immediate Office of the Secretary of Education");
      myAgencyCodes.Add(@"9102", @"Office of the Deputy Secretary of Education");
      myAgencyCodes.Add(@"9103", @"Office of the General Counsel");
      myAgencyCodes.Add(@"9104", @"Office of Inspector General");
      myAgencyCodes.Add(@"9105", @"Office of Management");
      myAgencyCodes.Add(@"9106", @"Office of the Chief Financial Officer");
      myAgencyCodes.Add(@"9107", @"Office of Legislation and Congressional Affairs");
      myAgencyCodes.Add(@"9108", @"Office of the Under Secretary");
      myAgencyCodes.Add(@"9109", @"Office of Office of Communications and Outreach");
      myAgencyCodes.Add(@"9110", @"Office of Planning, Evaluation and Program Development");
      myAgencyCodes.Add(@"9111", @"Office for Civil Rights");
      myAgencyCodes.Add(@"9115", @"Office of the Chief Information Officer");
      myAgencyCodes.Add(@"9120", @"Office of Vocational and Adult Education");
      myAgencyCodes.Add(@"9121", @"Office of English Language Acquisition");
      myAgencyCodes.Add(@"9124", @"Office of Special Education and Rehabilitative Services");
      myAgencyCodes.Add(@"9125", @"National Institute on Disability and Rehabilitation Research");
      myAgencyCodes.Add(@"9126", @"Rehabilitation Services Administration");
      myAgencyCodes.Add(@"9127", @"Office of Special Education Programs");
      myAgencyCodes.Add(@"9128", @"Immed. Office - Assist. Secretary for Spec. Ed. & Rehab. Services");
      myAgencyCodes.Add(@"9129", @"Fund for the Improvement of Postsecondary Education");
      myAgencyCodes.Add(@"9130", @"Immediate Office of the Assist. Secretary for Postsecondary Education");
      myAgencyCodes.Add(@"9131", @"Federal Student Aid");
      myAgencyCodes.Add(@"9132", @"Immediate Office of the Director of Education Sciences");
      myAgencyCodes.Add(@"9133", @"Office of Higher Education Programs");
      myAgencyCodes.Add(@"9134", @"Office of Postsecondary Education");
      myAgencyCodes.Add(@"9135", @"National Center for Education Research");
      myAgencyCodes.Add(@"9136", @"National Center for Special Education Research");
      myAgencyCodes.Add(@"9137", @"National Center for Educational Evaluation and Regional Assistance");
      myAgencyCodes.Add(@"9138", @"National Center for Education Statistics");
      myAgencyCodes.Add(@"9139", @"Institute of Education Sciences");
      myAgencyCodes.Add(@"9140", @"Student Achievement and School Accountability Program");
      myAgencyCodes.Add(@"9141", @"Immed. Office - Assist. Secretary for Elementary & Secondary Ed.");
      myAgencyCodes.Add(@"9142", @"Migrant Education Programs");
      myAgencyCodes.Add(@"9143", @"School Support and Technology Programs");
      myAgencyCodes.Add(@"9144", @"Impact Aid Programs");
      myAgencyCodes.Add(@"9145", @"Office of Indian Education");
      myAgencyCodes.Add(@"9146", @"Office of Elementary and Secondary Education");
      myAgencyCodes.Add(@"9147", @"Academic Improvement and Teacher Quality Programs");
      myAgencyCodes.Add(@"9150", @"Office of Innovation and Improvement");
      myAgencyCodes.Add(@"9155", @"Office of Safe and Drug-Free Schools");
      myAgencyCodes.Add(@"916A", @"Advisory Councils and Committees");
      myAgencyCodes.Add(@"916B", @"National Assessment Governing Board");
      myAgencyCodes.Add(@"916C", @"National Institute for Literacy");
      myAgencyCodes.Add(@"916D", @"Federal Interagency Committee on Education");
      myAgencyCodes.Add(@"9181", @"Student Loan Marketing Association (Sallie Mae)");
      myAgencyCodes.Add(@"9182", @"College Construction Loan Insurance Association (Connie Lee)");
      myAgencyCodes.Add(@"9191", @"American Printing House for the Blind");
      myAgencyCodes.Add(@"9192", @"Gallaudet University");
      myAgencyCodes.Add(@"9193", @"Howard University");
      myAgencyCodes.Add(@"9194", @"National Technical Institute for the Deaf");
      myAgencyCodes.Add(@"9300", @"Federal Mediation and Conciliation Service");
      myAgencyCodes.Add(@"9502", @"National Capital Planning Commission");
      myAgencyCodes.Add(@"9504", @"Federal Mine Safety and Health Review Commission");
      myAgencyCodes.Add(@"9506", @"Federal Election Commission");
      myAgencyCodes.Add(@"9507", @"Commodity Futures Trading Commission");
      myAgencyCodes.Add(@"9508", @"National Transportation Safety Board");
      myAgencyCodes.Add(@"9509", @"National Council on Disability");
      myAgencyCodes.Add(@"9510", @"Harry S. Truman Scholarship Foundation");
      myAgencyCodes.Add(@"9512", @"Japan-U.S. Friendship Commission");
      myAgencyCodes.Add(@"9513", @"Marine Mammal Commission");
      myAgencyCodes.Add(@"9514", @"Occupational Safety and Health Review Commission");
      myAgencyCodes.Add(@"9516", @"Defense Nuclear Facilities Safety Board");
      myAgencyCodes.Add(@"9517", @"Commission on Civil Rights");
      myAgencyCodes.Add(@"9518", @"Committee for Purchase from People who are Blind or Severely Disabled");
      myAgencyCodes.Add(@"9522", @"Centennial of Flight Commission");
      myAgencyCodes.Add(@"9523", @"Election Assistance Commission");
      myAgencyCodes.Add(@"9524", @"National Mediation Board");
      myAgencyCodes.Add(@"9525", @"National Counterintelligence Center");
      myAgencyCodes.Add(@"9527", @"National Commission on Libraries and Information Science");
      myAgencyCodes.Add(@"9530", @"Advisory Council On Historic Preservation");
      myAgencyCodes.Add(@"9531", @"United States Holocaust Memorial Museum");
      myAgencyCodes.Add(@"9532", @"Architectural and Transportation Barriers Compliance Board");
      myAgencyCodes.Add(@"9535", @"Institute of Am. Indian and Alaska Native Culture and Arts Development");
      myAgencyCodes.Add(@"9537", @"Commission of Fine Arts");
      myAgencyCodes.Add(@"9540", @"Federal Housing Finance Board");
      myAgencyCodes.Add(@"9541", @"James Madison Memorial Fellowship Foundation");
      myAgencyCodes.Add(@"9543", @"Millennium Challenge Corporation");
      myAgencyCodes.Add(@"9545", @"Morris K. Udall Scholar. & Excellence in National Enviro. Policy Foundation");
      myAgencyCodes.Add(@"9547", @"Delta Regional Authority");
      myAgencyCodes.Add(@"9549", @"Office of Government Ethics");
      myAgencyCodes.Add(@"9554", @"Interagency Council on the Homeless");
      myAgencyCodes.Add(@"9555", @"National Communications System");
      myAgencyCodes.Add(@"9559", @"Federal Reserve System, Board of Governors");
      myAgencyCodes.Add(@"9560", @"Barry Goldwater Scholarship and Excellence in Education Foundation");
      myAgencyCodes.Add(@"9562", @"Federal Financial Institutions Examination Council");
      myAgencyCodes.Add(@"9565", @"Chemical Safety and Hazard Investigation Board");
      myAgencyCodes.Add(@"9567", @"Commission for the Preservation of America's Heritage Abroad");
      myAgencyCodes.Add(@"9568", @"Broadcasting Board of Governors");
      myAgencyCodes.Add(@"9570", @"Presidio Trust");
      myAgencyCodes.Add(@"9572", @"Delani Commission");
      myAgencyCodes.Add(@"9574", @"White House Commission on the National Moment of Remembrance");
      myAgencyCodes.Add(@"9577", @"Corporation for National and Community Service");
      myAgencyCodes.Add(@"9580", @"United States Institute of Peace");
      myAgencyCodes.Add(@"9585", @"Financing Corporation (FICO)");
      myAgencyCodes.Add(@"9589", @"United Mine Workers of America Benefit Funds");
      myAgencyCodes.Add(@"9590", @"Valles Caldera Trust");
      myAgencyCodes.Add(@"9591", @"Vietnam Education Foundation");
      myAgencyCodes.Add(@"9593", @"U.S. Court of Veterans Appeals");
      myAgencyCodes.Add(@"9594", @"Court Services and Offender Supervision Agency for the District of Columbia");
      myAgencyCodes.Add(@"959D", @"Office of the Director - CSOSA");
      myAgencyCodes.Add(@"959P", @"Pretrial Services Agency - CSOSA");
      myAgencyCodes.Add(@"95HL", @"Federal Home Loan Banks");
      myAgencyCodes.Add(@"9668", @"Mississippi River Commission");
      myAgencyCodes.Add(@"96CE", @"U.S. Army Corps of Engineers - civil program financing only");
      myAgencyCodes.Add(@"96NC", @"Permanent International Association of Navigation Congresses");
      myAgencyCodes.Add(@"96RC", @"Permanent International Association of Road Congresses");
      myAgencyCodes.Add(@"9700", @"DEFENSE, Department of (except military departments)");
      myAgencyCodes.Add(@"9736", @"Army/Air Force Exchange Service");
      myAgencyCodes.Add(@"9748", @"Defense Human Resources Activity");
      myAgencyCodes.Add(@"9758", @"Defense Prisoner of War/Missing Personnel Office");
      myAgencyCodes.Add(@"9759", @"Consolidated Metropolitan Technical Personnel Center");
      myAgencyCodes.Add(@"9760", @"TRICARE Management Activity");
      myAgencyCodes.Add(@"9761", @"Defense Threat Reduction Agency");
      myAgencyCodes.Add(@"9762", @"Defense Career Management and Support Agency");
      myAgencyCodes.Add(@"9763", @"Defense Contract Management Agency");
      myAgencyCodes.Add(@"9765", @"Pentagon Force Protection Agency");
      myAgencyCodes.Add(@"9766", @"Department of Defense Counterintelligence Field Activity");
      myAgencyCodes.Add(@"9767", @"Unified Combatant Command Headquarters");
      myAgencyCodes.Add(@"9769", @"National Defense University");
      myAgencyCodes.Add(@"9770", @"Armed Forces Radiobiology Research Institute");
      myAgencyCodes.Add(@"9771", @"Defense Microelectronics Activity");
      myAgencyCodes.Add(@"9772", @"Pentagon Renovation Program Office");
      myAgencyCodes.Add(@"9773", @"Virginia Contracting Activity");
      myAgencyCodes.Add(@"9774", @"Defense Technical Information Center");
      myAgencyCodes.Add(@"97AB", @"National Geospatial-Intelligence Agency");
      myAgencyCodes.Add(@"97AD", @"Office of the Secretary of Defense");
      myAgencyCodes.Add(@"97AE", @"Defense Advanced Research Projects Agency");
      myAgencyCodes.Add(@"97AK", @"Defense Information Systems Agency");
      myAgencyCodes.Add(@"97AQ", @"Defense Legal Services Agency");
      myAgencyCodes.Add(@"97AR", @"Defense Contract Audit Agency");
      myAgencyCodes.Add(@"97AS", @"Defense Logistics Agency");
      myAgencyCodes.Add(@"97AT", @"Defense Security Cooperation Agency");
      myAgencyCodes.Add(@"97AU", @"Defense Technology Security Administration");
      myAgencyCodes.Add(@"97AV", @"Defense Security Service");
      myAgencyCodes.Add(@"97AZ", @"Defense Commissary Agency");
      myAgencyCodes.Add(@"97BJ", @"Organization of the Joint Chiefs of Staff");
      myAgencyCodes.Add(@"97BZ", @"Defense Finance and Accounting Service");
      myAgencyCodes.Add(@"97CG", @"National Security Agency/Central Security Service");
      myAgencyCodes.Add(@"97DL", @"Defense Intelligence Agency");
      myAgencyCodes.Add(@"97EX", @"Office of the Inspector General");
      myAgencyCodes.Add(@"97F1", @"American Forces Information Service");
      myAgencyCodes.Add(@"97F2", @"Department of Defense Education Activity");
      myAgencyCodes.Add(@"97F5", @"Washington Headquarters Services");
      myAgencyCodes.Add(@"97F6", @"Office of Economic Adjustment");
      myAgencyCodes.Add(@"97GZ", @"U.S. Court of Appeals for the Armed Forces");
      myAgencyCodes.Add(@"97JC", @"Missile Defense Agency");
      myAgencyCodes.Add(@"97NH", @"United States Naval Home");
    }

    #endregion

    #region "Constants / Declarations"

    /// <summary>
    /// Start Sentinal
    /// </summary>
    private const byte FASCN_SS = 0x0B;

    /// <summary>
    /// Field Sentinal
    /// </summary>
    private const byte FASCN_FS = 0x0D;

    /// <summary>
    /// End Sentinal
    /// </summary>
    private const byte FASCN_ES = 0x0F; 

    #endregion
  }
}
#endregion