﻿#region "Copyright"
/******************************************************************************************
 Copyright (C) 2012 Kim O'Sullivan (kim@makina.com.au)

 Permission is hereby granted, free of charge, to any person obtaining a copy of 
 this software and associated documentation files (the "Software"), to deal in the 
 Software without restriction, including without limitation the rights to use, copy, 
 modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, 
 and to permit persons to whom the Software is furnished to do so, subject to the 
 following conditions:
 
 The above copyright notice and this permission notice shall be included in all copies 
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE 
 LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
 OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
 OTHER DEALINGS IN THE SOFTWARE.
******************************************************************************************/
#endregion

#region "Namespace definitions"
using System;
using System.Collections.Generic;
using System.Text;
using CardFramework.Protocols.Iso7816;
using CardFramework.Applications.Iso7816;
using CardFramework.Helpers;
using CardFramework.Formats;
using System.IO;
using System.Xml.Serialization;
using Org.BouncyCastle.Asn1;
#endregion

#region "Class definitions"
namespace CardFramework.Applications.Plaid
{
  public class PlaidKey
  {
    public short KeysetId;
    public RSAKey IAKey;
    public byte[] FAKey;
  }

  public class PlaidPersonalisationRecord
  {
    public byte[] DivData;

    public List<AcsRecord> AcsRecords = new List<AcsRecord>();

    [XmlIgnore()]
    public byte[] Pin;

    [XmlIgnore()]
    public byte[] Minutiae;

    [XmlIgnore()]
    public List<PlaidKey> Keys = new List<PlaidKey>();
  }

  
  public class PlaidApplication : Iso7816Application
	{
		#region "Members - Public"

    public override bool Discover(Card card)
    {
      // Create a backup of the reference to our old card
      Card oldCard = this.Card;

      // Temporarily set the card
      this.Card = card;

      try
      {
        SelectApplication();
        return true;
      }
      catch (Exception)
      {
        return false;
      }
      finally
      {
        this.Card = oldCard;    
      }
    }

    public void SelectApplication()
    {
      byte[] response = SelectFile(AID, SelectMode.DfName);
      return;
    }

    /// <summary>
    /// This method performs the full PLAID authentication sequence, given a valid, selected PLAID SAM
    /// </summary>
    /// <param name="sam"></param>
    /// <param name="keysets"></param>
    /// <param name="opMode"></param>
    /// <param name="acl"></param>
    /// <returns></returns>
    public bool Authenticate(PlaidSamApplication sam, KeysetList keysets, short opMode, out byte[] acl)
    {
      byte[] estr1 = null;
      byte[] estr2 = null;
      byte[] estr3 = null;

      // Initial Authenticate
      PreciseTimer t = new PreciseTimer();

      t.Reset();
      //SelectApplication();
      InitialAuthenticate(keysets, out estr1);
      double a = t.Milliseconds;

      t.Reset();
      //sam.SelectApplication();
      sam.InitialAuthenticate(keysets, estr1, opMode, out estr2);
      double b = t.Milliseconds;

      // Final Authenticate
      t.Reset();
      //SelectApplication();
      FinalAuthenticate(estr2, out estr3);
      double c = t.Milliseconds;

      //sam.SelectApplication();
      bool result = sam.FinalAuthenticate(estr3, out acl);
      double d = t.Milliseconds;

      //Console.WriteLine("{0}\t{1}\t{2}\t{3}", a, b, c, d);
      return result;
    }

    public void InitialAuthenticate(KeysetList keysets, out byte[] estr1)
    {
      Iso7816Protocol protocol = Card.GetProtocol<Iso7816Protocol>();

      CApdu command = new CApdu();
      command.Cla = CLA;
      command.Ins = INS_INITIAL_AUTHENTICATE;
      command.P1 = P1;
      command.P2 = P2;
      command.Data = keysets.DerEncode();

      RApdu response = protocol.Transceive(command);

      // Validate the response,
      if (response.IsError)
      {
        throw response.ThrowBySW("InitialAuthenticate");
      }

      estr1 = response.Data;
      return;
    }

    public void FinalAuthenticate(byte[] estr2, out byte[] estr3)
    {
      Iso7816Protocol protocol = Card.GetProtocol<Iso7816Protocol>();

      CApdu command = new CApdu();
      command.Cla = CLA;
      command.Ins = INS_FINAL_AUTHENTICATE;
      command.P1 = P1;
      command.P2 = P2;
      command.Data = estr2;

      RApdu response = protocol.Transceive(command);

      // Validate the response,
      if (response.IsError)
      {
        throw response.ThrowBySW("FinalAuthenticate");
      }

      estr3 = response.Data;
      return;
    }

    public void SecurePlaidData()
    {
      Iso7816Protocol protocol = Card.GetProtocol<Iso7816Protocol>();

      // Generate the DER encoded data
      MemoryStream stream = new MemoryStream();
      DerSequenceGenerator sequence = new DerSequenceGenerator(stream);
      sequence.AddObject(new DerOctetString(new byte[] { TagSecureICC }));
      sequence.Close();

      // Construct the APDU
      CApdu command = new CApdu();
      command.Cla = CLA;
      command.Ins = INS_SETDATA;
      command.P1 = P1;
      command.P2 = P2;
      command.Data = stream.ToArray();
      stream.Close();
      RApdu response = protocol.Transceive(command);

      // Validate the response,
      if (response.IsError)
      {
        throw response.ThrowBySW("SecurePlaidData");
      }

      return;
    }

    public void SetAcsRecord(AcsRecord record)
    {
      Iso7816Protocol protocol = Card.GetProtocol<Iso7816Protocol>();

      // Generate the DER encoded data
      MemoryStream stream = new MemoryStream();
      DerSequenceGenerator sequence = new DerSequenceGenerator(stream);
      sequence.AddObject(new DerOctetString(new byte[] { TagAcsRecord }));
      sequence.AddObject(new DerOctetString(record.DerEncode()));
      sequence.Close();

      // Construct the APDU
      CApdu command = new CApdu();
      command.Cla = CLA;
      command.Ins = INS_SETDATA;
      command.P1 = P1;
      command.P2 = P2;
      command.Data = stream.ToArray();
      stream.Close();
      RApdu response = protocol.Transceive(command);

      // Validate the response,
      if (response.IsError)
      {
        throw response.ThrowBySW("SetAcsRecord");
      }

      return;
    }

    public void SetPlaidData(byte tag, byte[] data)
    {
      Iso7816Protocol protocol = Card.GetProtocol<Iso7816Protocol>();

      // Generate the DER encoded data
      MemoryStream stream = new MemoryStream();
      DerSequenceGenerator sequence = new DerSequenceGenerator(stream);
      sequence.AddObject(new DerOctetString(new byte[] { tag }));
      sequence.AddObject(new DerOctetString(data));
      sequence.Close();

      // Construct the APDU
      CApdu command = new CApdu();
      command.Cla = CLA;
      command.Ins = INS_SETDATA;
      command.P1 = P1;
      command.P2 = P2;
      command.Data = stream.ToArray();
      stream.Close();
      RApdu response = protocol.Transceive(command);

      // Validate the response,
      if (response.IsError)
      {
        throw response.ThrowBySW("SetPlaidData");
      }

      return;
    }


    public void SetPlaidKeysetData(byte tag, short keyset, byte[] data)
    {
      Iso7816Protocol protocol = Card.GetProtocol<Iso7816Protocol>();

      // Generate the DER encoded data
      MemoryStream stream = new MemoryStream();
      DerSequenceGenerator sequence = new DerSequenceGenerator(stream);
      sequence.AddObject(new DerOctetString(new byte[] { tag }));
      sequence.AddObject(new DerOctetString(Converters.IntToArray(keyset)));
      sequence.AddObject(new DerOctetString(data));
      sequence.Close();

      // Construct the APDU
      CApdu command = new CApdu();
      command.Cla = CLA;
      command.Ins = INS_SETDATA;
      command.P1 = P1;
      command.P2 = P2;
      command.Data = stream.ToArray();
      stream.Close();
      RApdu response = protocol.Transceive(command);

      // Validate the response,
      if (response.IsError)
      {
        throw response.ThrowBySW("SetPLaidKeysetData");
      }

      return;
    }

    public void Personalise(PlaidPersonalisationRecord record, bool secure)
    {
      SetPlaidData(TagDiversificationData, record.DivData);

      foreach (AcsRecord r in record.AcsRecords)
      {
        SetAcsRecord(r);
      }

      if (record.Pin != null) SetPlaidData(TagPin, record.Pin);
      if (record.Minutiae != null) SetPlaidData(TagMinutiae, record.Minutiae);

      foreach (PlaidKey k in record.Keys)
      {
        SetPlaidKeysetData(TagIAKey, k.KeysetId, k.IAKey.Modulus);
        SetPlaidKeysetData(TagFAKey, k.KeysetId, k.FAKey);
      }

      if (secure) SecurePlaidData();
    }

    #endregion

		#region "Members - Private / Protected"

		#endregion
	
		#region "Constants / Private Declarations"

    /// <summary>
    /// The PLAID Application Identifier (AID)
    /// </summary>
    public static readonly byte[] AID = { 0xA0, 0x00, 0x00, 0x04, 0x64 };

    /// <summary>
    /// The PLAID Package Identifier
    /// </summary>
    public static readonly byte[] PID = { 0xA0, 0x00, 0x00, 0x04, 0x64, 0x00, 0x61, 0x66, 0x67, 0x6D };

    /// <summary>
    /// Static CLA value for all commands
    /// </summary>
    public const byte CLA = 0x80; // Static CLA value for all commands

    /// <summary>
    /// Static P1 value for all commands
    /// </summary>
    public const byte P1 = 0x00; // Unused in PLAID

    /// <summary>
    /// Static P2 value for all commands
    /// </summary>
    public const byte P2 = 0x00; // Unused in PLAID

    // INS values
    public const byte INS_INITIAL_AUTHENTICATE = 0x8A;
    public const byte INS_FINAL_AUTHENTICATE = 0x8C;
    public const byte INS_SETDATA = 0xDB;
    public const byte INS_GETDATA = 0xCB;


    // ASN-1 static constants
    public const byte Asn1OctetString = 0x04;
    public const byte Asn1Sequence = 0x30;

    // Tag constants
    public const byte TagDiversificationData = 0x01;
    public const byte TagAcsRecord = 0x02;
    public const byte TagPin = 0x03;
    public const byte TagSecureICC = 0x04;
    public const byte TagVersionNo = 0x05;
    public const byte TagMinutiae = 0x06;
    public const byte TagIAKey = 0x07;
    public const byte TagFAKey = 0x08;
    public const byte TagReinitializeCard = 0x0A;

    // Length Constants
    public const int DivDataLength = 8;
    #endregion
  }
}
#endregion