﻿#region "Copyright"
/******************************************************************************************
 Copyright (C) 2012 Kim O'Sullivan (kim@makina.com.au)

 Permission is hereby granted, free of charge, to any person obtaining a copy of 
 this software and associated documentation files (the "Software"), to deal in the 
 Software without restriction, including without limitation the rights to use, copy, 
 modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, 
 and to permit persons to whom the Software is furnished to do so, subject to the 
 following conditions:
 
 The above copyright notice and this permission notice shall be included in all copies 
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE 
 LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
 OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
 OTHER DEALINGS IN THE SOFTWARE.
******************************************************************************************/
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;
using System.IO;

namespace CardFramework.Plaid
{
  public class PlaidPicc
  {
    /*
     * Constructors
     */
    public PlaidPicc()
    {
      Random r = new Random();

      // Initialise our crypto service providers
      myRSA = new RSACryptoServiceProvider(1024);
      myAES = new AesCryptoServiceProvider();
      mySHA1 = new SHA1CryptoServiceProvider();

      // Generate some basic keys
      IAKey = myRSA.ExportParameters(false);
      myAES.GenerateKey();
      FAKey = myAES.Key;      

      // Initialise some dummy data for our application variables
      AcsRecord = new byte[16]; r.NextBytes(AcsRecord);
      DivDat = new byte[8]; r.NextBytes(DivDat);
      Pin = new byte[8]; r.NextBytes(Pin);
      Minutiae = new byte[32];  r.NextBytes(Minutiae);
    }

    public PlaidPicc(RSA rsa)
    {
      myRSA = rsa;
    }

    /*
     * PLAID command set
     */
    public byte[] InitialAuthenticate(byte opMode, byte keyset)
    {
      /*
       * 7 (b) (i)
       * The ICC parses the DER listing of KeySetIDs and retrieves the first 
       * IAKey found which matches a KeySetID supported by the ICC.
       */

      /*
       * 7 (b) (ii)
       * If none of the KeySetIDs identified match a key stored by the ICC then the ICC 
       * responds using a random byte string encrypted with the ShillKey, thereby 
       * preventing any indication that an error has occurred.
       */

      /*
       * 7 (c) (i)
       * The ICC generates a random value (RND1) using a TRNG. The size of RND1 is 
       * identical to the key size of the selected AES cipher (16, 24 or 32 bytes).
       * 
       * NOTE: This implementation does not use a TRNG
       */

      /*
       * 7 (c) (ii)
       * The ICC retrieves the unique diversification data DivData.
       */

      /*
       * 7 (c) (iii)
       * The ICC creates the bit string STR1: KeySetID || DivData || RND1 || RND1.
       */

      /*
       * 7 (c) (iv)
       * The ICC computes the bit string ESTR1 where ESTR1 = RSAEncrypt IAKey (STR1).
       * This encryption only uses the modulus and public exponent of the IAKey.
       * Padding (PKCS1.) should be incorporated into the encryption.
       */

      /*
       * 7 (c) (v)
       * The ICC transmits the ESTR1 string to the IFD.
       */

      return null;
    }

    public byte[] InitialAuthenticateAdmin()
    {
      return null;
    }

    public byte[] FinalAuthenticate()
    {

      /*
       * 7 (f) (i)
       * The ICC calculates STR2 where STR2 = AESDecrypt FAKey (Div) (ESTR2). 
       * The FAKey (Div) to be used is referenced by the KeySetID used in the IA Response.
       */

      /*
       * 7 (f) (ii)
       * The ICC calculates RND3 as SHA [RND1||RND2] using RND1 generated in the 
       * FA Command step and RND2 extracted from STR2.
       */

      /*
       * 7 (f) (iii)
       * The ICC compares RND3 above with the RND3 extracted from STR2. 
       * If a mismatch occurs then the ICC responds using a random byte string encrypted 
       * with the ShillKey, thereby preventing any indication that an error has occurred. 
       */

      /*
       * 7 (g) (i)
       * The ICC retrieves the appropriate fields, based on the OpModeID extracted from 
       * STR2.
       */

      /*
       * 7 (g) (ii)
       * The ICC creates the bit string STR3: DivData || ACSRecord || (Null, PINHash 
       * and/or minutiae).
       */

      /*
       * 7 (g) (iii)
       * The ICC calculates ESTR3 where ESTR3 = AESEncrypt RND3 (STR3). The cipher mode 
       * for this operation must be CBC.
       */

      /*
       * 7 (g) (iv)
       * The ICC transmits the Final Authenticate string ESTR3 to the IFD.
       */

      return null;
    }

    public void SetData()
    {

    }

    public byte[] GetData(PlaidGetDataMode mode)
    {
      switch (mode)
      {
        case PlaidGetDataMode.AcsRecord:
          return AcsRecord;
        case PlaidGetDataMode.PINhash:
          return GeneratePinHash();
        case PlaidGetDataMode.VersionNo:
          return ToArray(Version);
        case PlaidGetDataMode.Minutiae:
          return Minutiae;
        default:
          throw new Exception(PlaidError.SwInsNotSupported.ToString());
      };
    }

    /*
     * Private helper methods
     */

    private byte[] Diversify(byte[] divdat)
    {
      byte[] iv = new byte[8];
      ICryptoTransform en = myAES.CreateEncryptor(FAKey, iv);

      byte[] output = new byte[8];
      en.TransformBlock(divdat, 0, divdat.Length, output, 0);
      en.TransformBlock(output, 0, output.Length, output, 0);
      en.TransformBlock(output, 0, output.Length, output, 0);
      en.TransformBlock(output, 0, output.Length, output, 0);

      return output;
    }

    private byte[] GenerateSessionKey(byte[] rnd1, byte[] rnd2)
    {
      byte[] rnd3 = new byte[32];

      for (int i = 0; i < 32; i++)
      {
        rnd3[i] = (byte)(rnd1[i] ^ rnd2[i]);
      }

      return rnd3;
    }

    private byte[] GeneratePinHash()
    {
      return mySHA1.ComputeHash(Pin);
    }

    private byte[] ToArray(byte value)
    {
      //var parser = new BinaryParser(ByteEndianess.LittleEndian);
      //parser.WriteUInt8(value);
      //return parser.ToArray();
      return null;
    }

    private byte[] ToArray(ushort value)
    {
      //var parser = new BinaryParser(ByteEndianess.LittleEndian);
      //parser.WriteUInt16(value);
      //return parser.ToArray();
      return null;
    }

    /*
     * Application variables
     */

    // Crypto service providers
    private RSA myRSA;
    private Aes myAES;
    private SHA1 mySHA1;

    // PLAID Application data
    private byte[] AcsRecord;
    private byte[] DivDat;
    private byte[] Pin;
    private byte[] Minutiae;

    private byte[] FAKey;
    private RSAParameters IAKey;

    // The allowed operation mode id (currently static)
    private const byte OperationMode = 1;

    // The allowed keyset version (currently static)
    private const byte Keyset = 1;

    private const byte Version = 1;
  }




  /*
   * Class enumeration definitions
   */

  public enum PlaidSetDataMode
  {
    DivDat = 0x01,
    AcsRecord = 0x02,
    PIN = 0x03,
    SecureICC = 0x04,
    Minutiae = 0x06,
    IAKey = 0x07,
    FAKey = 0x08,
    LTcount = 0x0A
  }

  public enum PlaidGetDataMode
  {
    AcsRecord = 0x02,
    PINhash = 0x03,
    VersionNo = 0x05,
    Minutiae = 0x06,
    LTcount = 0x0A,
    LTcountAdm = 0x0B,
    LUcount = 0x0C,
    LUcountAdm = 0x0D
  }

  public enum AuthState
  {
    PlaidSecured,
    PlaidUnsecured,
    FinalAuth,
    InitialAuth,
  }

  public enum PlaidError
  {
    SwWrongLength = 0x6700,
    SwCommandNotAllowed = 0x6986,
    SwInsNotSupported = 0x6D00,
    PlaidLocked = 0x69F1,
    PlaidTerminated = 0x69FD,
    SwOutOfKeyStores = 0x69F7,
  }
}
