﻿#region "Copyright"
/******************************************************************************************
 Copyright (C) 2012 Kim O'Sullivan (kim@makina.com.au)

 Permission is hereby granted, free of charge, to any person obtaining a copy of 
 this software and associated documentation files (the "Software"), to deal in the 
 Software without restriction, including without limitation the rights to use, copy, 
 modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, 
 and to permit persons to whom the Software is furnished to do so, subject to the 
 following conditions:
 
 The above copyright notice and this permission notice shall be included in all copies 
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE 
 LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
 OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
 OTHER DEALINGS IN THE SOFTWARE.
******************************************************************************************/
#endregion

#region "Namespace definitions"
using System;
using System.Collections.Generic;
using System.Text;
using CardFramework.Protocols.Iso7816;
using CardFramework.Applications.Iso7816;
using CardFramework.Helpers;
using CardFramework.Formats;
using System.IO;
using System.Security.Cryptography;
using Org.BouncyCastle.Asn1;
#endregion

#region "Class definitions"
namespace CardFramework.Applications.Plaid
{
  public class PlaidSamApplication : Iso7816Application
  {
    #region "Members - Public"

    public override bool Discover(Card card)
    {
      // Create a backup of the reference to our old card
      Card oldCard = this.Card;

      // Temporarily set the card
      this.Card = card;

      try
      {
        SelectApplication();
        return true;
      }
      catch (Exception)
      {
        return false;
      }
      finally
      {
        this.Card = oldCard;
      }
    }

    public void SelectApplication()
    {
      try
      {
        // First, attempt to select the registered AID
        IsReadOnly = false;
        SelectFile(AID, SelectMode.DfName);
        IsReadOnly = true;
      }
      catch (Exception)
      {
        // Next, attempt to select by the 'Read Only' instance AID
        SelectFile(ReadOnlyAID, SelectMode.DfName);
        IsReadOnly = false;
      }
    }

    public void InitialAuthenticate(KeysetList keysets, byte[] estr1, short opMode, out byte[] estr2)
    {
      Iso7816Protocol protocol = Card.GetProtocol<Iso7816Protocol>();

      CApdu command = new CApdu();
      command.Cla = PlaidApplication.CLA;
      command.Ins = PlaidApplication.INS_INITIAL_AUTHENTICATE;
      command.P1 = PlaidApplication.P1;
      command.P2 = PlaidApplication.P2;

      // Copy the keysets and the estr1 value to a new buffer
      MemoryStream stream = new MemoryStream();
      DerSequenceGenerator sequence = new DerSequenceGenerator(stream);
      sequence.AddObject(new DerOctetString(estr1));
      sequence.AddObject(new DerOctetString(Converters.IntToArray(opMode)));
      sequence.AddObject(new DerOctetString(Converters.IntToArray((short)keysets.Count)));
      foreach (ushort k in keysets)
      {
        sequence.AddObject(new DerOctetString(Converters.IntToArray(k)));
      }      
      sequence.Close();
      command.Data = stream.ToArray();
      stream.Close();

      RApdu response = protocol.Transceive(command);

      // Validate the response,
      if (response.IsError)
      {
        throw response.ThrowBySW("InitialAuthenticate");
      }

      estr2 = response.Data;
      return;
    }
    
    public bool FinalAuthenticate(byte[] estr3, out byte[] acl)
    {
      Iso7816Protocol protocol = Card.GetProtocol<Iso7816Protocol>();

      CApdu command = new CApdu();
      command.Cla = PlaidApplication.CLA;
      command.Ins = PlaidApplication.INS_FINAL_AUTHENTICATE;
      command.P1 = PlaidApplication.P1;
      command.P2 = PlaidApplication.P2; 
      command.Data = estr3;

      RApdu response = protocol.Transceive(command);

      // Validate the response,
      if (response.IsError)
      {
        throw response.ThrowBySW("FinalAuthenticate");
      }

      acl = response.Data;
      return true;
    }

    public void Personalise(List<PlaidKey> keys, bool secure)
    {
      /*
       * We don't need to explicitly create a Keyset by ID on the SAM,
       * because as of 805 it will look at any incoming keyset id and create
       * a new entry if it requires it.
       */

      foreach (PlaidKey k in keys)
      {
        // P
        SetPlaidData(TagP, k.KeysetId, k.IAKey.P);

        // Q
        SetPlaidData(TagQ, k.KeysetId, k.IAKey.Q);

        // PQ
        SetPlaidData(TagPQ, k.KeysetId, k.IAKey.PQ);

        // DP1
        SetPlaidData(TagDP1, k.KeysetId, k.IAKey.DP);

        // DQ1
        SetPlaidData(TagDQ1, k.KeysetId, k.IAKey.DQ);

        // FAKey
        SetPlaidData(TagFaKey, k.KeysetId, k.FAKey);
      }

      if (secure) SecurePlaidData();
    }

    public void SecurePlaidData()
    {
      Iso7816Protocol protocol = Card.GetProtocol<Iso7816Protocol>();

      // Generate the DER encoded data
      MemoryStream stream = new MemoryStream();
      DerSequenceGenerator sequence = new DerSequenceGenerator(stream);
      sequence.AddObject(new DerOctetString(new byte[] { TagSecured }));
      sequence.Close();

      // Construct the APDU
      CApdu command = new CApdu();
      command.Cla = PlaidApplication.CLA;
      command.Ins = PlaidApplication.INS_SETDATA;
      command.P1 = PlaidApplication.P1;
      command.P2 = PlaidApplication.P2;
      command.Data = stream.ToArray();
      stream.Close();

      RApdu response = protocol.Transceive(command);

      // Validate the response,
      if (response.IsError)
      {
        throw response.ThrowBySW("SecurePlaidData");
      }
    }

    public void SetPlaidData(byte tag, short keysetId, byte[] data)
    {
      Iso7816Protocol protocol = Card.GetProtocol<Iso7816Protocol>();

      // Generate the DER encoded data
      MemoryStream stream = new MemoryStream();
      DerSequenceGenerator sequence = new DerSequenceGenerator(stream);
      sequence.AddObject(new DerOctetString(new byte[] { tag }));
      sequence.AddObject(new DerOctetString(Converters.IntToArray(keysetId)));
      sequence.AddObject(new DerOctetString(data));
      sequence.Close();

      // Construct the APDU
      CApdu command = new CApdu();
      command.Cla = PlaidApplication.CLA;
      command.Ins = PlaidApplication.INS_SETDATA;
      command.P1 = PlaidApplication.P1;
      command.P2 = PlaidApplication.P2;
      command.Data = stream.ToArray();
      stream.Close();

      RApdu response = protocol.Transceive(command);

      // Validate the response,
      if (response.IsError)
      {
        throw response.ThrowBySW("SetPlaidData");
      }
    }

    public bool IsReadOnly
    {
      get
      {
        return myIsReadOnly;
      }
      private set
      {
        myIsReadOnly = value;
      }
    }

    #endregion

    #region "Members - Private / Protected"

    #endregion

    #region "Constants / Private Declarations"

    /// <summary>
    /// The PLAID SAM Application Identifier (AID)
    /// </summary>
    public readonly byte[] AID = { 0xA0, 0x00, 0x77, 0x6B, 0x67, 0x66 };

    /// <summary>
    /// The PLAID SAM Application Identifier (AID) - Read Only Instance
    /// </summary>
    public readonly byte[] ReadOnlyAID = { 0xA0, 0x00, 0x77, 0x6B, 0x67, 0x60 };

    /// <summary>
    /// The PLAID SAM Package Identifier
    /// </summary>
    public readonly byte[] PID = { 0xA0, 0x00, 0x77, 0x6B, 0x67, 0x67 };

    public const byte TagP = 0x01;
    public const byte TagQ = 0x02;
    public const byte TagDP1 = 0x03;
    public const byte TagDQ1 = 0x04;
    public const byte TagPQ = 0x05;
    public const byte TagFaKey = 0x06;
    public const byte TagSecured = 0x0A;

    /// <summary>
    /// Read-only indicator flag
    /// </summary>
    private bool myIsReadOnly = true;

    #endregion
  }
}
#endregion