﻿#region "Copyright"
/******************************************************************************************
 Copyright (C) 2012 Kim O'Sullivan (kim@makina.com.au)

 Permission is hereby granted, free of charge, to any person obtaining a copy of 
 this software and associated documentation files (the "Software"), to deal in the 
 Software without restriction, including without limitation the rights to use, copy, 
 modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, 
 and to permit persons to whom the Software is furnished to do so, subject to the 
 following conditions:
 
 The above copyright notice and this permission notice shall be included in all copies 
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE 
 LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
 OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
 OTHER DEALINGS IN THE SOFTWARE.
******************************************************************************************/
#endregion

#region "Namespace definitions"
using System;
using System.Collections.Generic;
using System.Text;
using System.Security.Cryptography;
#endregion

#region "Class definitions"

namespace CardFramework.Helpers
{
	public static class Crypto
	{
		#region "Members - Public"

    #region "TDea (TripleDES) Methods"

    public static byte[] TDeaStripVersion(byte[] key)
    {
      byte[] unversioned = new byte[key.Length];
      for (int i = 0; i < key.Length; i++)
      {
        unversioned[i] = (byte)(key[i] & 0xFE);
      }

      // Done
      return unversioned;
    }

    public static byte[] TDeaSetVersion(byte[] key, byte version)
    {
      // First strip the version
      byte[] versioned = TDeaStripVersion(key);

      // Set version bits in the LEFT half of the key
      for (int i = 0; i < 8; i++)
      {
        byte versionBit = (byte)((version >> (7 - i)) & 0x01);
        versioned[i] |= versionBit;
      }

      // Done
      return versioned;
    }

    public static byte[] TDeaEncrypt(byte[] key, byte[] data)
    {
      return TDeaEncrypt(key, data, new byte[BlockLength]);
    }

    public static byte[] TDeaEncrypt(byte[] key, byte[] data, byte[] iv)
    {
      TripleDESCryptoServiceProvider tdea = new TripleDESCryptoServiceProvider();

      // Setup our encryptor
      tdea.Padding = PaddingMode.Zeros;
      tdea.Mode = CipherMode.CBC;

      ICryptoTransform tdeaEnc = tdea.CreateEncryptor(key, iv);

      // Perform the crypto operation
      byte[] result = tdeaEnc.TransformFinalBlock(data, 0, data.Length);

      // Clean up the service provider
      tdea.Clear();

      // Done
      return result;
    }

    public static byte[] TDeaDecrypt(byte[] key, byte[] data)
    {
      return TDeaDecrypt(key, data, new byte[BlockLength]);
    }

    public static byte[] TDeaDecrypt(byte[] key, byte[] data, byte[] iv)
    {
      TripleDESCryptoServiceProvider tdea = new TripleDESCryptoServiceProvider();

      // Setup our encryptor
      tdea.Padding = PaddingMode.Zeros;
      tdea.Mode = CipherMode.CBC;

      ICryptoTransform tdeaDec = tdea.CreateDecryptor(key, iv);

      // Perform the crypto operation
      byte[] result = tdeaDec.TransformFinalBlock(data, 0, data.Length);

      // Clean up the service provider
      tdea.Clear();

      // Done
      return result;
    }

    #endregion

    #region "Aes Methods"

    public static byte[] AesEncrypt(byte[] key, byte[] data)
    {
      return AesEncrypt(key, data, new byte[BlockLength]);
    }

    public static byte[] AesEncrypt(byte[] key, byte[] data, byte[] iv)
    {
      AesCryptoServiceProvider AES = new AesCryptoServiceProvider();

      // Setup our encryptor
      AES.Padding = PaddingMode.Zeros;
      AES.Mode = CipherMode.CBC;

      ICryptoTransform AESEnc = AES.CreateEncryptor(key, iv);

      // Perform the crypto operation
      byte[] result = AESEnc.TransformFinalBlock(data, 0, data.Length);

      // Clean up the service provider
      AES.Clear();

      // Done
      return result;
    }

    public static byte[] AesDecrypt(byte[] key, byte[] data)
    {
      return AesDecrypt(key, data, new byte[BlockLength]);
    }

    public static byte[] AesDecrypt(byte[] key, byte[] data, byte[] iv)
    {
      AesCryptoServiceProvider AES = new AesCryptoServiceProvider();

      // Setup our encryptor
      AES.Padding = PaddingMode.Zeros;
      AES.Mode = CipherMode.CBC;

      ICryptoTransform AESDec = AES.CreateDecryptor(key, iv);

      // Perform the crypto operation
      byte[] result = AESDec.TransformFinalBlock(data, 0, data.Length);

      // Clean up the service provider
      AES.Clear();

      // Done
      return result;
    }

    public static byte[] CreateRandomEntropy(int length)
    {
      if (myRandomCSP == null) myRandomCSP = new RNGCryptoServiceProvider();

      // Create a byte array to hold the random value.
      byte[] entropy = new byte[length];

      // Create a new instance of the RNGCryptoServiceProvider.
      // Fill the array with a random value.
      myRandomCSP.GetBytes(entropy);

      // Return the array.
      return entropy;
    }

    /// <summary>
    /// Performs an XOR operation on two byte arrays, returning the result.
    /// </summary>
    /// <param name="a"></param>
    /// <param name="b"></param>
    /// <returns></returns>
    public static byte[] XorArray(byte[] a, byte[] b) {
      // Make sure that the two arrays are the same length
      if (a.Length != b.Length) throw new InvalidOperationException("Both arrays must be of equal length");

      byte[] result = new byte[a.Length];
      for (int i = 0; i < a.Length; i++)
      {
        result[i] = (byte)(a[i] ^ b[i]);
      }
      return result;
    }

    #endregion

    #endregion

    #region "Members - Private / Protected"

    private static RNGCryptoServiceProvider myRandomCSP = null;

    #endregion

    #region "Constants / Private Declarations"
    public const int BlockLength = 8;
    #endregion
	}
}

#endregion