﻿#region "Copyright"
/******************************************************************************************
 Copyright (C) 2012 Kim O'Sullivan (kim@makina.com.au)

 Permission is hereby granted, free of charge, to any person obtaining a copy of 
 this software and associated documentation files (the "Software"), to deal in the 
 Software without restriction, including without limitation the rights to use, copy, 
 modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, 
 and to permit persons to whom the Software is furnished to do so, subject to the 
 following conditions:
 
 The above copyright notice and this permission notice shall be included in all copies 
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE 
 LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
 OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
 OTHER DEALINGS IN THE SOFTWARE.
******************************************************************************************/
#endregion

#region "Namespace definitions"
using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Globalization;
#endregion

#region "Class definitions"
namespace CardFramework.Protocols.Iso7816
{
	public class RApdu
	{
		#region "Members - Public"

    static RApdu()
    {
      BuildErrorCodes();
    }

    /// <summary>
    /// Initializes a new instance of the RAPDU class.
    /// </summary>
    public RApdu()
    {
    }

    public static RApdu Parse(byte[] apdu)
    {
      RApdu result = new RApdu();

      if (apdu.Length < 2)
      {
        throw new ArgumentException(@"Invalid R-APDU length", @"apdu");
      }

      // Parse SW2/1 & Data
      result.SW1 = apdu[apdu.Length - 2];
      result.SW2 = apdu[apdu.Length - 1];

      if (apdu.Length > 2)
      {
        result.Data = new byte[apdu.Length - 2];
        Array.Copy(apdu, result.Data, result.Data.Length);
      }
      else
      {
        result.Data = new byte[0];
      }

      return result;
    }

    /// <summary>
    /// Initializes a new instance of the RAPDU class.
    /// </summary>
    /// <param name="SW1">The SW1 flag.</param>
    /// <param name="SW2">The SW2.</param>
    public RApdu(byte sw1, byte sw2)
    {
      Data = new byte[0];
      SW1 = sw1;
      SW2 = sw2;
    }

    public byte SW1 { get; set; }
    public byte SW2 { get; set; }

    public ushort SW12
    {
      get
      {
        return (ushort)(((ushort)SW1 << 8) | (ushort)SW2);
      
      }
    }

    public byte[] Data { get; set; }

    /// <summary>
    /// Returns this R-APDU instance in the form of a byte array.
    /// </summary>
    /// <returns>The array.</returns>
    public byte[] ToArray()
    {
      MemoryStream stream = new MemoryStream(Data.Length + 2);

      stream.Write(Data, 0, Data.Length);
      stream.WriteByte(SW1);
      stream.WriteByte(SW2);

      return stream.ToArray();
    }

    public override string ToString()
    {
      StringBuilder rapdu = new StringBuilder();

      foreach (byte b in Data)
      {
        rapdu.Append(string.Format(CultureInfo.InvariantCulture, "{0:X2}", b));
      }
      rapdu.Append(" ");
      rapdu.Append(string.Format(CultureInfo.InvariantCulture, "{0:X2}", SW1));
      rapdu.Append(string.Format(CultureInfo.InvariantCulture, "{0:X2}", SW2));

      return rapdu.ToString();
    }

    public string ToString(bool status)
    {
      StringBuilder rapdu = new StringBuilder();

      foreach (byte b in Data)
      {
        rapdu.Append(string.Format(CultureInfo.InvariantCulture, "{0:X2}", b));
      }
      if (status)
      {
        rapdu.Append(" ");
        rapdu.Append(string.Format(CultureInfo.InvariantCulture, "{0:X2}", SW1));
        rapdu.Append(string.Format(CultureInfo.InvariantCulture, "{0:X2}", SW2));
      }

      return rapdu.ToString();
    }

    /// <summary>
    /// Gets a value indicating whether this instance is an error.
    /// </summary>
    /// <value><c>true</c> if this instance is an error; otherwise, <c>false</c>.</value>
    public bool IsError
    {
      get
      {
        return !((0x90 == SW1) && (0x00 == SW2));
      }
    }

    public Exception ThrowBySW(string source)
    {
      if (myErrors.ContainsKey(SW12))
      {
        string msg = string.Format("{0}: {1} (SW={2:X4})", source, myErrors[SW12], SW12);
        return new ApplicationException(msg);
      }
      else
      {
        string msg = string.Format("{0}: Unspecified error (SW={1:X4})", source, SW12);
        return new ApplicationException(msg);
      }
    }

    #endregion

		#region "Members - Private / Protected"

    private static void BuildErrorCodes()
    {
      myErrors.Add(0x9000, "No error");
      myErrors.Add(0x6200, "Warning - No information");
      myErrors.Add(0x6281, "Warning - Part of returned data may be corrupted");
      myErrors.Add(0x6282, "Warning - End of file or record reached before reading N(e) bytes");
      myErrors.Add(0x6283, "Warning - Selected file deactivated");
      myErrors.Add(0x6284, "Warning - File control information not formatted according to 5.3.3");
      myErrors.Add(0x6285, "Warning - Selected file in termination state");
      myErrors.Add(0x6286, "Warning - No input data available from a sensor on the card");
      myErrors.Add(0x6300, "Warning - No information");
      myErrors.Add(0x6381, "Warning - File filled up by the last write");
      myErrors.Add(0x6400, "Error - Execution error");
      myErrors.Add(0x6401, "Error - Immediate response requiretd by the card");
      myErrors.Add(0x6500, "Error - No information given");
      myErrors.Add(0x6581, "Error - Memory failure");
      myErrors.Add(0x6800, "Error - No information given");
      myErrors.Add(0x6881, "Error - Logical channel not supported");
      myErrors.Add(0x6882, "Error - Secure messaging not supported");
      myErrors.Add(0x6883, "Error - Last command of the chain expected");
      myErrors.Add(0x6884, "Error - Command chaining not supported");
      myErrors.Add(0x6900, "Error - No information given");
      myErrors.Add(0x6981, "Error - Command incompatible with file structure");
      myErrors.Add(0x6982, "Error - Security status not satisfied");
      myErrors.Add(0x6983, "Error - Authentication method blocked");
      myErrors.Add(0x6984, "Error - Reference data not usable");
      myErrors.Add(0x6985, "Error - Conditions of use not satisfied");
      myErrors.Add(0x6986, "Error - Command not allowed");
      myErrors.Add(0x6987, "Error - Expected secure messaging data objects missing");
      myErrors.Add(0x6988, "Error - Incorrect secure messaging data objects");
      myErrors.Add(0x6A00, "Error - No information given");
      myErrors.Add(0x6A80, "Error - Incorrect parameters in the command data field");
      myErrors.Add(0x6A81, "Error - Function not supported");
      myErrors.Add(0x6A82, "Error - File or application not found");
      myErrors.Add(0x6A83, "Error - Record not found");
      myErrors.Add(0x6A84, "Error - Not enough memory space in the file");
      myErrors.Add(0x6A85, "Error - N(c) inconsistent with TLV structure");
      myErrors.Add(0x6A86, "Error - Incorrect parameters P1-P2");
      myErrors.Add(0x6A87, "Error - N(c) inconsistent with parameters P1-P2");
      myErrors.Add(0x6A88, "Error - Referenced data or reference data not found");
      myErrors.Add(0x6A89, "Error - File already exists");
      myErrors.Add(0x6A8A, "Error - DF name already exists");
    }

    protected static Dictionary<ushort, string> myErrors = new Dictionary<ushort, string>();
    
    #endregion
	
		#region "Constants / Private Declarations"

    public const byte SW1Ok = 0x90;
    public const byte SW1MoreData = 0x61;
    public const byte SW1FormatError = 0x62;

    #endregion
	}
}
#endregion