﻿
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CardFramework.Readers.PCSC;
using CardFramework.Applications.GlobalPlatform;
using CardFramework.Applications.Fips201;
using CardFramework;
using CardFramework.Protocols.Iso7816;
using System.Xml.Serialization;
using CardFramework.Applications.Plaid;
using PACManager.Properties;
using Org.BouncyCastle.Asn1;
using CardFramework.Protocols.Mifare;
using CardFramework.Helpers;
using System.Diagnostics;
using System.IO;

namespace PACManager
{
  public enum CardProcessorEventLevel
  {
    Info,
    Verbose,
    Progress,
    CardState,
    Error
  }

  public enum CardProcessorResult
  {
    None,
    Succeeded,
    Failed,
    Skipped,
    Stopped
  }

  public enum CardProcessorMode
  {
    Pause = 0,
    Encode,
    Verify
  }

  public class CardProcessorEventArgs : EventArgs
  {
    public CardProcessorEventArgs()
    {
    }

    public CardProcessorEventArgs(string message, CardProcessorEventLevel level, CardProcessorResult result)
    {
      Message = message;
      Level = level;
      Result = result;
    }

    public string Message { get; set; }
    public CardProcessorEventLevel Level { get; set; }
    public CardProcessorResult Result { get; set; }
    public int Progress { get; set; }
  }

  public class CardProcessor : IDisposable
  {
    public CardProcessor()
    {

    }

    ~CardProcessor()
    {
      Dispose();
    }

    public void Dispose()
    {
      SetMode(CardProcessorMode.Pause);
      Disconnect();
    }

    private CardProcessorMode myMode;

    public void SetMode(CardProcessorMode mode)
    {
      // Make sure we are actually changing something
      if (myMode == mode) return;

      if (mode == CardProcessorMode.Pause)
      {
        ClearCryptographicData();
        myMode = mode;
        return;
      }
      else
      {
        if (!myPCSC.Contains(TargetReader)) 
        {
          RaiseError("Target reader is not connected, stopping", CardProcessorResult.Stopped);
          return;
        }
      }

      if (mode == CardProcessorMode.Encode)
      {
        try
        {
          RaiseInfo("Loading the key values");
          LoadCryptographicData();
          RaiseInfo("Ready to encode ...");
        }
        catch (Exception ex)
        {
          RaiseError("Failed to load cryptographic data, stopping", CardProcessorResult.None);
          RaiseError(ex.Message, CardProcessorResult.Stopped);
          return;
        }
      }
      else
      {
        RaiseInfo("Ready to verify ...");
      }

      myMode = mode;
    }

    public CardProcessorMode Mode
    {
      get { return myMode; }
    }

    public void Connect()
    {
      // If we are already running, just switch the mode and return
      if (myPCSC != null) return;

      // Connect to the PCSC manager
      myPCSC = new PCSCManager();

      myPCSC.OnReaderInserted += new EventHandler<PCSCReaderEventArgs>(myPCSC_OnReaderInserted);
      myPCSC.OnReaderRemoved += new EventHandler<PCSCReaderEventArgs>(myPCSC_OnReaderRemoved);
      myPCSC.OnCardInserted += new EventHandler<CardEventArgs>(myPCSC_OnCardInserted);
      myPCSC.OnCardRemoved += new EventHandler<CardEventArgs>(myPCSC_OnCardRemoved);

      myPCSC.Connect();
      myPCSC.Start();
    }

    public void Disconnect()
    {
      if (myPCSC == null) return;

      // Pause any processing
      SetMode(CardProcessorMode.Pause);

      // Disconnect
      myPCSC.Disconnect();

      myPCSC.OnReaderInserted -= new EventHandler<PCSCReaderEventArgs>(myPCSC_OnReaderInserted);
      myPCSC.OnReaderRemoved -= new EventHandler<PCSCReaderEventArgs>(myPCSC_OnReaderRemoved);
      myPCSC.OnCardInserted -= new EventHandler<CardEventArgs>(myPCSC_OnCardInserted);
      myPCSC.OnCardRemoved -= new EventHandler<CardEventArgs>(myPCSC_OnCardRemoved);

      myPCSC = null;
    }
      
    void myPCSC_OnCardRemoved(object sender, CardEventArgs e)
    {
      if (myMode == CardProcessorMode.Pause) return;

      if (e.Reader == this.KeyAuthenticatorReader ||
          e.Reader == this.KeyStorageReader)
      {
        ClearCryptographicData();       
        RaiseError("A key storage device was removed, stopping.", CardProcessorResult.Stopped);
      }

      if (e.Reader == this.TargetReader)
      {
        RaiseProgress(0);
        RaiseCardState("Card removed, waiting for the next target ...");
      }
    }

    void myPCSC_OnCardInserted(object sender, CardEventArgs e)
    {
      // Ignore events if we are paused
      if (myMode == CardProcessorMode.Pause) return;

      if (e.Reader == this.TargetReader)
      {
        PCSCCard card = (PCSCCard)e.Card;

        // Encode mode
        if (myMode == CardProcessorMode.Encode)
        {
          try
          {
            RaiseCardState("Card inserted ...");

            card.Connect();
            card.BeginTransaction();

            RaiseCardState("Encoding card ...");
            RaiseInfo("Card found: Encoding");
            EncodeCard();
            Raise("Card encoding complete.", CardProcessorResult.Succeeded);
          }
          catch (Exception ex)
          {
            if (null != card) card.Disconnect();
            RaiseError("Error encoding the card.", CardProcessorResult.Failed);
            RaiseError(ex.Message, CardProcessorResult.None);
            return;
          }
        } 

        // Verify Mode
        else if (myMode == CardProcessorMode.Verify)
        {
          try
          {
            RaiseCardState("Card inserted ...");
            card.Connect();
            card.BeginTransaction();

            RaiseInfo("Card found: Verifying");
            RaiseCardState("Reading card ...");
            VerifyCard();
            Raise("Card verification complete.", CardProcessorResult.None);
          }
          catch (Exception ex)
          {
            if (null != card) card.Disconnect();
            RaiseError("Error verifying the card.", CardProcessorResult.None);
            RaiseError(ex.Message, CardProcessorResult.None);
          }
        }
        else
        {
          RaiseError("INVALID MODE", CardProcessorResult.None);
        }

        RaiseCardState("Waiting for removal ...");
      }
    }

    void myPCSC_OnReaderRemoved(object sender, PCSCReaderEventArgs e)
    {
      if (myMode == CardProcessorMode.Pause) return;

      // Is this one of our readers?
      if (e.Reader == this.KeyStorageReader || 
          e.Reader == this.KeyAuthenticatorReader ||
          e.Reader == this.TargetReader)
      {
        RaiseError("One of the readers in use has been removed, aborting.", CardProcessorResult.Stopped);
        SetMode(CardProcessorMode.Pause);
      }
    }

    void myPCSC_OnReaderInserted(object sender, PCSCReaderEventArgs e)
    {
    }

    private bool ValidateReader(string reader)
    {
      if (string.IsNullOrEmpty(reader))
      {
        string msg = string.Format(@"Invalid reader: '{0}'", reader);
        RaiseVerbose(msg);
        return false;
      }

      if (!myPCSC.Contains(reader))
      {
        string msg = string.Format(@"Reader not found: '{0}'", reader);
        RaiseVerbose(msg);
        return false;
      }

      if (!myPCSC[reader].IsCardPresent)
      {
        string msg = string.Format(@"No card found: '{0}'", reader);
        RaiseVerbose(msg);
        return false;
      }

      return true;
    }

    private void LoadCryptographicData()
    {
      // Validate the reader settings
      if (!ValidateReader(KeyAuthenticatorReader))
      {
        throw new ApplicationException("Operation cannot continue without a valid KAM, aborting ...");
      }
      if (!ValidateReader(KeyStorageReader))
      {
        throw new ApplicationException("Operation cannot continue without a valid KSM, aborting ...");
      }

      PCSCCard KAM = myPCSC[KeyAuthenticatorReader].ActiveCard;
      PCSCCard KSM = myPCSC[KeyStorageReader].ActiveCard;

      // Authenticate to the key storage device
      PlaidSamApplication samApp = new PlaidSamApplication();
      PlaidApplication plaidApp = new PlaidApplication();

      KeysetList keysets = new KeysetList();
      keysets.Add(Settings.Default.Keyset0);
      byte[] acl;

      mySecurityContext = new SecurityContext();

      // Connect to both cards
      KAM.Connect();
      KAM.BeginTransaction();
      samApp.SetCard(KAM);
      samApp.SelectApplication();

      KSM.Connect();
      KSM.BeginTransaction();
      plaidApp.SetCard(KSM);
      plaidApp.SelectApplication();

      // Retrieve the Admin IA key
      try
      {
        RaiseVerbose("- Retrieving Admin - IA Key");
        plaidApp.Authenticate(samApp, keysets, Settings.Default.OpModeKSIAAdmin, out acl);

        // Extract the value from the ASN1 DER encoded packet
        var asn1 = new Asn1InputStream(acl);
        Asn1Sequence sequence = (Asn1Sequence)asn1.ReadObject();
        DerOctetString der = (DerOctetString)sequence[1];

        mySecurityContext.AdminIAKey = der.GetOctets();
      }
      catch (Exception ex)
      {
        ClearCryptographicData();
        throw new ApplicationException("Error retrieving the Target IA key from the Key Storage container", ex);
      }

      // Retrieve the Admin FA key
      try
      {
        RaiseVerbose("- Retrieving Admin - FA Key");
        plaidApp.Authenticate(samApp, keysets, Settings.Default.OpModeKSFAAdmin, out acl);

        // Extract the value from the ASN1 DER encoded packet
        var asn1 = new Asn1InputStream(acl);
        Asn1Sequence sequence = (Asn1Sequence)asn1.ReadObject();
        DerOctetString der = (DerOctetString)sequence[1];

        mySecurityContext.AdminFAKey = der.GetOctets();
      }
      catch (Exception ex)
      {
        ClearCryptographicData();
        throw new ApplicationException("Error retrieving the Target FA key from the Key Storage container", ex);
      }

      // Retrieve the Target IA key
      try
      {
        RaiseVerbose("- Retrieving Target - IA Key");
        plaidApp.Authenticate(samApp, keysets, Settings.Default.OpModeKSIATarget, out acl);

        // Extract the value from the ASN1 DER encoded packet
        var asn1 = new Asn1InputStream(acl);
        Asn1Sequence sequence = (Asn1Sequence)asn1.ReadObject();
        DerOctetString der = (DerOctetString)sequence[1];

        mySecurityContext.TargetIAKey = der.GetOctets();
      }
      catch (Exception ex)
      {
        ClearCryptographicData();
        throw new ApplicationException("Error retrieving the Target IA key from the Key Storage container", ex);
      }


      // Retrieve the Target FA key
      try
      {
        RaiseVerbose("- Retrieving Target - FA Key");
        plaidApp.Authenticate(samApp, keysets, Settings.Default.OpModeKSFATarget, out acl);

        // Extract the value from the ASN1 DER encoded packet
        var asn1 = new Asn1InputStream(acl);
        Asn1Sequence sequence = (Asn1Sequence)asn1.ReadObject();
        DerOctetString der = (DerOctetString)sequence[1];

        mySecurityContext.TargetFAKey = der.GetOctets();
      }
      catch (Exception ex)
      {
        ClearCryptographicData();
        throw new ApplicationException("Error retrieving the Target FA key from the Key Storage container", ex);
      }


      // Retrieve the Target Mifare keys
      try
      {
        RaiseVerbose("- Retrieving Mifare Classic keys");
        plaidApp.Authenticate(samApp, keysets, Settings.Default.OpModeKSMifare, out acl);

        // NOTE: The KEYA+KEYB values are concatenated together into a single 12-byte record
        // Extract the value from the ASN1 DER encoded packet
        var asn1 = new Asn1InputStream(acl);
        Asn1Sequence sequence = (Asn1Sequence)asn1.ReadObject();
        DerOctetString der = (DerOctetString)sequence[1];
        acl = der.GetOctets();

        mySecurityContext.MifareKeyA = new byte[MifareProtocol.KeyLength];
        mySecurityContext.MifareKeyB = new byte[MifareProtocol.KeyLength];

        Array.Copy(acl, 0, mySecurityContext.MifareKeyA, 0, MifareProtocol.KeyLength);
        Array.Copy(acl, MifareProtocol.KeyLength, mySecurityContext.MifareKeyB, 0, MifareProtocol.KeyLength);

        RaiseVerbose(@"The keys have been loaded successfully");
      }
      catch (Exception ex)
      {
        ClearCryptographicData();
        throw new ApplicationException("Error retrieving the Target IA key from the Key Storage container", ex);
      }
    }

    private void ClearCryptographicData()
    {
      if (mySecurityContext == null) return;

      RaiseVerbose("Clearing all key values from memory");

      // Reset the values in memory
      if (mySecurityContext.AdminIAKey != null) mySecurityContext.AdminIAKey.Initialize();
      if (mySecurityContext.AdminFAKey != null) mySecurityContext.AdminFAKey.Initialize();
      if (mySecurityContext.TargetIAKey != null) mySecurityContext.TargetIAKey.Initialize();
      if (mySecurityContext.TargetFAKey != null) mySecurityContext.TargetFAKey.Initialize();
      if (mySecurityContext.MifareKeyA != null) mySecurityContext.MifareKeyA.Initialize();
      if (mySecurityContext.MifareKeyB != null) mySecurityContext.MifareKeyB.Initialize();

      // Delete the object
      mySecurityContext = null;
    }

    private void EncodeCard()
    {
      Card target = myPCSC[TargetReader].ActiveCard;

      RaiseVerbose("Encoding target card");

      CardContext context = new CardContext();
      context.ATR = target.GetProtocol<Iso7816Protocol>().Atr.Value;
      RaiseVerbose(string.Format("- ATR = '{0}'", Converters.ByteArrayToString(context.ATR)));

      // Read the GlobalPlatform CPLC data element
      try
      {
        RaiseProgress(10);
        RaiseVerbose("- Reading the GlobalPlatform CPL data element");
        GlobalPlatformApplication app = new GlobalPlatformApplication();
        app.SetCard(target);
        app.SelectApplication();
        context.CPLC = app.ReadCplc();
      }
      catch (Exception ex)
      {
        throw new ApplicationException("Error reading the GlobalPlatform CPLC data element", ex);
      }

      // Read the FIPS-201 CHUID data element
      try
      {
        RaiseProgress(20);
        RaiseVerbose("- Reading the FIPS-201 (PIV) CHUID data element");
        Fips201Application app = new Fips201Application();
        app.SetCard(target);
        app.SelectApplication();
        context.CHUID = app.ReadChuid();
      }
      catch (Exception ex)
      {
        throw new ApplicationException("Error reading the FIPS-201 CHUID data element", ex);
      }

      // Generate the OpMode access control values
      RaiseVerbose("- Generating the access control records");
      RaiseProgress(70);

      // OpMode 1 - Wiegand 46-bit
      context.Wiegand46 = new byte[sizeof(ulong)];
      ulong opMode1 = WiegandRecord.GenerateWiegand46(context.CPLC.ICSerialNo);
      Converters.IntToArray(opMode1, context.Wiegand46, 0);

      // OpMode 2 - Wiegand 26-bit
      uint opMode2 = WiegandRecord.GenerateWiegand26(context.CPLC.ICSerialNo);
      context.Wiegand26 = new byte[sizeof(uint)];
      Converters.IntToArray(opMode2, context.Wiegand26, 0);

      // OpMode 3 - PI
      context.PI = context.CHUID.FASCN.PI;

      // OpMode 4 - UUID
      context.UUID = context.CHUID.GUID;

      // Personalise the PLAID application
      PersonalisePlaid(target, context);
      RaiseProgress(80);

      // Validate the PLAID personalisation with an authentication
      PlaidSamApplication sam = new PlaidSamApplication();
      sam.SetCard(myPCSC[KeyAuthenticatorReader].ActiveCard);

      PlaidApplication plaid = new PlaidApplication();
      plaid.SetCard(target);
      KeysetList ksl = new KeysetList();
      ksl.Add(Settings.Default.Keyset1);
      byte[] acl;
      plaid.Authenticate(sam, ksl, 1, out acl);


      // Personalise the Mifare Classic application
      // NOTE: This must be done AFTER the PLAID personalisation as it will
      //       drop the card to ISO14443-3 layer
      PersonaliseMifare(target, context);

      // Create journal record
      RaiseProgress(100);
      WriteJournal(target, context);
    }

    private void WriteJournal(Card card, CardContext context)
    {
      string name = string.Format(@"{0}.Xml", context.CPLC.ICSerialNo);
      string path = string.Format(@"{0}\{1}", Settings.Default.JournalPath, name);

      if (File.Exists(path))
      {
        string msg = string.Format("WARNING - Journal record already exists '{0}'", name);
        RaiseError(msg, CardProcessorResult.None);
        path = string.Format(@"{0}\{1}-{2}-{3}-{4}-{5}-{6}-{7}.Xml", 
                                                                     DateTime.Now.Year.ToString("0000"),
                                                                     DateTime.Now.Month.ToString("00"),
                                                                     DateTime.Now.Day.ToString("00"),
                                                                     DateTime.Now.Hour.ToString("00"),
                                                                     DateTime.Now.Minute.ToString("00"),
                                                                     DateTime.Now.Second.ToString("00"),
                                                                     DateTime.Now.Millisecond.ToString("0000"));

        RaiseVerbose(string.Format("Recovery journal saved to: '{0}'.", path));
        Serialiser<CardContext>.Save(path, context);
      }
      else
      {
        try
        {
          RaiseVerbose(string.Format("Journal: '{0}'.", name));
          Serialiser<CardContext>.Save(path, context);
        }
        catch (Exception ex)
        {
          RaiseError("An writing the journal record, stopping.", CardProcessorResult.None);
          RaiseError(ex.Message, CardProcessorResult.Stopped);
        }
      }
    }

    private void PersonaliseMifare(Card card, CardContext context)
    {
      // Get a reference to our mifare protocol
      PCSCMifareProtocol protocol = card.GetProtocol<PCSCMifareProtocol>();
      RaiseVerbose("- Personalising the Mifare application");

      /*
       * Generate our sector trailer 
       */

      SectorPermissions permissions = new SectorPermissions();
      permissions.KeyA = mySecurityContext.MifareKeyA;
      permissions.KeyB = mySecurityContext.MifareKeyB;
      permissions.Block0 = AccessCondition.Mode2;
      permissions.Block1 = AccessCondition.Mode2;
      permissions.Block2 = AccessCondition.Mode2;
      permissions.SectorTrailer = AccessCondition.Mode5;
      permissions.UserData = 0;

      /*
       * Write the access control records
       */

      // OpMode1 - 46 bit wiegand
      if (Settings.Default.MifareOpMode1)
      {
        // Authenticate to the sector with the FACTORY KeyB
        protocol.Authenticate(Settings.Default.MifareOpMode1Sector, MifareKeyType.KeyB, MifareProtocol.FactoryKey);

        // Generate the wiegand value buffer
        byte[] buffer = new byte[MifareProtocol.BlockLength];
        Array.Copy(context.Wiegand46, buffer, context.Wiegand46.Length);

        // Write the buffer
        protocol.WriteBlock(Settings.Default.MifareOpMode1Block, buffer);

        // Read back and verify
        byte[] comparison = protocol.ReadBlock(Settings.Default.MifareOpMode1Block);
        if (!Converters.CompareArray<byte>(buffer, comparison))
        {
          // Our verification failed, throw an error
          throw new ApplicationException("Mifare OpMode1 write failed verification test");
        }

        // Write the permissions
        protocol.WritePermissions(Settings.Default.MifareOpMode1Sector, permissions);
      }

      // OpMode2 - 26 bit wiegand
      if (Settings.Default.MifareOpMode2)
      {
        // Authenticate to the sector with the FACTORY KeyB
        protocol.Authenticate(Settings.Default.MifareOpMode2Sector, MifareKeyType.KeyB, MifareProtocol.FactoryKey);

        // Generate the wiegand value buffer
        byte[] buffer = new byte[MifareProtocol.BlockLength];
        Array.Copy(context.Wiegand26, buffer, context.Wiegand26.Length);

        // Write the buffer
        protocol.WriteBlock(Settings.Default.MifareOpMode2Block, buffer);

        // Read back and verify
        byte[] comparison = protocol.ReadBlock(Settings.Default.MifareOpMode2Block);
        if (!Converters.CompareArray<byte>(buffer, comparison))
        {
          // Our verification failed, throw an error
          throw new ApplicationException("Mifare OpMode2 write failed verification test");
        }
      }

      // Write the permissions
      protocol.WritePermissions(Settings.Default.MifareOpMode2Sector, permissions);

      // Lock all the unused sector trailers
      if (Settings.Default.MifareLockUnused)
      {
        for (byte i = 0; i < MifareProtocol.SectorCount1K; i++)
        {
          // Skip the OpMode sectors if we have already written the keys
          if ((i != Settings.Default.MifareOpMode1Sector || !Settings.Default.MifareOpMode1) &&
              (i != Settings.Default.MifareOpMode2Sector || !Settings.Default.MifareOpMode2))
          {
            permissions = new SectorPermissions();
            permissions.KeyA = mySecurityContext.MifareKeyA;
            permissions.KeyB = mySecurityContext.MifareKeyB;
            permissions.Block0 = AccessCondition.Mode0;
            permissions.Block1 = AccessCondition.Mode0;
            permissions.Block2 = AccessCondition.Mode0;
            permissions.SectorTrailer = AccessCondition.Mode5;
            permissions.UserData = 0;

            // Authenticate to the sector with the FACTORY KeyB
            protocol.Authenticate(i, MifareKeyType.KeyB, MifareProtocol.FactoryKey);

            // Write the permissions
            protocol.WritePermissions(i, permissions);
          }
        }
      }

      // Done
    }

    private void PersonalisePlaid(Card card, CardContext context)
    {
      RaiseVerbose("- Personalising the PLAID application");

      // Select the PLAID application
      PlaidApplication app = new PlaidApplication();
      app.SetCard(card);
      app.SelectApplication();

      // Create a personalisation record
      PlaidPersonalisationRecord record = new PlaidPersonalisationRecord();
      
      // Generate the DIVDATA
      context.DivData = Crypto.CreateRandomEntropy(PlaidApplication.DivDataLength);
      record.DivData = context.DivData;

      // OpMode1 - Wiegand 46 bit
      if (Settings.Default.PlaidOpMode1)
      {
        AcsRecord acs = new AcsRecord();
        acs.OpModeId = Settings.Default.OpMode1Id;

        // Generate the wiegand value buffer
        acs.Data = context.Wiegand46;

        record.AcsRecords.Add(acs);
      }

      // OpMode2 - Wiegand 26 bit
      if (Settings.Default.PlaidOpMode2)
      {
        AcsRecord acs = new AcsRecord();
        acs.OpModeId = Settings.Default.OpMode2Id;

        // Generate the wiegand value buffer
        acs.Data = context.Wiegand26;

        record.AcsRecords.Add(acs);
      }

      // OpMode3 - PIV PI
      if (Settings.Default.PlaidOpMode3)
      {
        AcsRecord acs = new AcsRecord();
        acs.OpModeId = Settings.Default.OpMode3Id;

        // Generate the wiegand value buffer
        acs.Data = Converters.StringToByteArray(context.PI);

        record.AcsRecords.Add(acs);
      }

      // OpMode4 - RFC 4122 UUID
      if (Settings.Default.PlaidOpMode4)
      {
        AcsRecord acs = new AcsRecord();
        acs.OpModeId = Settings.Default.OpMode4Id;

        // Generate the wiegand value buffer
        acs.Data = Converters.StringToByteArray(context.PI);

        record.AcsRecords.Add(acs);
      }

      // Admin keyset
      {
        PlaidKey k = new PlaidKey();
        k.KeysetId = Settings.Default.Keyset0;
        k.IAKey = new RSAKey(mySecurityContext.AdminIAKey);
        k.FAKey = mySecurityContext.AdminFAKey;
        record.Keys.Add(k);
      }

      // Target keyset
      {
        PlaidKey k = new PlaidKey();
        k.KeysetId = Settings.Default.Keyset1;
        k.IAKey = new RSAKey(mySecurityContext.TargetIAKey);
        k.FAKey = mySecurityContext.TargetFAKey;
        record.Keys.Add(k);
      }

      // Personalise the PLAID applet
      try
      {
        app.Personalise(record, true);
      }
      catch (Exception ex)
      {
        throw new ApplicationException("An error occurred while personalising the PLAID applet", ex);
      }
    }

    public void VerifyCard()
    {
      PCSCCard target = myPCSC[TargetReader].ActiveCard;

      RaiseVerbose("Verifying target card");

      CardContext context = new CardContext();
      context.ATR = target.GetProtocol<Iso7816Protocol>().Atr.Value;
      RaiseVerbose(string.Format("- ATR = '{0}'", Converters.ByteArrayToString(context.ATR)));

      // Read the GlobalPlatform CPLC data element
      try
      {
        RaiseVerbose("- Reading the GlobalPlatform CPL data element");
        GlobalPlatformApplication app = new GlobalPlatformApplication();
        app.SetCard(target);
        app.SelectApplication();
        context.CPLC = app.ReadCplc();
      }
      catch (Exception ex)
      {
        throw new ApplicationException("Error reading the GlobalPlatform CPLC data element", ex);
      }

      // Read the FIPS-201 CHUID data element
      try
      {
        RaiseVerbose("- Reading the FIPS-201 (PIV) CHUID data element");
        Fips201Application app = new Fips201Application();
        app.SetCard(target);
        app.SelectApplication();
        context.CHUID = app.ReadChuid();
      }
      catch (Exception ex)
      {
        throw new ApplicationException("Error reading the FIPS-201 CHUID data element", ex);
      }

      // PLAID applet verification
      VerifyPlaid(target, context);

      // Mifare applet verification
      VerifyMifare(target, context);

      // Show the form
      FViewer viewer = new FViewer();
      viewer.LoadContext(context);
      viewer.ShowDialog();
    }

    private void VerifyPlaid(PCSCCard target, CardContext context)
    {
      // Authenticate to the PLAID application
      PCSCCard sam = (PCSCCard)myPCSC[KeyAuthenticatorReader].ActiveCard;
      PlaidApplication plaid = new PlaidApplication();
      PlaidSamApplication samApp = new PlaidSamApplication();

      plaid.SetCard(target);
      samApp.SetCard(sam);

      byte[] acl;

      // Authenticate with the verification key
      KeysetList keys = new KeysetList();
      keys.Add(Settings.Default.Keyset1);

      try
      {
        sam.Connect();
        plaid.SelectApplication();
        samApp.SelectApplication();
      }
      catch (Exception ex)
      {
        sam.Disconnect();
        throw new ApplicationException("Error connecting to the KAM", ex);
      }

      try
      {
        RaiseVerbose("- Reading OpMode 1");
        plaid.Authenticate(samApp, keys, Settings.Default.OpMode1Id, out acl);
        context.Wiegand46 = acl;
      }
      catch (Exception)
      {
        RaiseVerbose("- OpMode 1 could not be read.");
      }

      try
      {
        RaiseVerbose("- Reading OpMode 2");
        plaid.Authenticate(samApp, keys, Settings.Default.OpMode2Id, out acl);
        context.Wiegand26 = acl;
      }
      catch (Exception)
      {
        RaiseVerbose("- OpMode 2 could not be read.");
      }

      try
      {
        RaiseVerbose("- Reading OpMode 3");
        plaid.Authenticate(samApp, keys, Settings.Default.OpMode3Id, out acl);
        context.PI = Converters.ByteArrayToString(acl);
      }
      catch (Exception)
      {
        RaiseVerbose("- OpMode 3 could not be read.");
      }

      try
      {
        RaiseVerbose("- Reading OpMode 4");
        plaid.Authenticate(samApp, keys, Settings.Default.OpMode4Id, out acl);
        context.UUID = acl;
      }
      catch (Exception)
      {
        RaiseVerbose("- OpMode 4 could not be read.");
        //throw new ApplicationException("Error reading OpMode 4", ex);
      }
      finally
      {
        sam.Disconnect();
      }

    }

    public void VerifyMifare(PCSCCard target, CardContext context)
    {
      PCSCMifareProtocol protocol = target.GetProtocol<PCSCMifareProtocol>();

      /*
       * Attempt to authorise using the factory key, if it can't, then we flag as personalised because 
       * we don't have KEYA/KEYB to tell otherwise.
       */
      try
      {
        context.VerifyMifare = false;
        protocol.Authenticate(Settings.Default.MifareOpMode1Sector, MifareKeyType.KeyB, MifareProtocol.FactoryKey);
        protocol.ReadBlock(Settings.Default.MifareOpMode1Block);
      }
      catch (Exception)
      {
        context.VerifyMifare = true;
      }
    }

    #region "Event methods"

    public event EventHandler<CardProcessorEventArgs> OnProcessEvent;

    private void RaiseOnProcessEvent(string message, CardProcessorEventLevel level, CardProcessorResult result)
    {
      if (OnProcessEvent != null)
      {
        OnProcessEvent(this, new CardProcessorEventArgs(message, level, result));
      }
    }

    private void RaiseOnProcessEvent(CardProcessorEventArgs args)
    {
      if (OnProcessEvent != null)
      {
        OnProcessEvent(this, args);
      }
    }

    private void RaiseInfo(string message)
    {
      RaiseOnProcessEvent(message, CardProcessorEventLevel.Info, CardProcessorResult.None);
    }

    private void RaiseVerbose(string message)
    {
      RaiseOnProcessEvent(message, CardProcessorEventLevel.Verbose, CardProcessorResult.None);
    }

    private void RaiseProgress(int progress)
    {
      CardProcessorEventArgs args = new CardProcessorEventArgs();
      args.Level = CardProcessorEventLevel.Progress;
      args.Progress = progress;

      RaiseOnProcessEvent(args);
    }

    private void RaiseCardState(string state)
    {
      CardProcessorEventArgs args = new CardProcessorEventArgs();
      args.Level = CardProcessorEventLevel.CardState;
      args.Message = state;

      RaiseOnProcessEvent(args);
    }

    private void RaiseError(string message, CardProcessorResult result)
    {
      RaiseOnProcessEvent(message, CardProcessorEventLevel.Error, result);
    }

    public void Raise(string message, CardProcessorEventLevel level, CardProcessorResult result)
    {
      RaiseOnProcessEvent(message, level, result);
    }

    public void Raise(string message, CardProcessorResult result)
    {
      RaiseOnProcessEvent(message, CardProcessorEventLevel.Info, result);
    }

    #endregion

    public string KeyAuthenticatorReader { get; set; }
    public string KeyStorageReader { get; set; }
    public string TargetReader { get; set; }

    private PCSCManager myPCSC = null;
    private SecurityContext mySecurityContext = null;
  }


  public class SecurityContext
  {
    public SecurityContext()
    {
      AdminIAKey = new byte[0];
      AdminFAKey = new byte[0];
      TargetIAKey = new byte[0];
      TargetFAKey = new byte[0];

      MifareKeyA = new byte[0];
      MifareKeyB = new byte[0];
    }

    public byte[] AdminIAKey { get; set; }
    public byte[] AdminFAKey { get; set; }
    public byte[] TargetIAKey { get; set; }
    public byte[] TargetFAKey { get; set; }
    public byte[] MifareKeyA { get; set; }
    public byte[] MifareKeyB { get; set; }
  }
}
