﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CardFramework.Readers.PCSC;
using CardFramework;
using System.Threading;
using CardFramework.Applications.Fips201;
using CardFramework.Helpers;
using CardFramework.Applications.Plaid;
using PACManager.Properties;
using System.Security.Cryptography;
using CardFramework.Protocols.Mifare;
using CardFramework.Applications.GlobalPlatform;
using System.Diagnostics;

namespace PACManager
{
  /// <summary>
  /// A simple command-line utility to to assist during development. It performs a number
  /// of functions (Mifare wipe, Read card) but is mainly here to allow basic diagnostic
  /// utilities to be quickly written.
  /// </summary>
  public static class CardUtility
  {
    private static bool myIssuerKey = false;
    private static bool myCPLC = false;
    private static bool myCHUID = false;
    private static string myMode = "READ";

    static PCSCManager reader = new PCSCManager();

    public static void Run(string[] args)
    {
      // Read arguments
      if (args.Contains("-K")) myIssuerKey = true;
      if (args.Contains("-G")) myCPLC = true;
      if (args.Contains("-F")) myCHUID = true;

      if (args.Contains("-M"))
      {
        myMode = "MIFARE";
        Console.WriteLine("MODE: Mifare Reset (to Oberthur transport conditions)");
      }
      else
      {
        Console.WriteLine("MODE: Read card");
      }
      Console.WriteLine("Press the 'Q' key to abort the process.");


      // Hook events
      reader.OnReaderInserted += new EventHandler<PCSCReaderEventArgs>(reader_OnReaderInserted);
      reader.OnReaderRemoved += new EventHandler<PCSCReaderEventArgs>(reader_OnReaderRemoved);
      reader.OnCardInserted += new EventHandler<CardEventArgs>(reader_OnCardInserted);
      reader.OnCardRemoved += new EventHandler<CardEventArgs>(reader_OnCardRemoved);

      // Start polling
      reader.Connect();
      reader.Start();

      while (Console.ReadKey().Key != ConsoleKey.Q)
      {
        Thread.Sleep(0);
      }

      Console.WriteLine("Aborting the process.");

      // Stop polling
      reader.Stop();
      reader.Disconnect();
    }

    static void reader_OnReaderRemoved(object sender, PCSCReaderEventArgs e)
    {
      Console.WriteLine("Reader removed '{0}'", e.Reader);
    }

    static void reader_OnReaderInserted(object sender, PCSCReaderEventArgs e)
    {
      Console.WriteLine("Reader inserted '{0}'", e.Reader);
    }

    static void reader_OnCardRemoved(object sender, CardFramework.CardEventArgs e)
    {
      Console.WriteLine("Card removed from reader '{0}'", e.Reader.ToString());
    }

    static void reader_OnCardInserted(object sender, CardFramework.CardEventArgs e)
    {
      Console.WriteLine("Card inserted in reader '{0}'", e.Reader.ToString());

      // Connect to the card
      PCSCCard card = (PCSCCard)e.Card;

      try
      {
        Console.WriteLine("Connecting to the card ...");
        card.Connect();
        Console.WriteLine("Connected: (ATR '{0}')", e.Card.GetProtocol<PCSCIso7816Protocol>().Atr.ToString());

        /*
         * NOTE: This is where any test code goes
         */

        if (myMode == "MIFARE")
        {
          MifareWipe(card);
        }
        else
        {
          if (myCPLC) ReadCPLC(card);
          if (myCHUID) ReadChuid(card);
        }
      }
      catch (Exception ex)
      {
        Console.WriteLine(ex.Message);
      }
      finally
      {
        card.Disconnect();
      }
    }

    [Conditional("DEBUG")]
    static void MifareWipe(Card card)
    {
      MifareProtocol protocol = card.GetProtocol<MifareProtocol>();
      SectorPermissions permissions = new SectorPermissions();

      // Reset the permissions back to 'Oberthur Factory'
      permissions.KeyA = MifareProtocol.FactoryKey;
      permissions.KeyB = MifareProtocol.FactoryKey;
      permissions.Block0 = AccessCondition.Mode0;
      permissions.Block1 = AccessCondition.Mode0;
      permissions.Block2 = AccessCondition.Mode0;
      permissions.SectorTrailer = AccessCondition.Mode5;

      byte[] keyA = new byte[] { 0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA };
      byte[] keyB = new byte[] { 0xBB, 0xBB, 0xBB, 0xBB, 0xBB, 0xBB };

      byte[] buffer = new byte[16];

      for (byte i = 0; i < MifareProtocol.SectorCount1K; i++)
      {
        bool good = false;

        // First try to authenticate with the factory key
        try
        {
          protocol.Authenticate(i, MifareKeyType.KeyB, MifareProtocol.FactoryKey);
          buffer = protocol.ReadBlock(MifareProtocol.GetSectorTrailerBlock(i));
          good = true;
        }
        catch { }

        if (!good)
        {
          try
          {
            protocol.Authenticate(i, MifareKeyType.KeyB, keyB);
            buffer = protocol.ReadBlock(MifareProtocol.GetSectorTrailerBlock(i));
            good = true;
          }
          catch { }
        }

        if (!good)
        {
          try
          {
            protocol.Authenticate(i, MifareKeyType.KeyA, MifareProtocol.FactoryKey);
            buffer = protocol.ReadBlock(MifareProtocol.GetSectorTrailerBlock(i));
            good = true;
          }
          catch { }
        }

        if (!good)
        {
          try
          {
            protocol.Authenticate(i, MifareKeyType.KeyA, keyA);
            buffer = protocol.ReadBlock((byte)(i * 4));
            good = true;
          }
          catch { }
        }

        if (good)
        {
          Console.WriteLine("Trailer for sector " + i.ToString() + ": " + Converters.ByteArrayToString(buffer));

          buffer = new byte[16];
          buffer.Initialize();

          byte block = (byte)(i * 4);

          try
          {
            if (i != 0)
            {
              protocol.WriteBlock(block++, buffer);
            }
            else
            {
              block++;
            }

            protocol.WriteBlock(block++, buffer);
            protocol.WriteBlock(block++, buffer);
            protocol.WritePermissions(i, permissions);
            Console.WriteLine("Sector " + i.ToString() + " has been reset");
          }
          catch (Exception)
          {
            Console.WriteLine("Failed to reset sector " + i.ToString());
          }
        }
        else
        {
          Console.WriteLine("Failed to authenticate to sector: " + i.ToString());
        }
      }
    }

    static void ReadChuid(Card card)
    {
      Console.WriteLine(">> Reading FIPS-201 CHUID");
      {
        Fips201Application app = new Fips201Application();
        app.SetCard(card);

        Console.Write("SELECTING THE FIPS-201 APPLICATION ... ");
        app.SelectApplication();
        Console.WriteLine("Done");

        Console.WriteLine("READING THE CHUID ...");
        Chuid chuid = app.ReadChuid();

        Console.WriteLine("-----------------------------------------------");
        Console.WriteLine("FASC-N Agency Code: {0} - {1}", chuid.FASCN.AgencyCode, chuid.FASCN.GetAgencyCodeString());
        Console.WriteLine("FASC-N System Code: {0}", chuid.FASCN.SystemCode);
        Console.WriteLine("FASC-N Credential Number: {0}", chuid.FASCN.CredentialNumber);
        Console.WriteLine("FASC-N Credential Series: {0}", chuid.FASCN.CS);
        Console.WriteLine("FASC-N Individual Credential Issue: {0}", chuid.FASCN.ICI);
        Console.WriteLine("FASC-N Person Id: {0}", chuid.FASCN.PI);
        Console.WriteLine("GUID: {0}", Converters.ByteArrayToString(chuid.GUID));
        Console.WriteLine("Expiration Date: {0}", chuid.ExpirationDate);
        if (myIssuerKey) Console.WriteLine("Issuer Signature:\n{0}", Converters.ByteArrayToString(chuid.IssuerSignature));
        Console.WriteLine("-----------------------------------------------");
      }
    }

    static void ReadCPLC(Card card)
    {
      Console.WriteLine("\n>> Reading GlobalPlatform CPLC");
      Console.WriteLine("-----------------------------------------------");
      {
        GlobalPlatformApplication app = new GlobalPlatformApplication();
        app.SetCard(card);
        app.SelectApplication();

        Cplc cplc = app.ReadCplc();

        // -  IC Fabricator (2B)
        Console.WriteLine("IC Fabricator: {0:X}", cplc.ICFabricator);

        // -  IC Type (2B)
        Console.WriteLine("IC Type: {0:X}", cplc.ICType);

        // -  Operating System Provider Identifier (2B)
        Console.WriteLine("OS Provider Id: {0:X}", cplc.OSProviderId);

        // -  Operating System Release Date (2B)
        Console.WriteLine("OS Release Date: {0:X}", cplc.OSReleaseDate);

        // -  Operating System Release Level (2B)
        Console.WriteLine("OS Release Level: {0:X}", cplc.OSReleaseLevel);

        /*
         * EEPROM Identifiers contain chip, card, pre-perso & personalizer info., in the given order :
         */

        // -  Chip Identifiers:
        // -  IC Fabrication Date (2B)
        Console.WriteLine("IC Fabrication Date: {0:X}", cplc.ICFabricationDate);

        // -  IC Serial Number (4B)
        Console.WriteLine("IC Serial Number: {0:X}", cplc.ICSerialNo);

        // -  IC Batch Identifier (2B)
        Console.WriteLine("IC Batch Id: {0:X}", cplc.ICBatchId);

        // -  Card identifiers :
        // -  IC Module Fabricator  (2B)
        Console.WriteLine("IC Module Fabricator: {0:X}", cplc.ICModuleFabricator);

        // -  IC Module Packaging Date      (2B)
        Console.WriteLine("IC Module Packaging Date: {0:X}", cplc.ICModulePackagingDate);

        // -  ICC Manufacturer      (2B)
        Console.WriteLine("IC Manufacturer: {0:X}", cplc.ICCManufacturer);

        // -  IC Embedding date     (2B)
        Console.WriteLine("IC Embedding Date: {0:X}", cplc.ICEmbeddingDate);

        // -  Pre-personalization identifiers :
        // -  Pre-personalizer Identifier (2B)
        Console.WriteLine("Pre-personalisation Id: {0:X}", cplc.PrePersoId);

        // -  Pre-personalization Date (2B)
        Console.WriteLine("Pre-personalisation Date: {0:X}", cplc.PrePersoDate);

        // -  Pre-personalization Equipment (4B)
        Console.WriteLine("Pre-personalisation Equipment: {0:X}", cplc.PrePersoEquipment);

        // -  Personalization identifiers :
        // -  Personalization Identifier (2B)
        Console.WriteLine("Personalisation Id: {0:X}", cplc.PersoId);

        // -  Personalization Date (2B)
        Console.WriteLine("Personalisation Date: {0:X}", cplc.PersoDate);

        // -  Personalization Equipment (4B)   
        Console.WriteLine("Personalisation Equipment: {0:X}", cplc.PersoEquipment);
      }
    }
  }
}


// Snippet for PLAID on-card debugger
//PlaidApplication app = new PlaidApplication();
//app.SetCard(card);
//app.SelectApplication();
//byte[] estr1;
//var ks = new KeysetList();
//ks.Add(0);
//app.InitialAuthenticate(ks, out estr1);
