﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.IO;
using System.Windows.Forms;

using CardFramework;
using CardFramework.Readers.PCSC;
using CardFramework.Protocols;
using CardFramework.Applications.Plaid;
using CardFramework.Applications.Fips201;
using CardFramework.Applications.GlobalPlatform;

using PACManager.Properties;
using System.Threading;

namespace PACManager.Controls
{
  public partial class ConfigureScreen : PACManager.Controls.WizardScreen
  {
    public ConfigureScreen()
    {
      InitializeComponent();

      // Setup our thread sync context
      myContext = SynchronizationContext.Current;
    }

    private PCSCCard myPlaidSam = null;
    private PCSCCard myPlaid = null;
    private bool myIsReadOnlySam = false;

    public override void Startup()
    {
      base.Startup();

      // Set the default reader text
      const string ReaderSelectionText = "Please select a reader";

      cboPlaidSamReader.Items.Add(ReaderSelectionText);
      cboPlaidReader.Items.Add(ReaderSelectionText);
      cboTargetReader.Items.Add(ReaderSelectionText);

      // Connect to the PCSC manager
      myPCSC = new PCSCManager();
      myPCSC.OnReaderInserted += new EventHandler<PCSCReaderEventArgs>(myPCSC_OnReaderInserted);
      myPCSC.OnReaderRemoved += new EventHandler<PCSCReaderEventArgs>(myPCSC_OnReaderRemoved);
      myPCSC.OnCardInserted += new EventHandler<CardEventArgs>(myPCSC_OnCardInserted);
      myPCSC.OnCardRemoved += new EventHandler<CardEventArgs>(myPCSC_OnCardRemoved);

      this.Show();
      Application.DoEvents();

      myPCSC.Connect();
      myPCSC.Start();
    }

    public override void Shutdown()
    {
      base.Shutdown();

      myPCSC.Stop();
      myPCSC.Disconnect();

      myPCSC.OnReaderInserted -= new EventHandler<PCSCReaderEventArgs>(myPCSC_OnReaderInserted);
      myPCSC.OnReaderRemoved -= new EventHandler<PCSCReaderEventArgs>(myPCSC_OnReaderRemoved);
      myPCSC.OnCardInserted -= new EventHandler<CardEventArgs>(myPCSC_OnCardInserted);
      myPCSC.OnCardRemoved -= new EventHandler<CardEventArgs>(myPCSC_OnCardRemoved);
    }

    void myPCSC_OnCardRemoved(object sender, CardEventArgs e)
    {
      myContext.Post(delegate(Object obj)
      {
        if (e.Reader == cboPlaidSamReader.Text)
        {
          pboPlaidSamReader.Hide();
          lblReadOnlySam.Hide();
          myPlaidSam = null;
        }
        else if (e.Reader == cboPlaidReader.Text)
        {
          pboPlaidReader.Hide();
          myPlaid = null;
        }
      }, null);
    }

    void myPCSC_OnCardInserted(object sender, CardEventArgs e)
    {
      myContext.Post(delegate(Object obj)
      {
        UseWaitCursor = true;
        if (e.Reader == cboPlaidSamReader.Text)
        {
          myPlaidSam = (PCSCCard)e.Card;
          ValidatePlaidSam();
        }
        else if (e.Reader == cboPlaidReader.Text)
        {
          myPlaid = (PCSCCard)e.Card;
          ValidatePlaid();
        }
        UseWaitCursor = false;
      }, null);
    }

    void myPCSC_OnReaderRemoved(object sender, PCSCReaderEventArgs e)
    {
      myContext.Post(delegate(Object obj)
      {
        // Remove from the reader lists and de-select if necessary
        if (e.Reader == cboPlaidSamReader.Text)
        {
          // Deselect the reader
          cboPlaidSamReader.SelectedIndex = 0;
        }
        cboPlaidSamReader.Items.Remove(e.Reader);

        if (e.Reader == cboPlaidReader.Text)
        {
          // Deselect the reader
          cboPlaidReader.SelectedIndex = 0;
        }
        cboPlaidReader.Items.Remove(e.Reader);

        if (e.Reader == cboTargetReader.Text)
        {
          cboTargetReader.SelectedIndex = 0;
        }
        cboTargetReader.Items.Remove(e.Reader);
      }, null);
    }

    void myPCSC_OnReaderInserted(object sender, PCSCReaderEventArgs e)
    {
      myContext.Post(delegate(Object obj)
      {
        cboPlaidSamReader.Items.Add(e.Reader);
        if (myPCSC.Contains(Settings.Default.PlaidSamReader))
        {
          if (e.Reader == Settings.Default.PlaidSamReader) cboPlaidSamReader.Text = Settings.Default.PlaidSamReader;
        }
        else
        {
          cboPlaidSamReader.SelectedIndex = 0;
        }

        cboPlaidReader.Items.Add(e.Reader);
        if (myPCSC.Contains(Settings.Default.PlaidReader))
        {
          if (e.Reader == Settings.Default.PlaidReader) cboPlaidReader.Text = Settings.Default.PlaidReader;
        }
        else
        {
          cboPlaidReader.SelectedIndex = 0;
        }

        cboTargetReader.Items.Add(e.Reader);
        if (myPCSC.Contains(Settings.Default.TargetReader))
        {
          if (e.Reader == Settings.Default.TargetReader) cboTargetReader.Text = Settings.Default.TargetReader;
        }
        else
        {
          cboTargetReader.SelectedIndex = 0;
        }      
      }, null);
    }

    void ValidatePlaidSam()
    {
      bool valid;

      try
      {
        pboPlaidSamReader.Hide();

        myPlaidSam.Connect();
        myPlaidSam.BeginTransaction();

        PlaidSamApplication app = new PlaidSamApplication();
        valid = app.Discover(myPlaidSam);
        if (valid)
        {
          // Is this a read-only SAM?
          panKSM.Visible = (!app.IsReadOnly);
          lblReadOnlySam.Visible = (app.IsReadOnly);
          myIsReadOnlySam = app.IsReadOnly;
        }
      }
      catch (Exception)
      {
        valid = false;
      }
      finally
      {
        myPlaidSam.Disconnect();
      }
           
      if (valid)
      {
        pboPlaidSamReader.Image = global::PACManager.Properties.Resources.ValidationConnected;
        pboPlaidSamReader.Show();
      }
      else
      {
        pboPlaidSamReader.Image = global::PACManager.Properties.Resources.ValidationIncorrectICC;
        pboPlaidSamReader.Show();
      }
    }

    public void ValidatePlaid()
    {
      bool valid;

      try
      {
        pboPlaidReader.Hide();

        myPlaid.Connect();
        myPlaid.BeginTransaction();

        PlaidApplication app = new PlaidApplication();
        valid = app.Discover(myPlaid);
      }
      catch (Exception)
      {
        valid = false;
      }
      finally
      {
        myPlaid.Disconnect();
      }

      if (valid)
      {
        pboPlaidReader.Image = global::PACManager.Properties.Resources.ValidationConnected;
        pboPlaidReader.Show();
      }
      else
      {
        pboPlaidReader.Image = global::PACManager.Properties.Resources.ValidationIncorrectICC;
        pboPlaidReader.Show();
      }
    }

    private void btnJournalPath_Click(object sender, EventArgs e)
    {
      FolderBrowserDialog dialog = new FolderBrowserDialog();

      if (dialog.ShowDialog() == DialogResult.OK)
      {
        // Validate the selection
        if (string.IsNullOrEmpty(dialog.SelectedPath)) return;
        if (!Directory.Exists(dialog.SelectedPath)) return;

        // It's good, set the path
        tboJournalPath.Text = dialog.SelectedPath;
      }
    }

    private void btnNext_Click(object sender, EventArgs e)
    {
      // Check for valid readers by looking for the "VALID" tag on the combo box
      if (myPlaidSam == null)
      {
        MessageBox.Show("Please select a Key Authenticator reader and insert a valid PLAID SAM ICC.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        return;
      }

      if (myPlaid == null && !myIsReadOnlySam)
      {
        MessageBox.Show("Please select a Key Storage reader and insert a valid PLAID ICC.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        return;
      }
      if (cboTargetReader.Text == @"" || cboTargetReader.SelectedIndex == 0)
      {
        MessageBox.Show("Please select a Target reader.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        return;
      }

      if (!Directory.Exists(tboJournalPath.Text))
      {
        MessageBox.Show("Please select a valid journal path.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Warning);
        return;
      }

      // Save all the readers & other parameters
      Settings.Default.PlaidSamReader = cboPlaidSamReader.Text;
      Settings.Default.PlaidReader = cboPlaidReader.Text;
      Settings.Default.TargetReader = cboTargetReader.Text;
      Settings.Default.Save();

      ProcessScreen screen = new ProcessScreen();
      screen.CanEncode = (!myIsReadOnlySam);
      RaiseOnNextButtonEvent(screen);
    }

    private SynchronizationContext myContext;
    private PCSCManager myPCSC;

    private void cboPlaidSamReader_SelectedIndexChanged(object sender, EventArgs e)
    {
      myPlaidSam = null;
      pboPlaidSamReader.Hide();
      
      if (!myPCSC.Contains(cboPlaidSamReader.Text)) return;

      if (myPCSC[cboPlaidSamReader.Text].ActiveCard != null)
      {
        myPlaidSam = myPCSC[cboPlaidSamReader.Text].ActiveCard;
        ValidatePlaidSam();
      }
    }

    private void cboPlaidReader_SelectedIndexChanged(object sender, EventArgs e)
    {
      myPlaid = null;
      pboPlaidReader.Hide();

      if (!myPCSC.Contains(cboPlaidReader.Text)) return;

      if (myPCSC[cboPlaidReader.Text].ActiveCard != null)
      {
        myPlaid = myPCSC[cboPlaidReader.Text].ActiveCard;
        ValidatePlaid();
      }
    }

    private void ConfigureScreen_Load(object sender, EventArgs e)
    {
    }

    private void cboTargetReader_SelectedIndexChanged(object sender, EventArgs e)
    {
      pboTargetReader.Hide();

      if (!myPCSC.Contains(cboTargetReader.Text)) return;

      // Show the connected
      pboTargetReader.Image = global::PACManager.Properties.Resources.ValidationConnected;
      pboTargetReader.Show();
    }
  }
}
