﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CardFramework.Readers.PCSC;
using CardFramework;
using System.Threading;
using PACManager.Properties;
using CardFramework.Applications.Plaid;
using System.Security.Cryptography;
using System.IO;
using CardFramework.Helpers;

namespace PACManager.Controls
{
  public partial class KeyInjectionScreen : PACManager.Controls.WizardScreen
  {
    private SynchronizationContext myContext;
    private PCSCManager myPCSC;
    private PCSCCard myCard = null;

    private KeyStorageRecord myKeyStorage = null;

    public const string PROFILE_1 = @"KA1: Key Authenticator (Administration - PLAID SAM)";
    public const string PROFILE_2 = @"KA2: Key Authenticator (Read Only - PLAID SAM)";
    public const string PROFILE_3 = @"KS1: Key Storage (Administration - PLAID)";

    public KeyInjectionScreen()
    {
      InitializeComponent();

      cboEncodingProfile.Items.Add(PROFILE_1);
      cboEncodingProfile.Items.Add(PROFILE_2);
      cboEncodingProfile.Items.Add(PROFILE_3);

      // Setup our thread sync context
      myContext = SynchronizationContext.Current;
    }

    public override void Startup()
    {
      base.Startup();

      // Connect to the PCSC manager
      myPCSC = new PCSCManager();
      myPCSC.OnReaderInserted += new EventHandler<PCSCReaderEventArgs>(myPCSC_OnReaderInserted);
      myPCSC.OnReaderRemoved += new EventHandler<PCSCReaderEventArgs>(myPCSC_OnReaderRemoved);
      myPCSC.OnCardInserted += new EventHandler<CardEventArgs>(myPCSC_OnCardInserted);
      myPCSC.OnCardRemoved += new EventHandler<CardEventArgs>(myPCSC_OnCardRemoved);

      myPCSC.Connect();
      myPCSC.Start();
    }

    public override void Shutdown()
    {
      base.Shutdown();

      myPCSC.Stop();
      myPCSC.Disconnect();

      myPCSC.OnReaderInserted -= new EventHandler<PCSCReaderEventArgs>(myPCSC_OnReaderInserted);
      myPCSC.OnReaderRemoved -= new EventHandler<PCSCReaderEventArgs>(myPCSC_OnReaderRemoved);
      myPCSC.OnCardInserted -= new EventHandler<CardEventArgs>(myPCSC_OnCardInserted);
      myPCSC.OnCardRemoved -= new EventHandler<CardEventArgs>(myPCSC_OnCardRemoved);
    }

    void myPCSC_OnCardRemoved(object sender, CardEventArgs e)
    {
      myContext.Post(delegate(Object obj)
      {
        if (e.Reader == cboReader.Text)
        {
          pboReader.Hide();
          panEncodeComplete.Hide();
          panExecute.Hide();
          myCard = null;
        }
      }, null);
    }

    void myPCSC_OnCardInserted(object sender, CardEventArgs e)
    {
      myContext.Post(delegate(Object obj)
      {
        if (e.Reader == cboReader.Text)
        {
          myCard = (PCSCCard)e.Card;

          ValidateCard();
        }
      }, null);
    }

    void myPCSC_OnReaderRemoved(object sender, PCSCReaderEventArgs e)
    {
      myContext.Post(delegate(Object obj)
      {
        // Remove from the reader lists and de-select if necessary
        if (e.Reader == cboReader.Text)
        {
          // Deselect the reader
          cboReader.SelectedIndex = 0;
        }
        cboReader.Items.Remove(e.Reader);
      }, null);
    }

    void myPCSC_OnReaderInserted(object sender, PCSCReaderEventArgs e)
    {
      myContext.Post(delegate(Object obj)
      {
        cboReader.Items.Add(e.Reader);
      }, null);
    }

    /// <summary>
    /// This method validates that the card has the correct applet loaded
    /// </summary>
    void ValidateCard()
    {
      this.UseWaitCursor = true;

      bool passed = false;

      try
      {
        myCard.Connect();
      }
      catch (Exception)
      {
        myCard.Disconnect();
      }

      switch (cboEncodingProfile.Text)
      {
        case PROFILE_1: // PLAID SAM - ENCODE PROFILE
          {
            var app = new PlaidSamApplication();
            passed = app.Discover(myCard);
            if (passed) passed = !app.IsReadOnly;
          }
          break;
        case PROFILE_2: // PLAID SAM - READ ONLY PROFILE
          {
            var app = new PlaidSamApplication();
            passed = app.Discover(myCard);
            if (passed) passed = app.IsReadOnly;
          }
          break;

        case PROFILE_3: // PLAID
          {
            var app = new PlaidApplication();
            passed = app.Discover(myCard);
          }
          break;

        default:
          return;
      }

      myCard.Disconnect();

      try
      {
        pboReader.Hide();
        panExecute.Hide();

        if (passed)
        {
          pboReader.Image = global::PACManager.Properties.Resources.ValidationConnected;
          pboReader.Show();
          panExecute.Show();
          panExecute.Enabled = true;
        }
        else
        {
          pboReader.Image = global::PACManager.Properties.Resources.ValidationIncorrectICC;
          pboReader.Show();
        }
      }
      catch (Exception)
      {
        pboReader.Image = global::PACManager.Properties.Resources.ValidationIncorrectICC;
        pboReader.Show();
      }
      finally
      {
        this.UseWaitCursor = false;
      }

    }

    private void btnGenerateKeys_Click(object sender, EventArgs e)
    {
      string pathA = string.Format("{0}\\{1}-1of3.XML\n", Settings.Default.KeyStoragePath, Settings.Default.KeyStoragePrefix);
      string pathB = string.Format("{0}\\{1}-2of3.XML\n", Settings.Default.KeyStoragePath, Settings.Default.KeyStoragePrefix);
      string pathC = string.Format("{0}\\{1}-3of3.XML\n", Settings.Default.KeyStoragePath, Settings.Default.KeyStoragePrefix);

      if (File.Exists(pathA) ||
          File.Exists(pathB) ||
          File.Exists(pathC))
      {
        MessageBox.Show("Key generation not allowed while a keyset already exists.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        return;
      }

      this.UseWaitCursor = true;
      myKeyStorage = KeyStorageRecord.Generate();

      // Save the keys to a path
      try
      {
        KeyStorageRecord.SaveParts(myKeyStorage, Settings.Default.KeyStoragePath, Settings.Default.KeyStoragePrefix);
      
        // Notify the user of the archival
        string msg = "The keys have been generated and saved to the archive path:\n\n";
        msg += string.Format("{0}\\{1}-1of3.XML\n", Settings.Default.KeyStoragePath, Settings.Default.KeyStoragePrefix);
        msg += string.Format("{0}\\{1}-2of3.XML\n", Settings.Default.KeyStoragePath, Settings.Default.KeyStoragePrefix);
        msg += string.Format("{0}\\{1}-3of3.XML", Settings.Default.KeyStoragePath, Settings.Default.KeyStoragePrefix);
        MessageBox.Show(msg, "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);

        // Enable the next step
        this.UseWaitCursor = false;
        lblKeyLoadSuccess.Show();
        btnGenerateKeys.Enabled = false;
        btnImportKeys.Enabled = false;
        btnExport.Show();
        panEncodingProfile.Show();

      }
      catch (Exception ex)
      {
        myKeyStorage = null;
        string msg = string.Format("The keys could not be loaded.\nError: {0}", ex.Message);
        MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    private void cboEncodingProfile_SelectedIndexChanged(object sender, EventArgs e)
    {
      panEncodeComplete.Hide();
      panExecute.Hide();

      // Re-validate the card if necessary
      if (myPCSC.Contains(cboReader.Text))
      {
        if (myPCSC[cboReader.Text].IsCardPresent)
        {
          myCard = myPCSC[cboReader.Text].ActiveCard;
          ValidateCard();
        }
      }

      // Select which panels to show
      switch (cboEncodingProfile.Text)
      {
        case PROFILE_1: // PLAID SAM
        case PROFILE_2: // PLAID SAM
          panSelectReader.Show();
          panClassicKeys.Hide();
          break;

        case PROFILE_3: // PLAID
          panClassicKeys.Show();
          panSelectReader.Show();
          break;

        default:
          return;
      }

    }

    private void btnExecute_Click(object sender, EventArgs e)
    {
      panExecute.Enabled = false;

      try
      {
        myCard.Connect();
        myCard.BeginTransaction();

        switch (cboEncodingProfile.Text)
        {
          case PROFILE_1: // PLAID SAM - Admin & Target
            EncodeProfile1();
            break;

          case PROFILE_2: // PLAID SAM - Target Only
            EncodeProfile2();
            break;

          case PROFILE_3: // PLAID - Admin & Target
            EncodeProfile3();
            break;

          default:
            return;
        }
      }
      catch (Exception ex)
      {
        string msg = "An error occurred during personalisation:\n\n";
        msg += ex.Message;

        MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        return;
      }
      finally
      {
        myCard.Disconnect();
      }

      panEncodeComplete.Show();
    }

    private void EncodeProfile1()
    {
      // Generate keysets
      List<PlaidKey> keysets = new List<PlaidKey>();

      // Load the Admin keyset
      {
        PlaidKey k = new PlaidKey();
        k.KeysetId = Settings.Default.Keyset0;
        k.IAKey = myKeyStorage.IAKeyAdmin;
        k.FAKey = myKeyStorage.FAKeyAdmin;

        keysets.Add(k);
      }

      // Load the Target keyset
      {
        PlaidKey k = new PlaidKey();
        k.KeysetId = Settings.Default.Keyset1;
        k.IAKey = myKeyStorage.IAKeyTarget;
        k.FAKey = myKeyStorage.FAKeyTarget;

        keysets.Add(k);
      }

      // Personalise the PLAID SAM
      PlaidSamApplication app = new PlaidSamApplication();
      app.SetCard(myCard);
      app.SelectApplication();

      app.Personalise(keysets, true);
    }

    private void EncodeProfile2()
    {
      // Generate keysets
      List<PlaidKey> keysets = new List<PlaidKey>();

      // Load the Target keyset
      {
        PlaidKey k = new PlaidKey();
        k.KeysetId = Settings.Default.Keyset1;
        k.IAKey = myKeyStorage.IAKeyTarget;
        k.FAKey = myKeyStorage.FAKeyTarget;

        keysets.Add(k);
      }

      // Personalise the PLAID SAM
      PlaidSamApplication app = new PlaidSamApplication();
      app.SetCard(myCard);
      app.SelectApplication();
      app.Personalise(keysets, true);
    }

    private void EncodeProfile3()
    {
      /*
       * Validate the MIFARE KEYA+KEYB values
       */

      const int MifareKeyLength = 12; // # of characters for a 48-bit mifare key

      if (String.IsNullOrEmpty(tboKeyA.Text)
            || tboKeyA.Text.Length != MifareKeyLength
            || !Converters.IsHexString(tboKeyA.Text))
      {
        string msg = @"Key A must be a valid 12-character hexidecimal string.";
        throw new InvalidOperationException(msg);
      }

      if (String.IsNullOrEmpty(tboKeyB.Text)
            || tboKeyB.Text.Length != MifareKeyLength
            || !Converters.IsHexString(tboKeyB.Text))
      {
        string msg = @"Key B must be a valid 12-character hexidecimal string.";
        throw new InvalidOperationException(msg);
      }

      // Convert the key values
      byte[] keyA = Converters.StringToByteArray(tboKeyA.Text);
      byte[] keyB = Converters.StringToByteArray(tboKeyB.Text);

      // Generate the PLAID personalisation record
      PlaidPersonalisationRecord record = new PlaidPersonalisationRecord();

      // Random DivData
      record.DivData = Crypto.CreateRandomEntropy(PlaidApplication.DivDataLength);

      // ACSRecord 1 - Stored IA key - Admin
      {
        AcsRecord acs = new AcsRecord();
        acs.OpModeId = Settings.Default.OpModeKSIAAdmin;
        acs.Data = myKeyStorage.IAKeyAdmin.Modulus;

        record.AcsRecords.Add(acs);
      }

      // ACSRecord 2 - Stored FA key - Admin
      {
        AcsRecord acs = new AcsRecord();
        acs.OpModeId = Settings.Default.OpModeKSFAAdmin;
        acs.Data = myKeyStorage.FAKeyAdmin;

        record.AcsRecords.Add(acs);
      }

      // ACSRecord 3 - Stored IA key - Target
      {
        AcsRecord acs = new AcsRecord();
        acs.OpModeId = Settings.Default.OpModeKSIATarget;
        acs.Data = myKeyStorage.IAKeyTarget.Modulus;

        record.AcsRecords.Add(acs);
      }

      // ACSRecord 4 - Stored FA key - Target
      {
        AcsRecord acs = new AcsRecord();
        acs.OpModeId = Settings.Default.OpModeKSFATarget;
        acs.Data = myKeyStorage.FAKeyTarget;

        record.AcsRecords.Add(acs);
      }

      // ACSRecord 5 - MIFARE KeyA + KeyB
      {
        AcsRecord acs = new AcsRecord();
        acs.OpModeId = Settings.Default.OpModeKSMifare;
        acs.Data = new byte[keyA.Length + keyB.Length];
        Array.Copy(keyA, 0, acs.Data, 0, keyA.Length);
        Array.Copy(keyB, 0, acs.Data, keyA.Length, keyB.Length);

        record.AcsRecords.Add(acs);
      }

      // Admin keyset
      {
        PlaidKey k = new PlaidKey();
        k.KeysetId = Settings.Default.Keyset0;
        k.IAKey = myKeyStorage.IAKeyAdmin;
        k.FAKey = myKeyStorage.FAKeyAdmin;
        record.Keys.Add(k);
      }

      // Personalise the PLAID applet
      PlaidApplication app = new PlaidApplication();
      app.SetCard(myCard);
      app.SelectApplication();
      app.Personalise(record, true);
    }

    private void cboReader_SelectedIndexChanged(object sender, EventArgs e)
    {
      myCard = null;
      pboReader.Hide();

      if (!myPCSC.Contains(cboReader.Text)) return;

      if (myPCSC[cboReader.Text].IsCardPresent)
      {
        myCard = myPCSC[cboReader.Text].ActiveCard;
        ValidateCard();
      }
    }

    private void panSelectReader_Paint(object sender, PaintEventArgs e)
    {

    }

    private void btnImportKeys_Click(object sender, EventArgs e)
    {
      try
      {
        myKeyStorage = KeyStorageRecord.LoadParts(Settings.Default.KeyStoragePath, Settings.Default.KeyStoragePrefix);

        // Notify the user of the archival
        string msg = "The keys have been loaded from the archive path:\n\n";
        msg += string.Format("{0}\\{1}-1of3.XML\n", Settings.Default.KeyStoragePath, Settings.Default.KeyStoragePrefix);
        msg += string.Format("{0}\\{1}-2of3.XML\n", Settings.Default.KeyStoragePath, Settings.Default.KeyStoragePrefix);
        msg += string.Format("{0}\\{1}-3of3.XML", Settings.Default.KeyStoragePath, Settings.Default.KeyStoragePrefix);
        MessageBox.Show(msg, "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
        
        // Enable the next step
        this.UseWaitCursor = false;
        btnGenerateKeys.Enabled = false;
        btnImportKeys.Enabled = false;
        panEncodingProfile.Show();
        lblKeyLoadSuccess.Show();
        btnExport.Show();
      }
      catch (Exception ex)
      {
        string msg = string.Format("The keys could not be loaded.\nError: {0}", ex.Message);
        MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    private void chkShowMifare_CheckedChanged(object sender, EventArgs e)
    {
      tboKeyA.UseSystemPasswordChar = !chkShowMifare.Checked;
      tboKeyB.UseSystemPasswordChar = !chkShowMifare.Checked;
    }

    private void btnExport_Click(object sender, EventArgs e)
    {
      try
      {
        // Notify the user of the archival
        string msg = "";
        msg += "WARNING:\n\nStorage of this plain text record is not recommended except for validation purposes.\n";
        msg += "Use the 3-part format to securely store the key parts in separate physical/logical locations.\n\n";
        msg += "Do you wish to continue?";
        if (MessageBox.Show(msg, "Information", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.No) return;

        string path = string.Format(@"{0}\{1}-PLAIN.Xml", Settings.Default.KeyStoragePath, Settings.Default.KeyStoragePrefix);
        KeyStorageRecord.SaveSingle(path, myKeyStorage);

        // Notify the user of the archival
        msg = "The keys have been saved in plain text to the archive path:\n\n" + path + "\n\n";
        MessageBox.Show(msg, "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
      }
      catch (Exception ex)
      {
        string msg = string.Format("The keys could not be saved.\nError: {0}", ex.Message);
        MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }
  }
}
