﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using CardFramework;
using CardFramework.Applications.GlobalPlatform;
using CardFramework.Applications.Fips201;
using CardFramework.Readers.PCSC;
using PACManager.Properties;
using System.Threading;
using CardFramework.Helpers;
using CardFramework.Applications.Plaid;
using System.Security.Cryptography;
using CardFramework.Protocols.Iso7816;
using System.Reflection;
using System.IO;
using System.Media;

namespace PACManager.Controls
{
  public partial class ProcessScreen : PACManager.Controls.WizardScreen
  {
    public ProcessScreen()
    {
      InitializeComponent();

      myContext = SynchronizationContext.Current;
    }

    private void ProcessScreen_Load(object sender, EventArgs e)
    {
    }

    private SoundPlayer myIntroSound;
    private SoundPlayer myPassSound;
    private SoundPlayer myFailSound;
    private SoundPlayer mySkipSound;

    public override void Startup()
    {
      base.Startup();

      // Create the card processor
      myProcessor = new CardProcessor();
      myProcessor.KeyAuthenticatorReader = Settings.Default.PlaidSamReader;
      myProcessor.KeyStorageReader = Settings.Default.PlaidReader;
      myProcessor.TargetReader = Settings.Default.TargetReader;
      myProcessor.OnProcessEvent += new EventHandler<CardProcessorEventArgs>(myProcessor_OnProcessEvent);

      myProcessor.Connect();

      // Setup the sound engine
      Assembly a = Assembly.GetExecutingAssembly();

      if (File.Exists(Settings.Default.SoundIntroPath))
      {
        myIntroSound = new SoundPlayer(Settings.Default.SoundIntroPath);
        myIntroSound.Load();
      }

      myPassSound = new SoundPlayer(Settings.Default.SoundPassPath);
      myPassSound.Load();
      myFailSound = new SoundPlayer(Settings.Default.SoundFailPath);
      myFailSound.Load();
      mySkipSound = new SoundPlayer(Settings.Default.SoundSkipPath);
      mySkipSound.Load();


      // By default, go to 'Pause' mode.
      // NOTE: rbPause_CheckedChange doesn't fire here because of the state of the form, so we call ourselves
      rbPause.Checked = true;
      rbPause_CheckedChanged(null, null);
    }

    void myProcessor_OnProcessEvent(object sender, CardProcessorEventArgs e)
    {

      switch (e.Level)
      {
        case CardProcessorEventLevel.Info:
          WriteConsole(e.Message, Color.Green);
          break;

        case CardProcessorEventLevel.Error:
          WriteConsole(e.Message, Color.Red);
          break;

        case CardProcessorEventLevel.Verbose:
          if (!chkVerbose.Checked) return;
          WriteConsole(e.Message, Color.Orange);
          break;

        case CardProcessorEventLevel.CardState:
          myContext.Post(delegate(Object obj)
          {
            lblStatus.Text = e.Message;
            Application.DoEvents();
          }, null);
          break;

        case CardProcessorEventLevel.Progress:
          myContext.Post(delegate(Object obj)
          {
            //pgProcess.Visible = (e.Progress != 0);
            pgProcess.Value = e.Progress;
            Application.DoEvents();
          }, null);
          break;
      }

      myContext.Post(delegate(Object obj)
      {
        switch (e.Result)
        {
          case CardProcessorResult.Succeeded:
            lblPassCount.Text = string.Format("{0:000}", (Convert.ToInt32(lblPassCount.Text) + 1));
            myPassSound.Play();
            break;

          case CardProcessorResult.Failed:
            lblFailCount.Text = string.Format("{0:000}", (Convert.ToInt32(lblFailCount.Text) + 1));
            myFailSound.Play();
            break;

          case CardProcessorResult.Skipped:
            lblSkipCount.Text = string.Format("{0:000}", (Convert.ToInt32(lblSkipCount.Text) + 1));
            mySkipSound.Play();
            break;

          case CardProcessorResult.Stopped:
            rbPause.Checked = true;
            break;
        }
      }, null);
    }

    public override void Shutdown()
    {
      base.Shutdown();

      myProcessor.Disconnect();

      if (myIntroSound != null) myIntroSound.Dispose();
      if (myPassSound != null) myPassSound.Dispose();
      if (myFailSound != null) myFailSound.Dispose();
      if (mySkipSound != null) mySkipSound.Dispose();
    }

    public void UpdateProgress(int percentage)
    {
        RichTextBox box = rtbConsole;

        myContext.Post(delegate(Object obj)
        {
          pgProcess.Value = percentage;
          System.Windows.Forms.Application.DoEvents();
        }, null);
    }

    public bool CanEncode
    {
      get { return rbEncode.Enabled; }
      set { rbEncode.Enabled = value; }
    }

    public void WriteConsole(string text)
    {
      WriteConsole(text, false, Color.Black);
    }

    public void WriteConsole(string text, Boolean bold)
    {
      WriteConsole(text, bold, Color.Black);
    }

    public void WriteConsole(string text, Color color)
    {
      WriteConsole(text, false, color);
    }

    public void WriteConsole(string text, Boolean bold, Color color)
    {
      lock (mySaveThreadLock)
      {
        RichTextBox box = rtbConsole;

        myContext.Post(delegate(Object obj)
        {
          int start = box.TextLength;
          box.AppendText("> ");
          box.AppendText(text);
          box.AppendText(Environment.NewLine);
          int end = box.TextLength;

          // Textbox may transform chars, so (end-start) != text.Length
          box.Select(start, end - start);
          box.SelectionColor = color;
          if (bold) box.SelectionFont = new Font(box.SelectionFont, FontStyle.Bold);
          box.SelectionStart = box.TextLength;
          box.ScrollToCaret();
          box.SelectionLength = 0;
        }, null);

        System.Windows.Forms.Application.DoEvents();
      }
    }

    // Thread synchronization objects
    private SynchronizationContext myContext = null;
    private Object mySaveThreadLock = new Object();

    private CardProcessor myProcessor = null;

    private void rbEncode_CheckedChanged(object sender, EventArgs e)
    {
      if (!rbEncode.Checked) return;

      this.UseWaitCursor = true;

      if (myIntroSound != null) myIntroSound.Play();
      myProcessor.SetMode(CardProcessorMode.Encode);

      lblModeTitle.Text = "Encoding Mode";
      lblModeDescription.Text = "Please place an INITIALISED target card on the reader to begin the encode process.";
      lblStatus.Text = "Waiting for card ...";
      panStatus.Enabled = true;
      this.UseWaitCursor = false;
    }

    private void rbVerify_CheckedChanged(object sender, EventArgs e)
    {
      if (!rbVerify.Checked) return;

      this.UseWaitCursor = true;
      myProcessor.SetMode(CardProcessorMode.Verify);

      lblModeTitle.Text = "Verification Mode";
      lblModeDescription.Text = "Please place a PERSONALISED target card on the reader to begin the verification process.";
      lblStatus.Text = "Waiting for card ...";
      panStatus.Enabled = true;
      this.UseWaitCursor = false;
    }

    private void rbPause_CheckedChanged(object sender, EventArgs e)
    {
      if (!rbPause.Checked) return;

      myProcessor.SetMode(CardProcessorMode.Pause);

      lblModeTitle.Text = "Pause";
      lblModeDescription.Text = "Please select an operational mode to begin.";
      lblStatus.Text = "Idle ...";
      panStatus.Enabled = false;

    }

    private void panel2_Paint(object sender, PaintEventArgs e)
    {

    }

    private void pgProcess_Click(object sender, EventArgs e)
    {

    }
  }
}
