﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Security.Cryptography;
using System.IO;
using CardFramework.Helpers;
using System.Xml.Serialization;
using CardFramework.Applications.Plaid;
using PACManager.Properties;

namespace PACManager
{
  /// <summary>
  /// Stores the key material in an import/export format for the key management screen.
  /// </summary>
  public class KeyStorageRecord
  {
    public const int RSALength = 1024;
    public const int AESLength = 128;
    
    [XmlAttribute("part")]
    public int Part;

    [XmlAttribute("part_count")]
    public int PartCount;

    public RSAKey IAKeyAdmin;

    [XmlElement(DataType = "hexBinary")]
    public byte[] FAKeyAdmin;

    public RSAKey IAKeyTarget;

    [XmlElement(DataType = "hexBinary")]
    public byte[] FAKeyTarget;

    [XmlElement(DataType = "hexBinary")]
    public byte[] IAKeyAdminHash;

    [XmlElement(DataType = "hexBinary")]
    public byte[] FAKeyAdminHash;

    [XmlElement(DataType = "hexBinary")]
    public byte[] IAKeyTargetHash;

    [XmlElement(DataType = "hexBinary")]
    public byte[] FAKeyTargetHash;


    public static KeyStorageRecord Generate()
    {
      KeyStorageRecord r = new KeyStorageRecord();

      /*
       * STEP 1 - Generate PLAID keysets
       */

      // Key 0 - 'Initial Authenticate' Key - Administrative (Keyset 0000) 
      RSAParameters rsaAdmin = new RSACryptoServiceProvider(RSALength).ExportParameters(true);
      r.IAKeyAdmin = new RSAKey(rsaAdmin);

      // Key 1 - 'Initial Authenticate' Key - Target (Keyset 1111)      
      RSAParameters rsaTarget = new RSACryptoServiceProvider(RSALength).ExportParameters(true);
      r.IAKeyTarget = new RSAKey(rsaTarget);

      // Key 2 - 'Final Authenticate' Key - Administrative (Keyset 0000)
      AesCryptoServiceProvider aesAdmin = new AesCryptoServiceProvider();
      aesAdmin.KeySize = AESLength;
      aesAdmin.GenerateKey();
      r.FAKeyAdmin = new byte[aesAdmin.Key.Length];
      Array.Copy(aesAdmin.Key, r.FAKeyAdmin, aesAdmin.Key.Length);

      // Key 3 - 'Final Authenticate' Key - Target (Keyset 1111)
      AesCryptoServiceProvider aesTarget = new AesCryptoServiceProvider();
      aesTarget.KeySize = AESLength;
      aesTarget.GenerateKey();
      r.FAKeyTarget = new byte[aesTarget.Key.Length];
      Array.Copy(aesTarget.Key, r.FAKeyTarget, aesTarget.Key.Length);

      return r;
    }

    public static void SaveSingle(string path, KeyStorageRecord instance)
    {
      /*
       * Save this as a non-split instance
       */
      instance.Part = 1;
      instance.PartCount = 1;
      instance.ComputeHashes();
      Serialiser<KeyStorageRecord>.Save(path, instance);
    }

    public static KeyStorageRecord LoadSingle(string path)
    {
      KeyStorageRecord instance = Serialiser<KeyStorageRecord>.Load(path);
      if (!instance.VerifyHashes()) throw new InvalidOperationException("Key storage record failed hash verification");
      if (instance.Part != 1 || instance.PartCount != 1) throw new InvalidOperationException("Targeted path is a multi-part key, cannot load as a single.");
      return instance;
    }

    public void ComputeHashes()
    {
      SHA1CryptoServiceProvider sha1 = new SHA1CryptoServiceProvider();

      // IA - Admin
      IAKeyAdminHash = IAKeyAdmin.ComputeHash();
      IAKeyTargetHash = IAKeyTarget.ComputeHash();
      FAKeyAdminHash = sha1.ComputeHash(FAKeyAdmin);
      FAKeyTargetHash = sha1.ComputeHash(FAKeyTarget);
    }

    public bool VerifyHashes()
    {
      SHA1CryptoServiceProvider sha1 = new SHA1CryptoServiceProvider();
      byte[] hash;

      // IA - Admin
      hash = IAKeyAdmin.ComputeHash();
      if (!Converters.CompareArray<byte>(hash, IAKeyAdminHash)) return false;

      // IA Key - Target
      hash = IAKeyTarget.ComputeHash();
      if (!Converters.CompareArray<byte>(hash, IAKeyTargetHash)) return false;

      // FA Key - Admin
      hash = sha1.ComputeHash(FAKeyAdmin);
      if (!Converters.CompareArray<byte>(hash, FAKeyAdminHash)) return false;

      // FA Key - Target
      hash = sha1.ComputeHash(FAKeyTarget);
      if (!Converters.CompareArray<byte>(hash, FAKeyTargetHash)) return false;

      return true;
    }

    /// <summary>
    /// Splits the supplied key storage record into 3 securely separated parts, which
    /// will need to be combined before the key can be recovered.
    /// </summary>
    /// <param name="record"></param>
    /// <returns></returns>
    public static void SaveParts(KeyStorageRecord record, string directory, string prefix)
    {
      KeyStorageRecord a = new KeyStorageRecord();
      KeyStorageRecord b = new KeyStorageRecord();
      KeyStorageRecord c = new KeyStorageRecord();

      a.Part = 1;
      a.PartCount = 3;

      b.Part = 2;
      b.PartCount = 3;

      c.Part = 3;
      c.PartCount = 3;

      /*
       * Generate the hashes for the record we are about to split
       */
      record.ComputeHashes();

      /*
       * IA Key - ADMIN
       */
      a.IAKeyAdmin = new RSAKey();
      b.IAKeyAdmin = new RSAKey();
      c.IAKeyAdmin = new RSAKey();

      // P
      a.IAKeyAdmin.P = Crypto.CreateRandomEntropy(record.IAKeyAdmin.P.Length);
      b.IAKeyAdmin.P = Crypto.CreateRandomEntropy(record.IAKeyAdmin.P.Length);
      c.IAKeyAdmin.P = Crypto.XorArray(Crypto.XorArray(record.IAKeyAdmin.P, a.IAKeyAdmin.P), b.IAKeyAdmin.P);

      // Q
      a.IAKeyAdmin.Q = Crypto.CreateRandomEntropy(record.IAKeyAdmin.Q.Length);
      b.IAKeyAdmin.Q = Crypto.CreateRandomEntropy(record.IAKeyAdmin.Q.Length);
      c.IAKeyAdmin.Q = Crypto.XorArray(Crypto.XorArray(record.IAKeyAdmin.Q, a.IAKeyAdmin.Q), b.IAKeyAdmin.Q);

      // PQ
      a.IAKeyAdmin.PQ = Crypto.CreateRandomEntropy(record.IAKeyAdmin.PQ.Length);
      b.IAKeyAdmin.PQ = Crypto.CreateRandomEntropy(record.IAKeyAdmin.PQ.Length);
      c.IAKeyAdmin.PQ = Crypto.XorArray(Crypto.XorArray(record.IAKeyAdmin.PQ, a.IAKeyAdmin.PQ), b.IAKeyAdmin.PQ);

      // D
      a.IAKeyAdmin.D = Crypto.CreateRandomEntropy(record.IAKeyAdmin.D.Length);
      b.IAKeyAdmin.D = Crypto.CreateRandomEntropy(record.IAKeyAdmin.D.Length);
      c.IAKeyAdmin.D = Crypto.XorArray(Crypto.XorArray(record.IAKeyAdmin.D, a.IAKeyAdmin.D), b.IAKeyAdmin.D);

      // DQ
      a.IAKeyAdmin.DQ = Crypto.CreateRandomEntropy(record.IAKeyAdmin.DQ.Length);
      b.IAKeyAdmin.DQ = Crypto.CreateRandomEntropy(record.IAKeyAdmin.DQ.Length);
      c.IAKeyAdmin.DQ = Crypto.XorArray(Crypto.XorArray(record.IAKeyAdmin.DQ, a.IAKeyAdmin.DQ), b.IAKeyAdmin.DQ);

      // DP
      a.IAKeyAdmin.DP = Crypto.CreateRandomEntropy(record.IAKeyAdmin.DP.Length);
      b.IAKeyAdmin.DP = Crypto.CreateRandomEntropy(record.IAKeyAdmin.DP.Length);
      c.IAKeyAdmin.DP = Crypto.XorArray(Crypto.XorArray(record.IAKeyAdmin.DP, a.IAKeyAdmin.DP), b.IAKeyAdmin.DP);

      // Modulus
      a.IAKeyAdmin.Modulus = Crypto.CreateRandomEntropy(record.IAKeyAdmin.Modulus.Length);
      b.IAKeyAdmin.Modulus = Crypto.CreateRandomEntropy(record.IAKeyAdmin.Modulus.Length);
      c.IAKeyAdmin.Modulus = Crypto.XorArray(Crypto.XorArray(record.IAKeyAdmin.Modulus, a.IAKeyAdmin.Modulus), b.IAKeyAdmin.Modulus);

      // Exponent
      a.IAKeyAdmin.Exponent = Crypto.CreateRandomEntropy(record.IAKeyAdmin.Exponent.Length);
      b.IAKeyAdmin.Exponent = Crypto.CreateRandomEntropy(record.IAKeyAdmin.Exponent.Length);
      c.IAKeyAdmin.Exponent = Crypto.XorArray(Crypto.XorArray(record.IAKeyAdmin.Exponent, a.IAKeyAdmin.Exponent), b.IAKeyAdmin.Exponent);


      /*
       * FA Key - ADMIN
       */
      a.FAKeyAdmin = Crypto.CreateRandomEntropy(record.FAKeyAdmin.Length);
      b.FAKeyAdmin = Crypto.CreateRandomEntropy(record.FAKeyAdmin.Length);
      c.FAKeyAdmin = Crypto.XorArray(Crypto.XorArray(record.FAKeyAdmin, a.FAKeyAdmin), b.FAKeyAdmin);


      /*
       * IA Key - TARGET
       */
      a.IAKeyTarget = new RSAKey();
      b.IAKeyTarget = new RSAKey();
      c.IAKeyTarget = new RSAKey();

      // P
      a.IAKeyTarget.P = Crypto.CreateRandomEntropy(record.IAKeyTarget.P.Length);
      b.IAKeyTarget.P = Crypto.CreateRandomEntropy(record.IAKeyTarget.P.Length);
      c.IAKeyTarget.P = Crypto.XorArray(Crypto.XorArray(record.IAKeyTarget.P, a.IAKeyTarget.P), b.IAKeyTarget.P);

      // Q
      a.IAKeyTarget.Q = Crypto.CreateRandomEntropy(record.IAKeyTarget.Q.Length);
      b.IAKeyTarget.Q = Crypto.CreateRandomEntropy(record.IAKeyTarget.Q.Length);
      c.IAKeyTarget.Q = Crypto.XorArray(Crypto.XorArray(record.IAKeyTarget.Q, a.IAKeyTarget.Q), b.IAKeyTarget.Q);

      // PQ
      a.IAKeyTarget.PQ = Crypto.CreateRandomEntropy(record.IAKeyTarget.PQ.Length);
      b.IAKeyTarget.PQ = Crypto.CreateRandomEntropy(record.IAKeyTarget.PQ.Length);
      c.IAKeyTarget.PQ = Crypto.XorArray(Crypto.XorArray(record.IAKeyTarget.PQ, a.IAKeyTarget.PQ), b.IAKeyTarget.PQ);

      // D
      a.IAKeyTarget.D = Crypto.CreateRandomEntropy(record.IAKeyTarget.D.Length);
      b.IAKeyTarget.D = Crypto.CreateRandomEntropy(record.IAKeyTarget.D.Length);
      c.IAKeyTarget.D = Crypto.XorArray(Crypto.XorArray(record.IAKeyTarget.D, a.IAKeyTarget.D), b.IAKeyTarget.D);

      // DQ
      a.IAKeyTarget.DQ = Crypto.CreateRandomEntropy(record.IAKeyTarget.DQ.Length);
      b.IAKeyTarget.DQ = Crypto.CreateRandomEntropy(record.IAKeyTarget.DQ.Length);
      c.IAKeyTarget.DQ = Crypto.XorArray(Crypto.XorArray(record.IAKeyTarget.DQ, a.IAKeyTarget.DQ), b.IAKeyTarget.DQ);

      // DP
      a.IAKeyTarget.DP = Crypto.CreateRandomEntropy(record.IAKeyTarget.DP.Length);
      b.IAKeyTarget.DP = Crypto.CreateRandomEntropy(record.IAKeyTarget.DP.Length);
      c.IAKeyTarget.DP = Crypto.XorArray(Crypto.XorArray(record.IAKeyTarget.DP, a.IAKeyTarget.DP), b.IAKeyTarget.DP);

      // Modulus
      a.IAKeyTarget.Modulus = Crypto.CreateRandomEntropy(record.IAKeyTarget.Modulus.Length);
      b.IAKeyTarget.Modulus = Crypto.CreateRandomEntropy(record.IAKeyTarget.Modulus.Length);
      c.IAKeyTarget.Modulus = Crypto.XorArray(Crypto.XorArray(record.IAKeyTarget.Modulus, a.IAKeyTarget.Modulus), b.IAKeyTarget.Modulus);

      // Exponent
      a.IAKeyTarget.Exponent = Crypto.CreateRandomEntropy(record.IAKeyTarget.Exponent.Length);
      b.IAKeyTarget.Exponent = Crypto.CreateRandomEntropy(record.IAKeyTarget.Exponent.Length);
      c.IAKeyTarget.Exponent = Crypto.XorArray(Crypto.XorArray(record.IAKeyTarget.Exponent, a.IAKeyTarget.Exponent), b.IAKeyTarget.Exponent);


      /*
       * FA Key - TARGET
       */

      a.FAKeyTarget = Crypto.CreateRandomEntropy(record.FAKeyTarget.Length);
      b.FAKeyTarget = Crypto.CreateRandomEntropy(record.FAKeyTarget.Length);
      c.FAKeyTarget = Crypto.XorArray(Crypto.XorArray(record.FAKeyTarget, a.FAKeyTarget), b.FAKeyTarget);

      /*
       * Transfer the hashes across
       */
      a.IAKeyAdminHash = record.IAKeyAdminHash;
      a.FAKeyAdminHash = record.FAKeyAdminHash;
      a.IAKeyTargetHash = record.IAKeyTargetHash;
      a.FAKeyTargetHash = record.FAKeyTargetHash;

      b.IAKeyAdminHash = record.IAKeyAdminHash;
      b.FAKeyAdminHash = record.FAKeyAdminHash;
      b.IAKeyTargetHash = record.IAKeyTargetHash;
      b.FAKeyTargetHash = record.FAKeyTargetHash;

      c.IAKeyAdminHash = record.IAKeyAdminHash;
      c.FAKeyAdminHash = record.FAKeyAdminHash;
      c.IAKeyTargetHash = record.IAKeyTargetHash;
      c.FAKeyTargetHash = record.FAKeyTargetHash;

      /*
       * Save the resulting 3-parts
       */
      DirectoryInfo dir = new DirectoryInfo(directory);
      if (!dir.Exists) throw new InvalidOperationException("Specified path is invalid.");

      string pathA = string.Format(@"{0}\{1}-{2}of{3}.XML", dir.FullName, prefix, "1", "3");
      string pathB = string.Format(@"{0}\{1}-{2}of{3}.XML", dir.FullName, prefix, "2", "3");
      string pathC = string.Format(@"{0}\{1}-{2}of{3}.XML", dir.FullName, prefix, "3", "3");

      Serialiser<KeyStorageRecord>.Save(pathA, a);
      Serialiser<KeyStorageRecord>.Save(pathB, b);
      Serialiser<KeyStorageRecord>.Save(pathC, c);
    }

    public static KeyStorageRecord LoadParts(string directory, string prefix)
    {
      KeyStorageRecord r = new KeyStorageRecord();

      DirectoryInfo dir = new DirectoryInfo(directory);
      if (!dir.Exists) throw new InvalidOperationException("Specified path is invalid.");

      string pathA = string.Format(@"{0}\{1}-{2}of{3}.XML", dir.FullName, prefix, "1", "3");
      string pathB = string.Format(@"{0}\{1}-{2}of{3}.XML", dir.FullName, prefix, "2", "3");
      string pathC = string.Format(@"{0}\{1}-{2}of{3}.XML", dir.FullName, prefix, "3", "3");

      KeyStorageRecord a = Serialiser<KeyStorageRecord>.Load(pathA);
      KeyStorageRecord b = Serialiser<KeyStorageRecord>.Load(pathB);
      KeyStorageRecord c = Serialiser<KeyStorageRecord>.Load(pathC);

      /*
       * Confirm the pieces are for the same keyset by ID and hash
       */
      if (a.Part != 1 || b.Part != 2 || c.Part != 3)
      {
        throw new ArgumentException("Invalid part identifiers");
      }

      if (!Converters.CompareArray<byte>(a.IAKeyAdminHash, b.IAKeyAdminHash) || !Converters.CompareArray<byte>(b.IAKeyAdminHash, c.IAKeyAdminHash))
      {
        throw new ArgumentException("Hash comparison failed for key 'IA Key - Admin'");
      }

      if (!Converters.CompareArray<byte>(a.IAKeyTargetHash, b.IAKeyTargetHash) || !Converters.CompareArray<byte>(b.IAKeyTargetHash, c.IAKeyTargetHash))
      {
        throw new ArgumentException("Hash comparison failed for key 'IA Key - Target'");
      }

      if (!Converters.CompareArray<byte>(a.FAKeyAdminHash, b.FAKeyAdminHash) || !Converters.CompareArray<byte>(b.FAKeyAdminHash, c.FAKeyAdminHash))
      {
        throw new ArgumentException("Hash comparison failed for key 'FA Key - Admin'");
      }

      if (!Converters.CompareArray<byte>(a.FAKeyTargetHash, b.FAKeyTargetHash) || !Converters.CompareArray<byte>(b.FAKeyTargetHash, c.FAKeyTargetHash))
      {
        throw new ArgumentException("Hash comparison failed for key 'FA Key - Target'");
      }


      /*
       * IA Key - ADMIN
       */
      r.IAKeyAdmin = new RSAKey();

      // P
      r.IAKeyAdmin.P = Crypto.XorArray(Crypto.XorArray(c.IAKeyAdmin.P, b.IAKeyAdmin.P), a.IAKeyAdmin.P);

      // Q
      r.IAKeyAdmin.Q = Crypto.XorArray(Crypto.XorArray(c.IAKeyAdmin.Q, b.IAKeyAdmin.Q), a.IAKeyAdmin.Q);

      // PQ
      r.IAKeyAdmin.PQ = Crypto.XorArray(Crypto.XorArray(c.IAKeyAdmin.PQ, b.IAKeyAdmin.PQ), a.IAKeyAdmin.PQ);

      // D
      r.IAKeyAdmin.D = Crypto.XorArray(Crypto.XorArray(c.IAKeyAdmin.D, b.IAKeyAdmin.D), a.IAKeyAdmin.D);

      // DQ
      r.IAKeyAdmin.DQ = Crypto.XorArray(Crypto.XorArray(c.IAKeyAdmin.DQ, b.IAKeyAdmin.DQ), a.IAKeyAdmin.DQ);

      // DP
      r.IAKeyAdmin.DP = Crypto.XorArray(Crypto.XorArray(c.IAKeyAdmin.DP, b.IAKeyAdmin.DP), a.IAKeyAdmin.DP);

      // Modulus
      r.IAKeyAdmin.Modulus = Crypto.XorArray(Crypto.XorArray(c.IAKeyAdmin.Modulus, b.IAKeyAdmin.Modulus), a.IAKeyAdmin.Modulus);

      // Exponent
      r.IAKeyAdmin.Exponent = Crypto.XorArray(Crypto.XorArray(c.IAKeyAdmin.Exponent, b.IAKeyAdmin.Exponent), a.IAKeyAdmin.Exponent);

      /*
       * FA Key - ADMIN
       */

      r.FAKeyAdmin = Crypto.XorArray(Crypto.XorArray(c.FAKeyAdmin, b.FAKeyAdmin), a.FAKeyAdmin);


      /*
       * IA Key - TARGET
       */
      r.IAKeyTarget = new RSAKey();

      // P
      r.IAKeyTarget.P = Crypto.XorArray(Crypto.XorArray(c.IAKeyTarget.P, b.IAKeyTarget.P), a.IAKeyTarget.P);

      // Q
      r.IAKeyTarget.Q = Crypto.XorArray(Crypto.XorArray(c.IAKeyTarget.Q, b.IAKeyTarget.Q), a.IAKeyTarget.Q);

      // PQ
      r.IAKeyTarget.PQ = Crypto.XorArray(Crypto.XorArray(c.IAKeyTarget.PQ, b.IAKeyTarget.PQ), a.IAKeyTarget.PQ);

      // D
      r.IAKeyTarget.D = Crypto.XorArray(Crypto.XorArray(c.IAKeyTarget.D, b.IAKeyTarget.D), a.IAKeyTarget.D);

      // DQ
      r.IAKeyTarget.DQ = Crypto.XorArray(Crypto.XorArray(c.IAKeyTarget.DQ, b.IAKeyTarget.DQ), a.IAKeyTarget.DQ);

      // DP
      r.IAKeyTarget.DP = Crypto.XorArray(Crypto.XorArray(c.IAKeyTarget.DP, b.IAKeyTarget.DP), a.IAKeyTarget.DP);

      // Modulus
      r.IAKeyTarget.Modulus = Crypto.XorArray(Crypto.XorArray(c.IAKeyTarget.Modulus, b.IAKeyTarget.Modulus), a.IAKeyTarget.Modulus);

      // Exponent
      r.IAKeyTarget.Exponent = Crypto.XorArray(Crypto.XorArray(c.IAKeyTarget.Exponent, b.IAKeyTarget.Exponent), a.IAKeyTarget.Exponent);

      /*
       * FA Key - TARGET
       */

      r.FAKeyTarget = Crypto.XorArray(Crypto.XorArray(c.FAKeyTarget, b.FAKeyTarget), a.FAKeyTarget);

      /*
       * Validate the SHA-1 hash
       */
      r.IAKeyAdminHash = a.IAKeyAdminHash;
      r.FAKeyAdminHash = a.FAKeyAdminHash;
      r.IAKeyTargetHash = a.IAKeyTargetHash;
      r.FAKeyTargetHash = a.FAKeyTargetHash;


      if (!r.VerifyHashes()) throw new InvalidOperationException("Hash check failed on combined key part, key merging failed.");

      return r;
    }
  }
}
