// --------------------------------------------------------------------------- 
// Protocol for Lightweight Authentication of Identity (PLAID)
// 
// SAM applet
// 
// Reference Implementation compliant with AS 5185 - Javacard 2.x source code
// 
// --------------------------------------------------------------------------- 
// This implementation: Copyright Australian Government
// PLAID: Copyright Australian Government
// 
// A copy of the entire Licence is available upon email request from 
// plaid@humanservices.gov.au or by download from https://www.plaid.gov.au 
// 
// Subject to the terms of the Licence, the Australian Government grants to 
// the User a perpetual, irrevocable, world-wide, non-exclusive, royalty free and 
// no-charge licence to use, reproduce, adapt, modify, enhance, communicate, 
// sub-license and distribute PLAID and/or its source code. Clause 2.1 includes 
// the right to incorporate PLAID into any Product developed by the User.
// 
// By using PLAID and/or its source code you agree to be bound by the Licence.
// 
// ---------------------------------------------------------------------------
// Status: Prototype 0.805
// Issue Date: March 2012
// 
// Author: Glenn Mitchell (Australian Government)

package plaidSam;

import javacard.framework.*;
import javacard.security.*;
import javacardx.crypto.*;

public class SAM805 extends Applet
{
  // INS constants
  static final byte INS_INITIAL_AUTHENTICATE = (byte)0x8A;
  static final byte INS_FINAL_AUTHENTICATE = (byte)0x8C;
  static final byte INS_SET_DATA = (byte)0xDB;

  // Data element tag constants
  static final byte TAG_P = (byte)0x01;
  static final byte TAG_Q = (byte)0x02;
  static final byte TAG_DP1 = (byte)0x03;
  static final byte TAG_DQ1 = (byte)0x04;
  static final byte TAG_PQ = (byte)0x05;
  static final byte TAG_FAKEY = (byte)0x06;
  static final byte TAG_SECURED = (byte)0x0A;

  // Length Constants
  static final short LENGTH_P = (short)64;
  static final short LENGTH_Q = (short)64;
  static final short LENGTH_DP1 = (short)64;
  static final short LENGTH_DQ1 = (short)64;
  static final short LENGTH_PQ = (short)64;
  static final short LENGTH_CURRENT_SESSION = (short)2;
  static final short LENGTH_KEYSETID = (short)2;
  static final short LENGTH_BUFFER16 = (short)16;
  static final short LENGTH_BUFFER160 = (short)160;
  static final short LENGTH_RSA1024 = (short)128;
  static final short LENGTH_OPMODE = (short)2;
  static final short LENGTH_DIVDATA = (short)8;
  static final short LENGTH_AES_BLOCK = (short)16;
  static final short LENGTH_AES_KEY = (short)16;
  static final short LENGTH_SHA1_BLOCK = (short)20;
  static final short LENGTH_ESTR2 = (short)(LENGTH_AES_BLOCK * 3);
  static final short LENGTH_STR2 = (short)(LENGTH_OPMODE + LENGTH_AES_BLOCK * 2);
  static final short LENGTH_ESTR2_PADDING = (short)(LENGTH_ESTR2 - LENGTH_STR2);

  // Offsets
  static final short OFFSET_KEYSETID = (short)0;
  static final short OFFSET_KEYSET_INDEX = (short)1;
  static final short OFFSET_VALUE = (short)5;
  static final short OFFSET_DIVDATA = (short)2;
  static final short OFFSET_OPMODE = (short)8;

  static final short OFFSET_RND1A = (short)(LENGTH_KEYSETID + LENGTH_DIVDATA);
  static final short OFFSET_RND1B = (short)(LENGTH_KEYSETID + LENGTH_DIVDATA + LENGTH_AES_BLOCK);
  static final short OFFSET_RND2 = OFFSET_RND1B;
  static final short OFFSET_RND3 = (short)(OFFSET_RND2 + LENGTH_AES_BLOCK);
  static final short OFFSET_FADIV = (short)(OFFSET_RND3 + LENGTH_SHA1_BLOCK);
  static final short OFFSET_STR2 = (short)(OFFSET_RND2 - LENGTH_OPMODE);

  // Misc
  static final short TOTAL_KEYSETS = (short)8;
  static final byte NULL_VALUE = (byte)0x00;
  static final byte COMMAND_VALUE = (byte)0x80;


  /*
   * DEBUG_MODE: If this is set to true, the applet will ignore the 'SECURED' state.
   */
  static final boolean DEBUG_MODE = true;

  /*
   * Control flag values
   */
  
  // FLAG_NONE - A helper value to indicate no flags are set
  static final byte FLAG_NONE		= (byte)0x00;
  
  // FLAG_SECURED - If set, prohibits any usage of the SET DATA command 
  static final byte FLAG_SECURED	= (byte)0x01;

  // FLAG_DEBUG - If set, the applet will ignore the 'SECURED' state
  static final byte FLAG_DEBUG		= (byte)0x02;
  
  
  // Persistant objects
  private class Keyset
  {
    public short id;
    public RSAPrivateCrtKey iaKey;
    public AESKey faKey;
    public boolean isEmpty;
  }

  // Persistent data
  private Keyset[] keyTable;
  private byte controlFlags;

  // Cryptographic Service Providers
  private Cipher cspAES;
  private Cipher cspRSA;
  private MessageDigest cspSHA1;
  private RandomData cspRNG;

  // Transient objects
  private byte[] Buffer16;
  private byte[] Buffer160;
  private AESKey transientKey;

  public SAM805()
  {
    Buffer16 = JCSystem.makeTransientByteArray(LENGTH_BUFFER16, JCSystem.CLEAR_ON_DESELECT);
    Buffer160 = JCSystem.makeTransientByteArray(LENGTH_BUFFER160, JCSystem.CLEAR_ON_DESELECT);

    // Create our CSP's
    cspAES = Cipher.getInstance(Cipher.ALG_AES_BLOCK_128_CBC_NOPAD, false);
    cspRSA = Cipher.getInstance(Cipher.ALG_RSA_PKCS1, false);
    cspSHA1 = MessageDigest.getInstance(MessageDigest.ALG_SHA, false);
    cspRNG = RandomData.getInstance(RandomData.ALG_SECURE_RANDOM);

    // Build the key table
    keyTable = new Keyset[TOTAL_KEYSETS];
    
    for (short i = 0; i < TOTAL_KEYSETS; i++)
    {
      keyTable[i] = new Keyset();

      keyTable[i].id = (short)0; // Set to an unusable keyset Id
      keyTable[i].iaKey = (RSAPrivateCrtKey)KeyBuilder.buildKey(
        KeyBuilder.TYPE_RSA_CRT_PRIVATE,
        KeyBuilder.LENGTH_RSA_1024, false);
      keyTable[i].faKey = (AESKey)KeyBuilder.buildKey(
        KeyBuilder.TYPE_AES, KeyBuilder.LENGTH_AES_128, false);
      keyTable[i].isEmpty = true;
    }

    // Create the transient (session) key
    transientKey = (AESKey)KeyBuilder.buildKey(KeyBuilder.TYPE_AES_TRANSIENT_DESELECT, 
	    				       KeyBuilder.LENGTH_AES_128, false);

    // Initialise the control flags
    controlFlags = FLAG_NONE;
    if (DEBUG_MODE) controlFlags |= FLAG_DEBUG;
  }

  //  /**
  //   * <b>Description</b>
  //   * <p>
  //   * This method registers this applet instance with ICC's JCRE.
  //   */
  public static void install(byte[] params, short offset, byte length)
	      throws ISOException {
		(new SAM805()).register(params, (short) (offset + 1), params[offset]);
	  }


  public void process(APDU apdu)
  {
    // Ignore the APDU to select this applet
    if (this.selectingApplet()) return;

    // Retrieve and validate the APDU
    byte[] buffer = apdu.getBuffer();

    // CLA
    if (apdu.isISOInterindustryCLA())
    {
      ISOException.throwIt(ISO7816.SW_CLA_NOT_SUPPORTED);
    }

    // P1 (must be zero for all commands)
    if (buffer[ISO7816.OFFSET_P1] != 0)
    {
      ISOException.throwIt(ISO7816.SW_WRONG_P1P2);
    }

    // P2 (must be zero for all commands)
    if (buffer[ISO7816.OFFSET_P2] != 0)
    {
      ISOException.throwIt(ISO7816.SW_WRONG_P1P2);
    }

    // Process the instruction
    switch (buffer[ISO7816.OFFSET_INS])
    {
      case INS_INITIAL_AUTHENTICATE:
	// Validate the secure flags and debug mode
	if ((controlFlags & FLAG_DEBUG) == 0x00) {
	    if ((controlFlags & FLAG_SECURED) == 0x00) {
		ISOException.throwIt(ISO7816.SW_COMMAND_NOT_ALLOWED);
	    }
	}
        initialAuthenticate(apdu);
        return;

      case INS_FINAL_AUTHENTICATE:
	// Validate the secure flags and debug mode
	if ((controlFlags & FLAG_DEBUG) == 0x00) {
	    if ((controlFlags & FLAG_SECURED) == 0x00) {
		ISOException.throwIt(ISO7816.SW_COMMAND_NOT_ALLOWED);
	    }
	}
        finalAuthenticate(apdu);
        return;

      case INS_SET_DATA:
	// Validate the secure flags and debug mode
	if ((controlFlags & FLAG_DEBUG) == 0x00) {
	    if ((controlFlags & FLAG_SECURED) != 0x00) {
		ISOException.throwIt(ISO7816.SW_COMMAND_NOT_ALLOWED);
	    }
	}
        setData(apdu);
        return;

      default:
        ISOException.throwIt(ISO7816.SW_INS_NOT_SUPPORTED);
        break;
    }

  }

  public void initialAuthenticate(APDU apdu)
  {
    byte[] apduBuffer = apdu.getBuffer();
    apdu.setIncomingAndReceive();

    /*
     * COMMAND APDU CDATA: 
     * SEQUENCE; 
     * 	STRING_OCTET (128B); ESTR-1
     * 	STRING_OCTET (2B); OpModeId 
     * 	STRING_OCTET (1B); Keyset Count
     * 	STRING_OCTET (2B); Keyset [0] ... 
     * 	STRING_OCTET (2B); Keyset[Count - 1];
     */

    /*
     * SETUP
     */

    short debug = (short)0x7000;
    
    try
    {
        short offset = ISO7816.OFFSET_CDATA;
        
        // Is this our sequence?
        if (Tlv.getTagNumber(apduBuffer, offset) != Tlv.ASN1_SEQUENCE)
        {
            ISOException.throwIt(ISO7816.SW_DATA_INVALID);
        }
        
        // Get the offset for our ESTR1 value
        short estr1Offset = Tlv.findNext(apduBuffer, offset, Tlv.ASN1_STRING_OCTET);
        
        // Read the requested OpMode
        offset = Tlv.findNext(apduBuffer, estr1Offset, Tlv.ASN1_STRING_OCTET);
        
        short opModeId = Tlv.toShort(apduBuffer, offset);
        
        // Now that we have the OpMode, find the start of our data buffer for ESTR1
        estr1Offset = Tlv.getDataOffset(apduBuffer, estr1Offset);
        
        // Read the keyset count
        offset = Tlv.findNext(apduBuffer, offset, Tlv.ASN1_STRING_OCTET);
        short numKeysets = Tlv.toShort(apduBuffer, offset);

        /*
         * EXECUTION - PLAID (3) - IFD responds to the IA response
         */
        
        // 3a)
        
        // Loop through each supplied keyset
        short keysetId;
        short keyIndex = (byte)0xFF;

        for (short i = 0; i < numKeysets; i++)
        {
        // Read the keyset
        offset = Tlv.findNext(apduBuffer, offset, Tlv.ASN1_STRING_OCTET);
        keysetId = Tlv.toShort(apduBuffer, offset);

        for (short j = 0; j < TOTAL_KEYSETS; j++)
        {
        	// If it's not our keyset, skip
        	if (keysetId != keyTable[j].id) continue;
    
        	// We have a matching keyset, decrypt and test
        	cspRSA.init(keyTable[j].iaKey, Cipher.MODE_DECRYPT);
        	cspRSA.doFinal(apduBuffer, estr1Offset, LENGTH_RSA1024, Buffer160, (short)0);
    
        	/*
        	  * STR1 format: KeysetId (2B) | DivData (8B) | RND1[A] (16B) |
        	  * RND[B] (16B)
        	  */
    
        	// Compare RND1[A] and RND1[B]
        	if (Util.arrayCompare(
        	  Buffer160,
        	  (short)(LENGTH_KEYSETID + LENGTH_DIVDATA),
        	  Buffer160,
        	  (short)(LENGTH_KEYSETID + LENGTH_DIVDATA + LENGTH_AES_BLOCK),
        	  LENGTH_AES_BLOCK) != 0x00)
        	{
        	  ISOException.throwIt(ISO7816.SW_DATA_INVALID);
        	}
        	else
        	{
        	  // We have a match
        	  keyIndex = j;
        	  debug++;
    
        	  // 3.b) The IFD extracts the KeySetId from STR1 to confirm
        	  // which key set is being used
        	  keysetId = Util.getShort(Buffer160, (short)0);
        	  debug++;
        	  break;
        	}
    
        	if (keyIndex > 0) break;
        }
            	      
            if (keyIndex == 0xFF) ISOException.throwIt(ISO7816.SW_DATA_INVALID);
    
            // 3c - extract DivData (Not necessary)
    
            // 3d - Generate RND2, overwriting RND1[B]
            cspRNG.generateData(Buffer160, OFFSET_RND2, LENGTH_AES_BLOCK);
    
            // 3e - Generate RND3 = SHA1(RND1 | RND2) and place after RND2
            cspSHA1.doFinal(Buffer160, OFFSET_RND1A,
            	(short)(LENGTH_AES_BLOCK * 2), Buffer160, OFFSET_RND3);
    
            // Copy the first 16 bytes of RND3 to Buffer16 so we can use it
            // later in FinalAuthentica	te
            Util.arrayCopyNonAtomic(Buffer160, OFFSET_RND3, Buffer16, (short)0, LENGTH_AES_KEY);
            debug++;	
    
            // 3f - Generate FAKey(Div) (zero out first 8 bytes of RND1[A])
            Util.arrayFillNonAtomic(Buffer160, OFFSET_RND1A, LENGTH_DIVDATA, (byte)0);
            cspAES.init(keyTable[keyIndex].faKey, Cipher.MODE_ENCRYPT);
            cspAES.doFinal(Buffer160, OFFSET_DIVDATA, LENGTH_AES_BLOCK, Buffer160, OFFSET_FADIV);
            transientKey.setKey(Buffer160, OFFSET_FADIV);
            debug++;	
    
            // 3g - Generate STR2
    
            /*
            	* STR2 format: OpModeId (2b) | RND2 (16B) | RND3 (16B)
            	*/
    
            // Copy OpModeId
            Util.setShort(Buffer160, OFFSET_STR2, opModeId);
    
            // Zero out the remaining padding bytes after the first 16 bytes of
            // RND3
            Util.arrayFillNonAtomic(Buffer160,
            	(short)(OFFSET_RND3 + LENGTH_SHA1_BLOCK),
            	LENGTH_ESTR2_PADDING, (byte)0);
    
            // We should now have the STR2 format at position OFFSET_STR2.
    
            // 3h - Generate ESTR2 = AES_ENCRYPT(FADivKey, STR2)
            cspAES.init(transientKey, Cipher.MODE_ENCRYPT);
            cspAES.doFinal(Buffer160, OFFSET_STR2, LENGTH_ESTR2, apduBuffer, (short)0);
    
            // 3i - Transmit ESTR2
            apdu.setOutgoingAndSend((short)0, LENGTH_ESTR2);
        }       
    }
    catch (Exception e)
    {
	ISOException.throwIt(debug);
    }
  }

  public void finalAuthenticate(APDU apdu)
  {
    byte[] apduBuffer = apdu.getBuffer();
    short apduLen = apdu.setIncomingAndReceive();

    /*
     * COMMAND APDU CDATA: 
     * ESTR3 (128B); 
     */

    /*
     * 7) The IFD processes the credential
     */

    // First, set our transient key to the value of RND3 we copied during IA
    transientKey.setKey(Buffer16, (short)0);

    // Now that we have used RND3, copy DivData out so we can compare later
    Util.arrayCopyNonAtomic(Buffer160, OFFSET_DIVDATA, Buffer16, (short)0,
      LENGTH_DIVDATA);

    // 7a Calculate STR3 = AES_DECRYPT(RND3, ESTR3)
    cspAES.init(transientKey, Cipher.MODE_DECRYPT);
    cspAES.doFinal(apduBuffer, ISO7816.OFFSET_CDATA, apduLen, Buffer160,
      (short)0);

    /*
     * STR3 FORMAT: DIVDATA(8B) | ACSRECORD (1-160B)
     */

    // 7b Compare the returned DIVDATA with our own copy in Buffer16
    if (Util.arrayCompare(Buffer160, (short)0, Buffer16, (short)0,
      LENGTH_DIVDATA) == 0)
    {
      // Copy the ACSRecord and send
      short length = Tlv.getLength(Buffer160, LENGTH_DIVDATA);

      // Add the SEQUENCE header
      length += (short)(Tlv.getDataOffset(Buffer160, LENGTH_DIVDATA) - LENGTH_DIVDATA);        

      Util.arrayCopyNonAtomic(Buffer160, LENGTH_DIVDATA, apduBuffer, (short)0, length);

      apdu.setOutgoingAndSend((short)0, length);
    }
    else
      ISOException.throwIt(ISO7816.SW_DATA_INVALID);
  }

  public void setData(APDU apdu)
  {
    byte[] apduBuffer = apdu.getBuffer();
    // short apduLen = apdu.setIncomingAndReceive();

    /*
     * COMMAND APDU CDATA: 
     * SEQUENCE; 
     * 	STRING_OCTET (1B); 		Tag 
     * 	STRING_OCTET (2B); 		Keyset Id 
     * 	STRING_OCTET (Variable); 	Data
     */

    /*
     * SETUP
     */

    short offset = ISO7816.OFFSET_CDATA;

    // Is this our sequence?
    if (Tlv.getTagNumber(apduBuffer, offset) != Tlv.ASN1_SEQUENCE)
    {
      ISOException.throwIt(ISO7816.SW_DATA_INVALID);
    }

    // Get the offset for our TAG value
    offset = Tlv.findNext(apduBuffer, offset, Tlv.ASN1_STRING_OCTET);
    byte tag = Tlv.toByte(apduBuffer, offset);

    // Check for the special 'SECURED' tag
    if (tag == TAG_SECURED) {
	if ((controlFlags & FLAG_SECURED) == 0x00) controlFlags |= FLAG_SECURED;
	return;
    }
    
    // All SET DATA commands pass the applicable KEYSET id through
    offset = Tlv.findNext(apduBuffer, offset, Tlv.ASN1_STRING_OCTET);
    short keyset = Tlv.toShort(apduBuffer, offset);

    // Find an existing or vacant slot for the keysets
    short index = 0xFF;
    for (byte i = 0; i < TOTAL_KEYSETS; i++)
    {
      if ((keyTable[i].id == keyset) || keyTable[i].isEmpty)
      {
        keyTable[i].id = keyset;
        if (keyTable[i].isEmpty) keyTable[i].isEmpty = false;

        index = i;
        break;
      }
    }

    if (index == 0xFF)
    {
      ISOException.throwIt(ISO7816.SW_FILE_FULL);
    }

    // Get the offset of the data
    offset = Tlv.findNext(apduBuffer, offset, Tlv.ASN1_STRING_OCTET);
    offset = Tlv.getDataOffset(apduBuffer, offset);

    switch (tag)
    {
      case TAG_P:
        keyTable[index].iaKey.setP(apduBuffer, offset, LENGTH_P);
        return;

      case TAG_Q:
        keyTable[index].iaKey.setQ(apduBuffer, offset, LENGTH_Q);
        return;

      case TAG_PQ:
        keyTable[index].iaKey.setPQ(apduBuffer, offset, LENGTH_PQ);
        return;

      case TAG_DP1:
        keyTable[index].iaKey.setDP1(apduBuffer, offset, LENGTH_DP1);
        return;

      case TAG_DQ1:
        keyTable[index].iaKey.setDQ1(apduBuffer, offset, LENGTH_DQ1);
        return;

      case TAG_FAKEY:
        keyTable[index].faKey.setKey(apduBuffer, offset);
        return;

      default:
        ISOException.throwIt(ISO7816.SW_FILE_INVALID);
        break;
    }
  }
}